/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 24.05.2005
 *
 */
package de.tudarmstadt.events;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author markus
 *
 */
public class ActionEventGenerator {
	private HashSet<ActionListener> listeners = new HashSet<ActionListener>();
	
	public synchronized void registerListener(ActionListener listener) {
		listeners.add(listener);
	}
	
	public synchronized void unregisterListener(ActionListener listener) {
		listeners.remove(listener);
	}
	
	/** Return a copy of the listeners set so we can iterate over it.
	 * @return A copy of the set listeners.
	 */
	@SuppressWarnings("unchecked")
	private synchronized HashSet getListenersCopy() { 
		return new HashSet(listeners);
	}

	/**
	 * @param event
	 */
	public void emitActionPerformed(ActionEvent event) {
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			ActionListener lis = (ActionListener) i.next();
			lis.actionPerformed(event);
		}
	}
}
