
package de.tudarmstadt.lang;

import java.util.Locale;
import java.util.ResourceBundle;




/**
 * @author O, Hung Lun 
 * @date 10.01.2007
 */
public class Lang 
{
	/** The singleton reference. */
	private static Lang self = null;

	private static LanguageTranslation currentLanguage;
	
	private static LanguageTranslation languageTranslationSet[]={
			new LanguageTranslation("english","englisch"),
			new LanguageTranslation("german","deutsch")
	};
	/** The actual resource bundle. */
	private ResourceBundle bundle = null;
	
	/**
	 * Protected constructor for singelton purpose.
	 */
	protected Lang() {
		bundle = null;
	}
	
	/**
	 * Returns the singleton instance of Lang.
	 * @return The singleton instance.
	 */
	public static Lang instance() {
		if (self == null) {
			self = new Lang();
		}
		return self;
	}

	/**
	 * Returns the string translated in the current language.
	 * @param string The string to translate.
	 * @return The translated string.
	 */
	public static String getString(String string) {
		return instance().getResourceBundle().getString(string);
	}
	
	/**
	 * Sets the language.
	 * @param String The language to set.
	 */
	public static void setLanguage(String language){
		Locale locale = new Locale("en");
		setCurrentLanguage(language);
		if (language.equalsIgnoreCase("english")||language.equalsIgnoreCase("englisch"))
		{
			locale = new Locale("en");
		}
		else if (language.equalsIgnoreCase("deutsch")||language.equalsIgnoreCase("german"))
		{
			locale = new Locale("de");	
		}
		
		Lang.instance().setResourceBundle(ResourceBundle.getBundle("de.tudarmstadt.lang.MessageBundle", locale));

	}
	
	
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	private static void setCurrentLanguage(String language) {
		for (int i=0; i<languageTranslationSet.length;i++)
		{
			if (languageTranslationSet[i].equals(language))
				currentLanguage=languageTranslationSet[i];
		}
	}

	/**
	 * Sets the default language for UI.
	 *  @param bundle The bundle to set.
	 */
	public static void setDefaultLanguage(String language)
	{
		//Lang.instance().setResourceBundle(ResourceBundle.getBundle("de.tudarmstadt.lang.MessageBundle", new Locale("en")));
		setLanguage(language);
		setCurrentLanguage(language);
		
	}

	/**
	 * Returns the actual ResourceBundle.
	 * @return The actual ResourceBundle.
	 */
	public ResourceBundle getResourceBundle() {
		if (bundle == null) {
			bundle = ResourceBundle.getBundle(
					"de.tudarmstadt.lang.MessageBundle", new Locale("de"));
		}
		return bundle;
	}
	
	/**
	 * Sets the actual ResourceBundle.
	 * @param bundle The bundle to set.
	 */
	public void setResourceBundle(ResourceBundle bundle) {
		this.bundle = bundle;
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public static boolean isCurrentLanguage(String language){
		return currentLanguage.equals(language);
	}
}