/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.ObservableMap;
import de.tudarmstadt.yajs.gui.contestant.ContestantUIModel;
import de.tudarmstadt.yajs.gui.contestant.YAJSContestantUI;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEvent;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEvent;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.FileSelectionEvent;
import de.tudarmstadt.yajs.gui.contestant.events.FileSelectionEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEvent;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEvent;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.RefreshResultsEvent;
import de.tudarmstadt.yajs.gui.contestant.events.RefreshResultsEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEvent;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.TestEvent;
import de.tudarmstadt.yajs.gui.contestant.events.TestEventListener;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventListener;
import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.Console;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.LanguageSet;
import de.tudarmstadt.yajs.lib.Login;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.ProblemSet;
import de.tudarmstadt.yajs.lib.Result;
import de.tudarmstadt.yajs.lib.Status;
import de.tudarmstadt.yajs.lib.Submission;
import de.tudarmstadt.yajs.lib.contestant.Contestant;
import de.tudarmstadt.yajs.lib.events.AutoConfigEvent;
import de.tudarmstadt.yajs.lib.events.AutoConfigEventListener;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEvent;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEventListener;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEvent;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEventListener;
import de.tudarmstadt.yajs.lib.events.LoginEvent;
import de.tudarmstadt.yajs.lib.events.LoginEventListener;
import de.tudarmstadt.yajs.lib.events.ResultEvent;
import de.tudarmstadt.yajs.lib.events.ResultEventListener;
import de.tudarmstadt.yajs.lib.events.ScoreboardEvent;
import de.tudarmstadt.yajs.lib.events.ScoreboardEventListener;
import de.tudarmstadt.yajs.lib.events.StatusEvent;
import de.tudarmstadt.yajs.lib.events.StatusEventListener;
import de.tudarmstadt.yajs.tester.Diff;
import de.tudarmstadt.yajs.tester.TestCompletedEvent;
import de.tudarmstadt.yajs.tester.TestCompletedEventListener;
import de.tudarmstadt.yajs.tester.Tester;
/**
 * The contestant application, providing functionality needed for the contestants.
 */
class YAJSContestantApp extends YAJSClientApp
{
	/** The gui associated with this app. */
	private YAJSContestantUI gui;
	/** An observable map containing the obtained results. */
	private ObservableMap results = new ObservableMap();
	/** An observable map containing the obtained clarifications. */
	private ObservableMap clarifications = new ObservableMap();
	/** The mapping from extensions to languages. */
	private Map extensionMap;
	/**check if the tlimit file is present*/
	private boolean tlimitAvailable=true;
	private boolean doTesting=true;
	String userDefinedInput=""; //$NON-NLS-1$
	/** Constructor taking a ContestInfo object. */	
	public YAJSContestantApp(String clientConfigFilename) {
		super(clientConfigFilename);
		this.extensionMap = createExtensionMap(contestinfo.getLanguages());
		buildClient();
		buildGUI();
		wireGUI();
		wireClient();
		setGUITitle();
		
		try {
			if (isOnline())
			fetchAllRemoteTestData();
				else fetchAllOfflineTestData();
		} catch (final IOException e_) {
			final IOException e = e_;	// circumvent a gcj bug
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					showErrorMessage(Lang.getString("IOErrorwhilefetchingsampledata") + e.getMessage() + Lang.getString("Pleasecontactstaff")); //$NON-NLS-1$ //$NON-NLS-2$
				}
			});
		}
	}
	
	
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	private void fetchAllOfflineTestData() throws IOException{
		Iterator iter = getContestInfo().getProblems().iterator();
		while(iter.hasNext())
		{
			Problem problem = (Problem) iter.next();
			if(!checkTestDirectoryExists(problem))
				fetchOfflineTestDataOnStartup(problem,null);
		}
	}


	
	


	/** Creates the mapping from extensions to languages.
	 * The language id is considered to be an extension itself.
	 * @param languages The set of existing languages.
	 * @param aliases A map from extensions to language ids.
	 */
	
	protected Map createExtensionMap(LanguageSet languages) {
		Map m = new HashMap();
		{	
			Iterator i = languages.iterator();
			while(i.hasNext()) {
				Language l = (Language) i.next();
				m.put(l.getID(), l);
			}
		}	return m;
	}
	
	/** Initializing method that constructs the GUI. */
	protected void buildGUI()
	{
		try {
			SwingUtilities.invokeAndWait(new Runnable()	{
				public void run() {
					gui = new YAJSContestantUI(new ContestantUIModel() {

						public ObservableMap getResults() {
							return results;
						}

						public ProblemSet getProblems() {
							return YAJSContestantApp.this.getProblemSet();
						}

						public LanguageSet getLanguages() {
							return YAJSContestantApp.this.getLanguageSet();
						}

						public ObservableMap getClarifications() {
							return clarifications; 
						}
						
					}, isOnline());
					gui.setLocationRelativeTo(null);
				}			
			});
				
		} catch (InterruptedException e2) {
			e2.printStackTrace();
			close();
			System.exit(-1);
		} catch (InvocationTargetException e2) {
			e2.printStackTrace();
			close();
			System.exit(-1);
		}
	}
	
	/** Initializing method that constructs the Client. */
	protected void buildClient()
	{
		try {
			if (contestinfo.getMode().equalsIgnoreCase("online"))
				client = new Contestant(contestinfo, getLoginGUI("contestant"), new Console());	
			else{
				client = new Contestant(contestinfo, new Console());	
				super.setLogin(new Login("","",false));
			}//$NON-NLS-1$
		} catch (IOException e1) {
			cancelLoginTimeout();
			close();
			//e1.printStackTrace();
			System.err.println(Lang.getString("Unabletoconnecttoserver")); //$NON-NLS-1$
			System.exit(-1);
		}
	}
	
	/** Initializing method that registers listeners for GUI events. */
	protected void wireGUI() {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					gui.addWindowListener(new WindowAdapter(){
						/* (non-Javadoc)
						 * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
						 */
						public void windowClosing(WindowEvent arg0) {
							super.windowClosing(arg0);
							client.closeSocket();
							OfflineHelper.setClientFrameIsClosed();
						}
					});
					
					gui.addSubmitListener( new SubmitEventListener(){
						public void submitProblem(SubmitEvent e) {
							
							Language language = e.getLanguage();
							String source;
							try {
								source = readFileContents(e.getFile());
							} catch (IOException e1) {
								e1.printStackTrace();
								System.err.println(Lang.getString("Yoursubmissionwascancelled")); //$NON-NLS-1$
								return;
							}
							getContestant().submit(new Submission(e.getProblem(), language, source));
						}
					});
			
					gui.addRefetchListener(new RefetchEventListener() {
						public void refetch(final RefetchEvent event) {
							new Thread(new Runnable() {
								public void run() {
									
									try {
										SwingUtilities.invokeAndWait(new Runnable() {
											public void run() {
												gui.enableFetching(event.getProblem(), false);	
											
											}
										});
										if (isOnline()){
											
											fetchRemoteTestData(event.getProblem());
										
										}else {
											fetchOfflineTestData(event.getProblem(),event.getLanguage());
//											JOptionPane.showMessageDialog(null, Lang.getString("YAJSContestantApp.6"));
											} //$NON-NLS-1$
									} catch (Exception e) { 
										showErrorMessage(Lang.getString("Unabletofetchtestdataforproblem") + event.getProblem().getID() + ":\n" + e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
									} finally {
										try {
											SwingUtilities.invokeAndWait(new Runnable() {
												public void run() {
													gui.enableFetching(event.getProblem(), true);	
												}
											});
										} catch (Exception e1) {
											e1.printStackTrace();
										}
									}
								}
							}).start();
						}
					});
					gui.addRefreshResultsListener( new RefreshResultsEventListener(){
						public void refreshResults(RefreshResultsEvent e) {
							getContestant().queryResults();
						}
					});
					
					gui.addRefreshScoreboardListener( new RefreshScoreboardEventListener() {
						public void refreshScoreboard(RefreshScoreboardEvent e) {
							getContestant().queryScoreboard();
						}
					});

					gui.addAskClarificationEventListener(new AskClarificationEventListener() {
						public void askClarification(AskClarificationEvent e) {
							showNewClarificationDialog();
						}
					});

					gui.addRefreshClarificationsEventListener(new RefreshClarificationsEventListener() {
						public void refreshClarifications(RefreshClarificationsEvent e) {
							getContestant().queryClarifications();
						}
					});
				
					gui.addEditorEventListener(new EditorEventListener() {
						public void edit(EditorEvent event)  {
							Language language=event.getLanguage();
							if(language== null) language =askForLanguage();
//							if user cancel the language selection, then return
							if(language== null) return;
							
							//check if editor exists. If yes continue, else show dialog and return
							String EDITORCOMMAND=language.getEditor();
//                            if (new File(System.getProperty("user.dir")+File.separator+language.getEditor()).exists())
//                                    EDITORCOMMAND=System.getProperty("user.dir")+File.separator+language.getEditor();
//                            if (new File(System.getProperty("user.dir")+File.separator+language.getEditor()+".exe").exists())
//                            		EDITORCOMMAND=System.getProperty("user.dir")+File.separator+language.getEditor()+".exe";
////$NON-NLS-1$
//                            if (new File(language.getEditor()).exists())
//                                    EDITORCOMMAND= language.getEditor(); //$NON-NLS-1$
//                            if (new File(language.getEditor()+".exe").exists())
//                                EDITORCOMMAND= language.getEditor()+".exe"; //$NON-NLS-1$
//                            if(EDITORCOMMAND==" "){
//                                    JOptionPane.showMessageDialog(gui,Lang.getString("NoEditorAvailable")+System.getProperty("user.dir"));
//                                    return;
//                            }
							//check if file already chosen
							String PROBLEMCOMMAND ;
							if (event.getFile()!=null){
								PROBLEMCOMMAND=event.getFile().getPath();
								//if yes open the file
								try {
									Runtime.getRuntime().exec(new String[] { EDITORCOMMAND, PROBLEMCOMMAND});
									return;
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}	
							}
								//if no, ask for language
							if (event.getFile()==null){
								//if language is selected, check if solution file exists
								Problem problem=event.getProblem();
								String solutionFilename=System.getProperty("user.dir")+File.separator+problem.getName()+
									File.separator+	problem.getID()+"."+language.getID(); //$NON-NLS-1$ //$NON-NLS-2$
								PROBLEMCOMMAND=solutionFilename;
								if(new File(solutionFilename).exists()){
									try {
										Runtime.getRuntime().exec(new String[] { EDITORCOMMAND, PROBLEMCOMMAND});
									} catch (IOException e) {
										// TODO Auto-generated catch block
										e.printStackTrace();
									}	
									gui.setFileForEditor(problem, new File(PROBLEMCOMMAND));
									gui.setLanguageForProblem(problem, language);									
									return;
								}
								else{
									
									try {
										if(isOnline()){
											if (fetchRemoteTemplate(problem,language)){
												Runtime.getRuntime().exec(new String[] { EDITORCOMMAND, PROBLEMCOMMAND});	
												gui.setFileForEditor(problem, new File(PROBLEMCOMMAND));
												gui.setLanguageForProblem(problem, language);																						
												return;
											}
											else return;
										}
										if(!isOnline()){
											if (fetchOfflineTemplate(problem,language)){
												Runtime.getRuntime().exec(new String[] { EDITORCOMMAND, PROBLEMCOMMAND});	
												gui.setFileForEditor(problem, new File(PROBLEMCOMMAND));
												gui.setLanguageForProblem(problem, language);																						
												return;
											}
											else return;
										}
										
										
									} catch (IOException e) {
										// TODO Auto-generated catch block
										e.printStackTrace();
									}
								}
				
							}
						
						}




							});
					gui.addManualTestEventListener(new ManualTestEventListener() {
						public void test(ManualTestEvent event){
							final Problem problem = event.getProblem();
							final Language language = event.getLanguage();
							final File file=event.getFile();
							try {
							    BufferedWriter out = new BufferedWriter(new FileWriter(getTestInputFile(problem).getParent()+File.separator+"min")); //$NON-NLS-1$
							    
							    showManualInputDialog(gui,Lang.getString("YAJSContestantApp.13")); //$NON-NLS-1$
								
							    if(doTesting)
							    {	out.write(userDefinedInput);
							    	out.close();
							    }
							} catch (IOException e) {
							} 
							
							try {
								if(!checkTestDirectoryExists(problem)&&isOnline()) {
									fetchRemoteTestData(problem);
								}
							} catch (IOException e) {
								JOptionPane.showMessageDialog(gui, Lang.getString("Couldnotfetchtestdata.AnIOErroroccured") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
								return;
							}							
							
							double tl = getTimeLimit(problem);
							
							if(doTesting)
							try {
								final String code = readFileContents(event.getFile());
								
//								replace the sin with min in the relevant problem directory
								final File inputFile = new File(getTestInputFile(problem).getParent()+File.separator+"min"); //$NON-NLS-1$
							//	final File expFile = File.createTempFile("empty", "out"); //$NON-NLS-1$ //$NON-NLS-2$
								final double timeLimit = tl;
								final Tester tester = new Tester(problem, language, timeLimit, code, inputFile, contestinfo);
								
								tester.addTestCompletedEventListener(new TestCompletedEventListener() {

									public void testCompleted(final TestCompletedEvent re) {
										
										Diff diff = null;
										String mOutput=null;
										try {
											File f = re.getOutputFile();
											if (Dcc32Utility.isDCC32(language)) f=Dcc32Utility.compilerInfoFilter(f);
											BufferedReader outReader = new BufferedReader(new FileReader(f));
											
//											BufferedReader expReader = new BufferedReader(new FileReader(expFile));
											
											diff = Diff.createFromReaders(outReader);
											mOutput=diff.getOutput();
											 
										} catch (final IOException e_) {
											final IOException e = e_;	// circumvent a gcj bug
											SwingUtilities.invokeLater(new Runnable() {
												public void run() {
													gui.showErrorMessage(Lang.getString("CouldnottestIOErroroccured") + e.getMessage()); //$NON-NLS-1$
												}
											});
											return;
										} finally {
											tester.deleteAllTemporaries();
										}
										
										final String themOutput = mOutput;
										
										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												gui.displayManualTestResult(problem,file,language,themOutput, re.getError(), re.getSuggestion());
											}
										});
									}

									

									public void testInterrupted(final String error) {
										tester.deleteAllTemporaries();
										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												gui.showErrorMessage(Lang.getString("CouldnottestTestinterruptedunexpectedly") + error); //$NON-NLS-1$
											}
										});
									}
									
								});
								tester.start();
							} catch (IOException e) {
								JOptionPane.showMessageDialog(gui, Lang.getString("CouldnottestIOErroroccured") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
							}	
						
						}

						private void showManualInputDialog(YAJSContestantUI gui, String string) {
								
							
							final JDialog dialog=new JDialog(gui, string, true);
//						dialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
						    
							JDialog.setDefaultLookAndFeelDecorated(true);
							final JTextArea textArea=new JTextArea();
							JScrollPane scrollPane = new JScrollPane(textArea);
							
						
							JButton ok=new JButton(Lang.getString("YAJSContestantApp.26")); //$NON-NLS-1$
							
							ok.addActionListener(new ActionListener(){
								public void actionPerformed(ActionEvent e) {
									userDefinedInput = textArea.getText();
									dialog.dispose();
									textArea.setText(""); //$NON-NLS-1$
									doTesting=true;
								}
							});
							dialog.addWindowListener(new WindowAdapter(){
								public void windowClosing(WindowEvent we) {
									dialog.dispose();
					                textArea.setText(""); //$NON-NLS-1$
					                doTesting=false;
					            }
							});
							JButton cancel=new JButton(Lang.getString("YAJSContestantApp.4")); //$NON-NLS-1$
							cancel.addActionListener(new ActionListener(){
								public void actionPerformed(ActionEvent e) {
									dialog.dispose();
									textArea.setText(""); //$NON-NLS-1$
					                doTesting=false;
								}
							});
							JPanel panel=new JPanel();
							panel.setLayout(new BorderLayout());
							scrollPane.setPreferredSize(new Dimension(260, 280));
							panel.add(scrollPane,BorderLayout.PAGE_START);
							JPanel buttons =new JPanel();
							buttons.setLayout(new GridLayout(1,3));
							buttons.add(ok);
							buttons.add(new JLabel("")); //$NON-NLS-1$
							buttons.add(cancel);
							panel.add(buttons,BorderLayout.PAGE_END);
							

							dialog.setContentPane(panel);
							dialog.pack();
							dialog.setLocationRelativeTo(gui);
							dialog.show();
						}
						});
			gui.addCreateTemplateSrcEventListener(new CreateTemplateSrcEventListener(){
			public void test(CreateTemplateSrcEvent event){
				final Problem problem = event.getProblem();
				Language language=null;
				language=askForLanguage();
				try {
					
					if (language!=null){			
						if (isOnline()){
							if(fetchRemoteTemplate(problem,language)){
								String filename=System.getProperty("user.dir")+File.separator+problem.getName() //$NON-NLS-1$
								+File.separator+problem.getID()+"."+language.getID();
							File file=new File(filename);
							gui.setFileForEditor(event.getProblem(), file);
							gui.setLanguageForProblem(event.getProblem(), language);
						
							}
						}
						else{
								if(fetchOfflineTemplate(problem,language)){
									String filename=System.getProperty("user.dir")+File.separator+problem.getName() //$NON-NLS-1$
										+File.separator+problem.getID()+"."+language.getID();
									File file=new File(filename);
									gui.setFileForEditor(event.getProblem(), file);
									gui.setLanguageForProblem(event.getProblem(), language);
								}
						}
					}
				} catch (IOException e2) {
					// TODO Auto-generated catch block
					e2.printStackTrace();
				}
			}
			
			});
					gui.addTestEventListener(new TestEventListener() {

						public void test(TestEvent event) {
							final Problem problem = event.getProblem();
							final Language language = event.getLanguage();
							final File file=event.getFile();
							try {
								if(!checkTestDirectoryExists(problem)&&isOnline()) {
									fetchRemoteTestData(problem);
								}
							} catch (IOException e) {
								JOptionPane.showMessageDialog(gui, Lang.getString("Couldnotfetchtestdata.AnIOErroroccured") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
								return;
							}							
							
							double	tl = getTimeLimit(problem);								
							
						
							try {
								final String code = readFileContents(event.getFile());
								final File inputFile = getTestInputFile(problem);
								final double timeLimit = tl;
								final Tester tester = new Tester(problem, language, timeLimit, code, inputFile, contestinfo);
								
								tester.addTestCompletedEventListener(new TestCompletedEventListener() {

									public void testCompleted(final TestCompletedEvent re) {
										
										Diff diff = null;
										try {
											File f = re.getOutputFile();
											if (Dcc32Utility.isDCC32(language)) f=Dcc32Utility.compilerInfoFilter(f);
											BufferedReader outReader = new BufferedReader(new FileReader(f));
											BufferedReader expReader = new BufferedReader(new FileReader(getTestOutputFile(problem)));
											diff = Diff.createFromReaders(outReader, expReader);
											 
										} catch (final IOException e_) {
											final IOException e = e_;	// circumvent a gcj bug
											SwingUtilities.invokeLater(new Runnable() {
												public void run() {
													gui.showErrorMessage(Lang.getString("CouldnottestIOErroroccured") + e.getMessage()); //$NON-NLS-1$
												}
											});
											return;
										} finally {
											tester.deleteAllTemporaries();
										}
										
										final Diff theDiff = diff;
										
										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												gui.displayTestResult(problem, file, language,theDiff, re.getError(), re.getSuggestion());
											}
										});
									}

									public void testInterrupted(final String error) {
										tester.deleteAllTemporaries();
										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												gui.showErrorMessage(Lang.getString("CouldnottestTestinterruptedunexpectedly") + error); //$NON-NLS-1$
											}
										});
									}
									
								});
								tester.start();
							} catch (IOException e) {
								JOptionPane.showMessageDialog(gui, Lang.getString("CouldnottestIOErroroccured") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
							}				
							
						}	});
					
			
					gui.addFileSelectionEventListener(new FileSelectionEventListener() {
						
						public void fileSelected(FileSelectionEvent event) throws PropertyVetoException {
							Language language = determineLanguageForProblem(event.getSelectedFile(), event.getProblem());
							if(language == null)
								throw new PropertyVetoException(null, null);
							else{
								gui.setLanguageForProblem(event.getProblem(), language);
							//set Label name
								gui.setFileForEditor(event.getProblem(), event.getSelectedFile());
							}
						}
					});
				}
				});
	
			// TODO here? elsewhere? where?
			getContestInfo().addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent arg0) {
					try { 
						fetchAllRemoteTestData();
					} catch (final IOException e_) {
						final IOException e = e_;	// circumvent a gcj bug
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								showErrorMessage(Lang.getString("IOErrorwhilefetchingsampledata") + e.getMessage() + Lang.getString("Pleasecontactstaff")); //$NON-NLS-1$ //$NON-NLS-2$
							}
						});
					}
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							gui.updateProblemPanels();	
						}
					});
				}
			});	
			
		} catch (InterruptedException e) {
			e.printStackTrace();
			close();
			System.exit(-1);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			close();
			System.exit(-1);
		}
				
	}
	
	/** Get the first <em>accepted</em> result or, 
	 * if this does not exist, the most recent one. 
	 */
	protected Result getBestResult(Problem problem) {
		Result best = null;
		synchronized(results) {
			Iterator iter = results.values().iterator();
			while(iter.hasNext())
			{
				Result tmp = (Result) iter.next();
				if(!tmp.getProblem().equals(problem)) continue;
				best = tmp;
				if(best.getStatus() == Status.ACCEPTED) return best;
			}
		}
		return best;
	}

	/** Initializing method that registers listeners for Client events. */	
	protected void wireClient() {
//		System.err.println(123);
		getContestant().addResultEventListener( new ResultEventListener(){
			public void resultReceived(final ResultEvent e)
			{
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						results.put(new Long(e.getResult().getIndex()), e.getResult());
						gui.updateResultText(getBestResult(e.getResult().getProblem()));
						if(e.isNotification()) gui.showUpdatedResult(e.getResult());
					}
				});
			}
		});
		
		getContestant().addLoginEventListener( new LoginEventListener(){
			public void loginCompleted(LoginEvent e) {
				
				cancelLoginTimeout();
				
				if(e.successful())
				{
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							gui.show();
						}
					});
					if (isOnline())startHeartbeatSignal();
				}
				    else{
					  System.err.println(Lang.getString("Loginfailed")); //$NON-NLS-1$
					  close();
					  JOptionPane.showMessageDialog(null, Lang.getString("TheserverhasnotacceptedtheloginattemptMaybeyouhavesuppliedawrongpasswordorawrongusername"), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE);					 //$NON-NLS-1$ //$NON-NLS-2$
				    }
				
			}
		});

		getContestant().addAutoConfigEventListener( new AutoConfigEventListener() {
			public void autoConfig(AutoConfigEvent event) {
				getContestInfo().setLanguages(event.getLanguages());
				getContestInfo().setProblems(event.getProblems());
				// Keep old aliases...
				extensionMap = createExtensionMap(event.getLanguages());
			}
		});
	
		getContestant().addConnectionErrorEventListener(new ConnectionErrorEventListener() {
			public void connectionError(ConnectionErrorEvent e) {
				showErrorMessage(e.getError());
			}
		});
		
		getContestant().addScoreboardEventListener(new ScoreboardEventListener() {
			public void scoreboardReceived(final ScoreboardEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						gui.updateScoreboard(e.getScoreboard());
					}
				});
			}
		});
		
		getContestant().addClientClarificationAnswerEventListener(new ClientClarificationAnswerEventListener() {
			public void clientClarificationAnswer(ClientClarificationAnswerEvent event) {
				
				clarifications.put(new Long(event.getAnswer().getRequestIndex()), event.getAnswer());
				
				// If unnotified, do not pop up windows...
				// The same thing, if the clarification is _unanswered_!
				if(		(event.getAnswer().getState() == ClarificationAnswerState.UNANSWERED)
					||	(!event.isNotified()))
					return;
				
				// otherwise...
				showClarificationAnswer(event.getAnswer());
			}
		});

		getContestant().addStatusEventListener(new StatusEventListener() {

			public void statusChanged(final StatusEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						gui.updateContestStatus(e.getContestStatus());							
					}
				});
			}
			
		});
	}

	/** Sets the title for the GUI.
	 */
	private void setGUITitle() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.setTitle(getProgramName() + " - " + getLoginGUI("contestant").getName()); //$NON-NLS-1$ //$NON-NLS-2$
			}
		});
	}
	/** Get the programming language in which a given file is written. 
	 * If the language cannot be inferred, this method pops up a dialog 
	 * to ask the user.
	 * @param file The file to inspect. 
	 * @param problem The problem (<b>REFACTOR</b> is not needed any more)
	 * @return Returns a language that has been found to be suitable.
	 */
	protected Language determineLanguageForProblem(File file, Problem problem) {
		String filename = file.getName();
		String suffix = filename.substring(filename.lastIndexOf('.')+1, filename.length());
		Language language = (Language) extensionMap.get(suffix);
		if(language == null) {
			language = askForLanguage();
		}
		return language;
	}
	
	/** Show an error message. 
	 * @param message The message to show.
	 */
	protected void showErrorMessage(String message) {
		gui.showErrorMessage(message);
	}
	
	/** Convenience wrapper around getClient().
	 * Performs the cast to Contestant.
	 * @return Returns the client as a Contestant object.
	 */
	protected Contestant getContestant() 
	{
		return (Contestant) getClient(); 
	}
	
	/** Get the test input for a problem. 
	 * @param problem The problem to get the input for.
	 * @return Returns the File containing the test input.
	 * @throws IOException
	 */
	protected File getTestInputFile(Problem problem) throws IOException {
		//return readFileContents(getContestInfo().getLocalTestInputLocation(problem));
		return getContestInfo().getLocalTestInputLocation(problem);
	}
 
	/** Get the test output for a problem.
	 * @param problem The problem to get the output for.
	 * @return Returns the File containing the test output.
	 * @throws IOException
	 */
	protected File getTestOutputFile(Problem problem) throws IOException {
		//return readFileContents(getContestInfo().getLocalTestOutputLocation(problem));
		return getContestInfo().getLocalTestOutputLocation(problem);
	}
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#dispose()
	 */
	protected void dispose() {
		super.dispose();
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					gui.dispose();	
				}
			});
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#getGUI()
	 */
	protected JFrame getGUI() {
		return gui;
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#getProgramName()
	 */
	protected String getProgramName() {
		return Lang.getString("TurmericRefefreeContestant"); //$NON-NLS-1$
	}
	/** Private inner interface extending Runnable in order to be able to acquire a return value. */
	private interface MyRunnable extends Runnable {

		public int getReturnValue();
	}
	public void fetchOfflineTestDataOnStartup (Problem problem, Language language) throws IOException{
		String [] sin={"sin"};
		String[] sout={"sout"};
		fetchOfflineData(problem, language,sin,false);

		fetchOfflineData(problem, language,sout,false);
		//fetchOnlineData(problem, language,offlineData);

//		String inFilename = System.getProperty("user.dir")+File.separator+problem.getName()+".zip"; //$NON-NLS-1$ //$NON-NLS-2$
//		try {	
//			ZipFile a= new ZipFile(inFilename);	
//	        ZipInputStream in = new ZipInputStream(new FileInputStream(inFilename));
//	        String zipEntry;
//	        FileOutputStream outputFile;
//	      
//	        for(int i=0; i<a.size();i++){    	
//		    	zipEntry=in.getNextEntry().getName();
//		    	String problemDir=System.getProperty("user.dir")+File.separator+problem.getName(); //$NON-NLS-1$
//		    	String outputfilename=problemDir+File.separator+zipEntry;
//		    		if(!(new File(outputfilename)).exists()) {
//		    			if (!(new File(problemDir)).exists() ){
//				    		(new File(problemDir)).mkdirs();								    		
//						}	    	
//				    	if (!(new File(outputfilename)).exists() ){
//				    		outputFile=new FileOutputStream(outputfilename);	
//				    		overWriteFromZipFile(outputFile,in);
//				    	}
//		    		}
//		    }
//	        in.close();
//		    } catch (IOException e) {	
//		    	throw new IOException(Lang.getString("Couldnotfetchtestdatafrom") +inFilename+"\n" + e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
//		}		
	}
	//return true only when overwrite is successful.
	public boolean fetchRemoteTemplate (Problem problem, Language language) throws IOException{
		String [] data={problem.getID()+"."+language.getID()};
		return fetchRemoteTestData(problem, language,data,true);
	}
	public boolean fetchOfflineTemplate (Problem problem, Language language) throws IOException{
		String [] offlineData={problem.getID()+"."+language.getID()};
		return fetchOfflineData(problem, language,offlineData,true);
//				String zipFilename = System.getProperty("user.dir")+File.separator+problem.getName()+".zip"; //$NON-NLS-1$ //$NON-NLS-2$
//		String templateFilename=System.getProperty("user.dir")+File.separator+problem.getName()+
//				File.separator+	problem.getID()+"."+language.getID(); //$NON-NLS-1$ //$NON-NLS-2$
//		String problemDir=System.getProperty("user.dir")+File.separator+problem.getName(); //$NON-NLS-1$
//		int confirmDialogResult=JOptionPane.NO_OPTION;
//		boolean templateExists=false;
//		
//		try {	
//			ZipFile a= new ZipFile(zipFilename);	
//	        ZipInputStream in = new ZipInputStream(new FileInputStream(zipFilename));
//	        String zipEntryName;
////	      check if the template exists in zip file.
//	        for(int i=0; i<a.size();i++){    	
//		    	zipEntryName=in.getNextEntry().getName();		     	
//		    	if (zipEntryName.equals(problem.getID()+"."+language.getID())){ //$NON-NLS-1$ //$NON-NLS-2$
//		    		templateExists= true;   
//		    		if (templateExists)break;
//		   	   	}
//		    }
//	    	//if no, throw a dialog showing template missing and return false.	
//	        if (!templateExists){
//	        	JOptionPane.showMessageDialog(gui, Lang.getString("notemplateavailable"));
//	        	return false;
//	        }	
//	   
//			if (templateExists){
//				FileOutputStream outputFile;
//	//			if yes, check if the file already exists in problem directory
//				if((new File(templateFilename)).exists()){
//	//				if file exists, ask for confirmation.				
//					 confirmDialogResult=JOptionPane.showConfirmDialog
//					  	(gui, Lang.getString("YAJSContestantApp.57")+" "+problem.getName()+File.separator+
//						problem.getID()+"."+language.getID(),"",JOptionPane.YES_NO_OPTION);
//	//			if confirmation return true, then copy and open.
//					 if (confirmDialogResult==JOptionPane.YES_OPTION){
//						 if (!(new File(problemDir)).exists() ){
//						 		(new File(problemDir)).mkdirs();								    		
//							}	
//						 outputFile=new FileOutputStream(templateFilename);
//						 overWriteFromZipFile(outputFile,in);
//						 return true;
//					 }
//						//if confirmation return false, then simply return false.
//					 else return false;
//				}
//	//			if file does not exist, copy and open the template silently.
//		  		else {
//					if (!(new File(problemDir)).exists() ){
//					 		(new File(problemDir)).mkdirs();								    		
//						}	
//					 	outputFile=new FileOutputStream(templateFilename);	
//					 	overWriteFromZipFile(outputFile,in);
//					 	return true;
//				}
//			}
//		in.close();
//		}
//		catch (IOException e) {	
//		    	throw new IOException(Lang.getString("Couldnotfetchtestdatafrom") +zipFilename+"\n" + e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
//		}	
//		return false;
//			
	}
	public void fetchOfflineTestData (Problem problem, Language language) throws IOException{
		String [] sin={"sin"};
		String[] sout={"sout"};
		boolean carryOn=fetchOfflineData(problem, language,sin,true);
		if(carryOn)fetchOfflineData(problem, language,sout,false);
		
//		String inFilename = System.getProperty("user.dir")+File.separator+problem.getName()+".zip"; //$NON-NLS-1$ //$NON-NLS-2$
//		String sin=System.getProperty("user.dir")+File.separator+problem.getName()+
//			File.separator+	"sin";
//		String sout=System.getProperty("user.dir")+File.separator+problem.getName()+
//			File.separator+	"sout";
//		int confirmResult=JOptionPane.YES_OPTION;//copy from zip by default.
//	    //confirm if the sin and sout exist in the problem directory 
//		if (new File(sin).exists()||new File(sout).exists())
//			confirmResult=JOptionPane.showConfirmDialog(gui, Lang.getString("YAJSContestantApp.57")+" "+problem.getName()+File.separator+"sin,sout","",JOptionPane.YES_NO_OPTION);
//        if (confirmResult==JOptionPane.NO_OPTION)return;	
//		
//        try {	
//			
//			ZipFile a= new ZipFile(inFilename);	
//	        ZipInputStream in = new ZipInputStream(new FileInputStream(inFilename));
//	        String zipEntry;
//	        FileOutputStream outputFile;
//	        //if yes , confirm overwriting, 
//	        //if no, overwrite silently.
//	      
//	 	    boolean sinExistsInZip=false;
//	 	    boolean soutExistsInZip=false;
//	        for(int i=0; i<a.size();i++){    	
//		    	zipEntry=in.getNextEntry().getName();		    	
//		    	String problemDir=System.getProperty("user.dir")+File.separator+problem.getName(); //$NON-NLS-1$
//		    	String outputfilename=problemDir+File.separator+zipEntry;
//		    	
//		    	if (zipEntry.equals("sin")){ //$NON-NLS-1$ //$NON-NLS-2$
//		    		if (!(new File(problemDir)).exists() ){
//				      (new File(problemDir)).mkdirs();								    		
//					}	
//				    outputFile=new FileOutputStream(outputfilename);	
//				    overWriteFromZipFile(outputFile,in);
//				    sinExistsInZip=true;
//		    	}
//		    	if (zipEntry.equals("sout")){ //$NON-NLS-1$ //$NON-NLS-2$
//		    		if (!(new File(problemDir)).exists() ){
//				      (new File(problemDir)).mkdirs();								    		
//					}	
//				    outputFile=new FileOutputStream(outputfilename);	
//				    overWriteFromZipFile(outputFile,in);
//				    soutExistsInZip=true;
//		    	}
//			   
//		    }
//	        if (!sinExistsInZip)JOptionPane.showMessageDialog(gui, Lang.getString("sinnotavailablefromZipfile"));
//
//	        if (!soutExistsInZip)JOptionPane.showMessageDialog(gui, Lang.getString("soutnotavailablefromZipfile"));
//	        
//	        in.close();
//		    } catch (IOException e) {	
//		    	throw new IOException(Lang.getString("Couldnotfetchtestdatafrom") +inFilename+"\n" + e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
//		}		
	}
	public boolean fetchOfflineData (Problem problem, Language language, String[] offlineData,boolean askOverwrite)throws IOException{
	String zipFilename = System.getProperty("user.dir")+File.separator+problem.getName()+".zip"; //$NON-NLS-1$ //$NON-NLS-2$
	String problemDir=System.getProperty("user.dir")+File.separator+problem.getName(); //$NON-NLS-1$
	
	boolean copyFromZipSuccessful=true;
	//	String templateFilename=System.getProperty("user.dir")+File.separator+problem.getName()+
//			File.separator+	problem.getID()+"."+language.getID(); //$NON-NLS-1$ //$NON-NLS-2$
	for (int j=0;j<offlineData.length;j++){
		int confirmDialogResult=JOptionPane.NO_OPTION;
		boolean zipEntryExists=false;
		String fileInProblemDir=problemDir+File.separator+offlineData[j];
		String missingFiles="";
		try {	
			ZipFile a= new ZipFile(zipFilename);	
	        ZipInputStream in = new ZipInputStream(new FileInputStream(zipFilename));
	        String zipEntryName;
	//      check if the template exists in zip file.
	        for(int i=0; i<a.size();i++){    	
		    	zipEntryName=in.getNextEntry().getName();		     	
		    	if (zipEntryName.equals(offlineData[j])){ //$NON-NLS-1$ //$NON-NLS-2$
		    		zipEntryExists= true;   
		    		if (zipEntryExists)break;
		   	   	}
		    }
	    	//if no, throw a dialog showing template missing and return false.	
	        if (!zipEntryExists){
	        	missingFiles=missingFiles+offlineData[j];
	        	copyFromZipSuccessful=false;
	        	
	        }	
	   
			if (zipEntryExists){
				FileOutputStream outputFile;
	//			if yes, check if the file already exists in problem directory
				if((new File(fileInProblemDir)).exists()){
	//				if file exists, ask for confirmation.	
					if(offlineData[j].equalsIgnoreCase("sin"))
						offlineData[j]="sin, sout";
					 if(askOverwrite){confirmDialogResult=JOptionPane.showConfirmDialog
					  	(gui, Lang.getString("YAJSContestantApp.57")+" "+problem.getName()+File.separator+offlineData[j],"",JOptionPane.YES_NO_OPTION);}
				
					//			if confirmation return true, then copy and open.
					 if (!askOverwrite||confirmDialogResult==JOptionPane.YES_OPTION){
						 if (!(new File(problemDir)).exists() ){
						 		(new File(problemDir)).mkdirs();								    		
							}	
						 outputFile=new FileOutputStream(fileInProblemDir);
						 overWriteFromZipFile(outputFile,in);
						 break;
					 }
						//if confirmation return false, then simply return false.
					 else 	copyFromZipSuccessful=false;
				        
				}
	//			if file does not exist, copy and open the template silently.
		  		else {
					if (!(new File(problemDir)).exists() ){
					 		(new File(problemDir)).mkdirs();								    		
						}	
					 	outputFile=new FileOutputStream(fileInProblemDir);	
					 	overWriteFromZipFile(outputFile,in);
					 	
				}
			}
		in.close();}
		catch (IOException e) {	
		    	showErrorMessage(Lang.getString("Couldnotfetchtestdatafrom") +zipFilename+"\n" + e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
		    	return false;
		}	
		if (missingFiles!="")showErrorMessage(missingFiles+" missing in zip file");
	}
	return copyFromZipSuccessful;
}
	
	

	
}
