/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 15.03.2005
 *
 */
package de.tudarmstadt.yajs;


import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.ObservableMap;
import de.tudarmstadt.yajs.gui.judge.ClarificationDialog;
import de.tudarmstadt.yajs.gui.judge.JudgeUIModel;
import de.tudarmstadt.yajs.gui.judge.JudgementDialog;
import de.tudarmstadt.yajs.gui.judge.YAJSJudgeUI;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventListener;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEvent;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshServerStatusEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshServerStatusEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshSubmissionsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshSubmissionsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEvent;
import de.tudarmstadt.yajs.gui.judge.events.SubmissionSelectedEventListener;
import de.tudarmstadt.yajs.gui.judge.events.TogglePauseEvent;
import de.tudarmstadt.yajs.gui.judge.events.TogglePauseEventListener;
import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.Console;
import de.tudarmstadt.yajs.lib.JudgeResult;
import de.tudarmstadt.yajs.lib.Login;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.events.AutoConfigEvent;
import de.tudarmstadt.yajs.lib.events.AutoConfigEventListener;
import de.tudarmstadt.yajs.lib.events.ClarificationLockEvent;
import de.tudarmstadt.yajs.lib.events.ClarificationLockEventListener;
import de.tudarmstadt.yajs.lib.events.ClarificationRequestEvent;
import de.tudarmstadt.yajs.lib.events.ClarificationRequestEventListener;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEvent;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEventListener;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEvent;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEventListener;
import de.tudarmstadt.yajs.lib.events.JudgeClarificationAnswerEvent;
import de.tudarmstadt.yajs.lib.events.JudgeClarificationAnswerEventListener;
import de.tudarmstadt.yajs.lib.events.JudgementEvent;
import de.tudarmstadt.yajs.lib.events.JudgementEventListener;
import de.tudarmstadt.yajs.lib.events.LoginEvent;
import de.tudarmstadt.yajs.lib.events.LoginEventListener;
import de.tudarmstadt.yajs.lib.events.ScoreboardEvent;
import de.tudarmstadt.yajs.lib.events.ScoreboardEventListener;
import de.tudarmstadt.yajs.lib.events.ServerStatusEvent;
import de.tudarmstadt.yajs.lib.events.ServerStatusEventListener;
import de.tudarmstadt.yajs.lib.events.StatusEvent;
import de.tudarmstadt.yajs.lib.events.StatusEventListener;
import de.tudarmstadt.yajs.lib.events.SubmissionLockEvent;
import de.tudarmstadt.yajs.lib.events.SubmissionLockEventListener;
import de.tudarmstadt.yajs.lib.judge.Judge;
import de.tudarmstadt.yajs.tester.Diff;
import de.tudarmstadt.yajs.tester.TestCompletedEvent;
import de.tudarmstadt.yajs.tester.TestCompletedEventListener;
import de.tudarmstadt.yajs.tester.Tester;


/**
 * The judge application, providing functionality needed for the judges. 
 * @author  markus
 */
public class YAJSJudgeApp extends YAJSClientApp {
	/** The gui associated with this app. */
	private YAJSJudgeUI gui;
	/** An observable map containing the judgements. */
	private ObservableMap judgements;
	/** An observable map containing the clarifications and requests. */
	private ObservableMap clarifications;

	/** Constructor taking a ContestInfo object. */	
	public YAJSJudgeApp(String clientConfigFileName)
	{
		super(clientConfigFileName);
		this.judgements = new ObservableMap();
		this.clarifications = new ObservableMap();
		buildGUI();
		buildClient();
		wireGUI();
		wireClient();
		setGUITitle();
	}
	
	/** Sets the title for the GUI.
	 */
	private void setGUITitle() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.setTitle(getProgramName() + " - " + getLoginGUI("judge").getName());
			}
		});
	}
	
	/** Initializing method that constructs the GUI. */
	protected void buildGUI()
	{
		try {
			final Login login = getLoginGUI("judge");
			SwingUtilities.invokeAndWait(new Runnable()	{			
				public void run() {
					gui = new YAJSJudgeUI(new JudgeUIModel(login, judgements, clarifications));
					gui.setLocationRelativeTo(null);
				}
			});
		} catch (InterruptedException e2) {
			e2.printStackTrace();
			close();
			System.exit(-1);
		} catch (InvocationTargetException e2) {
			e2.printStackTrace();
			close();
			System.exit(-1);
		}
	}
	
	/** Initializing method that constructs the Client. */
	protected void buildClient()
	{
		try {	
			client = new Judge(contestinfo, getLoginGUI("judge"), new Console());		
		} catch (IOException e1) {
			cancelLoginTimeout();
			close();
			System.err.println(Lang.getString("Unabletoconnecttoserver"));
			System.exit(-1);
		}
	}

	/** Initializing method that registers listeners for Client events. */	
	protected void wireClient() {
		getJudge().addServerStatusEventListener( new ServerStatusEventListener() {
			public void serverStatusReceived(final ServerStatusEvent event) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						gui.updateServerStatus(event.getStatus());
					}
				});
			}
		});
		
		getJudge().addJudgementEventListener( new JudgementEventListener(){
			public void judgementReceived(final JudgementEvent e)
			{		
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						judgements.put(new Long(e.getJudgement().getIndex()), e.getJudgement());
					}
				});
			}
		});
		
		getJudge().addSubmissionLockEventListener( new SubmissionLockEventListener() {

			public void submissionLocked(final SubmissionLockEvent sle) {
				JudgeResult currentResult = (JudgeResult) judgements.get(new Long(sle.getSubmissionNumber()));
				String code = sle.getCode();
				YAJSJudgeApp.this.showJudgementDialog(currentResult, code);
			}
			
			public void submissionUnlockable(SubmissionLockEvent sle) {
				gui.showErrorMessage(Lang.getString("Couldnotobtainlockforsubmissionnumber") + sle.getSubmissionNumber() + Lang.getString("Thisprobablymeansthatsomebodyelsehaslockedthefile."));
			}
			
		});

		getJudge().addClarificationLockEventListener(new ClarificationLockEventListener() {

			public void clarificationLocked(ClarificationLockEvent event) {
				showAnswerClarificationDialog(gui, event.getClarificationNumber());
			}

			public void clarificationUnlockable(ClarificationLockEvent event) {
				gui.showErrorMessage(Lang.getString("Couldnotobtainlockforclarificationnumber") + event.getClarificationNumber() + Lang.getString("Thisprobablymeansthatsomebodyelsehaslockedtheclarification."));
			}
			
		});
		
		getJudge().addStatusEventListener(new StatusEventListener() {

			public void statusChanged(final StatusEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						gui.updateContestStatus(e.getContestStatus());							
					}
				});
			}
			
		});
		
		getJudge().addConnectionErrorEventListener(new ConnectionErrorEventListener() {
			public void connectionError(ConnectionErrorEvent e) {
				gui.showErrorMessage(Lang.getString("CouldnotconnecttoserverErroris") + e.getError());
			}
		});	
		
		getJudge().addLoginEventListener( new LoginEventListener(){
			public void loginCompleted(LoginEvent e) {
				
				cancelLoginTimeout();
				
				if(e.successful())
				{
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							gui.show();
						}
					});
					startHeartbeatSignal();
				}
				else
				{
					System.err.println(Lang.getString("Loginfailed"));
					close();
					gui.showErrorMessage(Lang.getString("TheserverhasnotacceptedtheloginattemptMaybeyouhavesuppliedawrongpasswordorawrongusername"));
					//JOptionPane.showMessageDialog(null, "The server has not accepted the login attempt.\nMaybe you have supplied a wrong password or a wrong username?", "Error", JOptionPane.ERROR_MESSAGE);					
				}
			}
		});
		
		getJudge().addScoreboardEventListener(new ScoreboardEventListener() {
			public void scoreboardReceived(final ScoreboardEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						gui.updateScoreboard(e.getScoreboard());
					}
				});
			}
		});
		
		getJudge().addAutoConfigEventListener(new AutoConfigEventListener() {
			public void autoConfig(AutoConfigEvent event) {
				getContestInfo().setLanguages(event.getLanguages());
				getContestInfo().setProblems(event.getProblems());	
			}
		});
	
		getJudge().addClarificationRequestEventListener(new ClarificationRequestEventListener() {
			public void clarificationRequest(ClarificationRequestEvent event) {
				Long key = new Long(event.getRequest().getIndex());
				ClarificationPair c = (ClarificationPair) clarifications.get(key);
				if(c == null) clarifications.put(key, new ClarificationPair(event.getRequest(), null));
				else c.setRequest(event.getRequest());
			}
		});
		
		getJudge().addJudgeClarificationAnswerEventListener(new JudgeClarificationAnswerEventListener() {
			public void judgeClarificationAnswer(JudgeClarificationAnswerEvent event) {
				Long key = new Long(event.getAnswer().getRequestIndex());
				ClarificationPair c = (ClarificationPair) clarifications.get(key);
				if(c == null) clarifications.put(key, new ClarificationPair(null, event.getAnswer()));
				else clarifications.put(key, new ClarificationPair(c.getRequest(), event.getAnswer()));
			}
		});
	
		getJudge().addClientClarificationAnswerEventListener(new ClientClarificationAnswerEventListener() {
			public void clientClarificationAnswer(ClientClarificationAnswerEvent event) {
				// TODO: is it reasonable to ignore the notification flag?
				// Frankly, I'd like to know about _any_ clarification!
				// Even if it pops up! (Is that true for the judge?)
				// Anyway, we do not need to store it somewhere, as it will arrive as a JudgeAnswer, too.
				
				// If unnotified, do not pop up windows...
				// The same thing, if the clarification is _unanswered_!
				if(		(event.getAnswer().getState() == ClarificationAnswerState.UNANSWERED)
					||	(!event.isNotified()))
					return;
				
				showClarificationAnswer(event.getAnswer());
			}
		});
	}

	/** Show a dialog to answer clarification requests.
	 * @param gui <b>REFACTOR</b> not needed any more (use getGUI())
	 * @param requestIndex The index of the request to answer.
	 */
	protected void showAnswerClarificationDialog(final YAJSJudgeUI gui, final long requestIndex) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				final ClarificationPair c = (ClarificationPair) clarifications.get(new Long(requestIndex));
				ClarificationDialog dlg = new ClarificationDialog(c.getRequest(), c.getAnswer());
				dlg.addClarificationEventListener(
						new de.tudarmstadt.yajs.gui.judge.events.ClarificationEventListener() {
					public void clarificationChanged(ClarificationEvent e) {
						getJudge().sendClarificationAnswer(c.getRequest().getIndex(), e.getAnswer().getState(), e.getAnswer().getAnswer());
					}

					public void clarificationUnchanged(ClarificationEvent e) {
						getJudge().cancelClarificationAnswer(c.getRequest().getIndex());
					}
				});
				dlg.show();
			}
		});
	}
	
	/** Get the test input for a problem. 
	 * @param problem The problem to get the input for.
	 * @return Returns the String containing the test input.
	 * @throws IOException
	 */
	protected File getTestInputFile(Problem problem) throws IOException {
		//return readURIContents(getContestInfo().getRemoteTestInputLocation(problem));
		return getContestInfo().getLocalTestInputLocation(problem);
	}
	
	/** Get the test output for a problem.
	 * @param problem The problem to get the output for.
	 * @return Returns the File containing the test output.
	 * @throws IOException
	 */
	protected File getTestOutputFile(Problem problem) throws IOException {
		return getContestInfo().getLocalTestOutputLocation(problem);
	}

	/** Show a judgement dialog.
	 * Called when a judge is supposed to make a verdict.
	 * @param currentResult The previous result, if any.
	 * @param code The code that was received.
	 */
	protected void showJudgementDialog(final JudgeResult currentResult, final String code) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {			
				
				final JudgementDialog dlg = new JudgementDialog(getLoginGUI("judge").getName(), currentResult, code);
				
				if(!currentResult.needsJudgement()) {
					if(JOptionPane.YES_OPTION != JOptionPane.showConfirmDialog(dlg, Lang.getString("Doyoureallywanttorejudgesubmission") + currentResult.getIndex() + "?", Lang.getString("Rejudge"), JOptionPane.YES_NO_OPTION)) {
						getJudge().cancelJudgement(currentResult);
						return;
					}
				}
				
				dlg.addJudgementEventListener(new de.tudarmstadt.yajs.gui.judge.events.JudgementEventListener() {
					public void judgementChanged(de.tudarmstadt.yajs.gui.judge.events.JudgementEvent je) {
						getJudge().sendJudgement(je.getResult());
					}

					public void judgementUnchanged(de.tudarmstadt.yajs.gui.judge.events.JudgementEvent je) {
						getJudge().cancelJudgement(je.getResult());
					}
					
					public void testSubmission(de.tudarmstadt.yajs.gui.judge.events.JudgementEvent je) {
						dlg.setTestingEnabled(false);
						try {
							final Problem problem = currentResult.getProblem();
							try {
								String[] data={"sin","sout"};
								fetchRemoteTestData(problem, null,data,false);			
							} catch (IOException e) {
								JOptionPane.showMessageDialog(gui, Lang.getString("Couldnotfetchtestdata.AnIOErroroccured") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE);
								return;
							}
							final File input = getTestInputFile(problem);
							final double timeLimit = getTimeLimit(problem);
							Tester tester = new Tester(problem, currentResult.getLanguage(), timeLimit, code, input,contestinfo);
							try {
								tester.addTestCompletedEventListener(new TestCompletedEventListener() {
	
									public void testCompleted(final TestCompletedEvent re) {
										Diff diff = null;
										try {
											File f = re.getOutputFile();
											BufferedReader outReader = new BufferedReader(new FileReader(f));
											BufferedReader expReader = new BufferedReader(new FileReader(getTestOutputFile(problem)));

											diff = Diff.createFromReaders(outReader, expReader);
											
											/*
											String l1 = null;
											String l2 = null;
											do {
												l1 = outReader.readLine();
												l2 = expReader.readLine();
												if(l1 == null && l2 != null) {
													diff = new Diff("<EOF>", l2);
												}
												else if(l1 != null && l2 == null) {
													diff = new Diff(l1, "<EOF>");
												} else if(l1 != null && l2 != null && !l1.equals(l2)) {
													diff = new Diff(l1, l2);
													break;
												}
											} while(l1 != null && l2 != null);
											*/
										} catch (final IOException e_) {
											final IOException e = e_;	// circumvent a gcj bug
											SwingUtilities.invokeLater(new Runnable() {
												public void run() {
													JOptionPane.showMessageDialog(dlg, Lang.getString("CouldnottestIOErroroccured") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE);
												}
											});
											return;
										} finally {
											re.getOutputFile().delete();
										}
										
										final Diff theDiff = diff;
										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												dlg.setTestingEnabled(true);
												dlg.displayTestResult(theDiff, re.getError(), re.getSuggestion());
											}
										});
									}

									public void testInterrupted(final String error) {
										dlg.setTestingEnabled(true);
										SwingUtilities.invokeLater(new Runnable() {
											public void run() {
												JOptionPane.showMessageDialog(dlg, Lang.getString("CouldnottestTestinterruptedunexpectedly") + error, Lang.getString("Error"), JOptionPane.ERROR_MESSAGE);
											}
										});
									}
									
								});
								tester.start();
							} finally {
								tester.deleteAllTemporaries();
							}
						} catch (final IOException e) {
							JOptionPane.showMessageDialog(dlg, Lang.getString("Couldnottestfile") + e.getMessage(), Lang.getString("Error"), JOptionPane.ERROR_MESSAGE);
							dlg.setTestingEnabled(true);
						}
					}
				});
				dlg.show();
			}
		});
	}
	
	/** Initializing method that registers listeners for GUI events. */
	protected void wireGUI() {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					gui.addRefreshServerStatusEventListener(new RefreshServerStatusEventListener() {
						public void refreshServerStatus(RefreshServerStatusEvent e) {
							getJudge().queryServerStatus();
						}
					});
					
					gui.addWindowListener(new WindowAdapter(){
						public void windowClosing(WindowEvent arg0) {
							super.windowClosing(arg0);
							close();
						}
					});
				
					gui.addRefreshSubmissionsListener( new RefreshSubmissionsEventListener(){
						public void refreshSubmissions(RefreshSubmissionsEvent e) {
							getJudge().querySubmissions();
						}
					});
		
					gui.addRefreshScoreboardListener( new RefreshScoreboardEventListener() {
						public void refreshScoreboard(RefreshScoreboardEvent e) {
							getJudge().queryScoreboard();
						}
					});
					
					gui.addSubmissionSelectedEventListener( new SubmissionSelectedEventListener() {
						public void handleSubmission(SubmissionSelectedEvent sse) {
							getJudge().fetchSubmission(sse.getIndex());
						}
					});
					
					gui.addTogglePauseEventListener( new TogglePauseEventListener() {
						public void togglePause(TogglePauseEvent sse) {
							getJudge().switchState(sse.getNewState());
						}
					});
					
					gui.addClarificationSelectedEventListener( new ClarificationSelectedEventListener() {
						public void clarificationSelected(ClarificationSelectedEvent e) {
							getJudge().lockClarification(e.getIndex());
						}
					});
				
					gui.addAskClarificationEventListener(new AskClarificationEventListener() {
						public void askClarification(AskClarificationEvent e) {
							showNewClarificationDialog();
						}
					});
					
					gui.addRefreshClarificationsEventListener(new RefreshClarificationsEventListener() {
						public void refreshClarifications(RefreshClarificationsEvent e) {
							getJudge().queryClarifications();
						}
					});
				}
			});
		
		} catch (InterruptedException e) {
			e.printStackTrace();
			close();
			System.exit(-1);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			close();
			System.exit(-1);
		}
	}
	
	/** Convenience wrapper around getClient().
	 * Performs the cast to Judge.
	 * @return Returns the client as a Judge object.
	 */
	protected Judge getJudge() 
	{
		return (Judge) getClient(); 
	}
	
	/** Writes a submission to disk (in order to compile/run it).
	 * Writes to getSubmissionFileName(submissionNumber).
	 * @param submissionNumber The index of the submission.
	 * @param code The code that was submitted.
	 * @return Returns the File to which the data was written.
	 * @throws IOException
	 */
	private File writeSubmissionFile(long submissionNumber, String code) throws IOException {
		File filename = getSubmissionFileName(submissionNumber);
		FileWriter writer = new FileWriter(filename);
		writer.write(code.toCharArray());
		writer.close();
		return filename;
	}

	/** Get a unique filename for a submission.
	 * This simply builds a file &lt;submissionNumber&gt;.
	 * @param submissionNumber The index of the submission.
	 * @return Returns a File named <em>submissionNumber</em>.
	 */
	private File getSubmissionFileName(long submissionNumber) {
		String path = System.getProperty("user.dir") + System.getProperty("file.separator") + String.valueOf(submissionNumber);
		return new File(path);
	}
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#dispose()
	 */
	protected void dispose() {
		super.dispose();
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					gui.dispose();	
				}
			});
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#showErrorMessage(java.lang.String)
	 */
	protected void showErrorMessage(String message) {
		gui.showErrorMessage(message);
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#getGUI()
	 */
	protected JFrame getGUI() {
		return gui;
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.YAJSClientApp#getProgramName()
	 */
	protected String getProgramName() {
		return Lang.getString("SageRefefreeJudge");
	}
}
