/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui;

import java.util.Collection;
import java.util.Iterator;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;

/** A combo box model based on Collections, not on arrays.
 * This is itself a collection.
 * @author markus
 */
public class CollectionComboBoxModel extends AbstractListModel implements
		ComboBoxModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5407148058058735046L;

	/** The underlying collection. */
	private Collection collection;
	
	/** The currently selected object. */
	private Object selected;
	
	/** Constructs this model from the collection c.
	 * @param c The collection this model encapsulates.
	 */
	public CollectionComboBoxModel(Collection c)
	{
		this.collection = c;
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.ComboBoxModel#getSelectedItem()
	 */
	public Object getSelectedItem() {
		return selected;
	}

	/* (non-Javadoc)
	 * @see javax.swing.ComboBoxModel#setSelectedItem(java.lang.Object)
	 */
	public void setSelectedItem(Object arg0) {
		selected = arg0;
	}

	/* (non-Javadoc)
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize() {
		return collection.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int arg0) {
		Iterator iter = collection.iterator();
		for(int i=0; i<arg0; ++i) iter.next();
		return iter.next();
	}

}
