/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.tudarmstadt.yajs.gui;

import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.lib.ContestStatus;
import de.tudarmstadt.yajs.lib.ContestStatusEnumeration;
import java.awt.GridLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
/**
 * A status panel indicating the contest status (Time elapsed, running or not...).
 * @author  markus
 */
public class ContestStatusPanel extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** The string to display if no status was received yet. */
	private final static String NO_STATUS = Lang.getString("Conteststatusisnotavailable");
	private String statusID="notRunning";
	/** The label to display the status in. */
	protected JLabel statusLabel;
	
	/** Constructor. */
	public ContestStatusPanel() {
		build();
	}
	
	/** Initialization method building the panel. */
	protected void build() {
		setLayout(new GridLayout(1,1));
		add(getStatusLabel());
	}
	
	/**
	 * Return the status label.
	 * @return  Returns the status label.
	 * @uml.property  name="statusLabel"
	 */
	protected JLabel getStatusLabel() {
		if(statusLabel == null) {
			statusLabel = new JLabel(NO_STATUS);
		}
		return statusLabel;
	}
	
	/** Update the contest status.
	 * @param contestStatus
	 */
	public void updateContestStatus(ContestStatus contestStatus) {
		String time = Lang.getString("Time[min]") + Long.toString(contestStatus.getElapsed()) + "/" + Long.toString(contestStatus.getDuration());
		String status = "?";
		statusID="notRunning";
		ContestStatusEnumeration cstatus = contestStatus.getStatus();
		if(cstatus == ContestStatusEnumeration.BEFORE)
			status = Lang.getString("Thecontesthasnotyetstarted");
		else if(cstatus == ContestStatusEnumeration.RUNNING){
			status = Lang.getString("Thecontestisrunning");
			statusID="isRunning";
		}
		else if(cstatus == ContestStatusEnumeration.PAUSED)
			status = Lang.getString("Thecontestispaused");
		else if(cstatus == ContestStatusEnumeration.AFTER)
			status = Lang.getString("Thecontestisover");
		getStatusLabel().setText(status + " - " + time);
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public String getStatusID() {
		return statusID;
	}

}
