/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui;

import java.awt.Color;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import de.tudarmstadt.lang.*;
/** A panel that shows differences between two strings.
 * For ease of implementation, this only highlights everything starting from the first difference.
 * @author markus
 */
public class DiffPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -1614328626117275814L;
	private JTextPane left;
	private JTextPane right;
	
	/** Constructor. Create a <code>DiffPanel</code> for comparison of <code>leftText</code> and <code>rightText</code>.
	 * <b>REFACTOR</b> I think this should in fact display a Diff-object, rather than calculate the difference itself.
	 * @param leftText First string to inspect.
	 * @param rightText Second string to inspect.
	 */
	public DiffPanel(String leftText, String rightText) {
		super(new GridLayout(1,2));
		this.left = new JTextPane();
		this.right = new JTextPane();
		left.setEditable(false);
		right.setEditable(false);
		JScrollPane leftPane = new JScrollPane(left);
		leftPane.setBorder(BorderFactory.createTitledBorder(Lang.getString("Output")));
		JScrollPane rightPane = new JScrollPane(right);
		rightPane.setBorder(BorderFactory.createTitledBorder(Lang.getString("Expected")));
		setText(leftText, rightText);
		add(leftPane);
		add(rightPane);
	}

	/** Sets the two strings to compare.
	 * @param leftText First string to inspect.
	 * @param rightText Second string to inspect.
	 */
	public void setText(String leftText, String rightText) {
		int i = findFirstDiffers(leftText, rightText);
		int i1 = findLine(leftText, i);
		int i2 = findLine(rightText, i);

		setText(left, leftText, i1);
		setText(right, rightText, i2);
	}
	
	/** Sets the text in one of the text panes, highlighting the substring starting at character <code>split</code>.
	 */
	protected void setText(JTextPane pane, String text, int split) {
		Document doc = pane.getEditorKit().createDefaultDocument();
		try {
			doc.insertString(doc.getLength(), text.substring(0, split), getDefaultAttributes());
			doc.insertString(doc.getLength(), text.substring(split, text.length()), getHighlightAttributes());
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1000);
		}
		pane.setDocument(doc);
	}
	

	/** Convenience method that returns the AttributeSet to use for highlighting.
	 * @return Returns an AttributeSet.
	 */
	private AttributeSet getHighlightAttributes() {
		MutableAttributeSet as = new SimpleAttributeSet();
		StyleConstants.setBackground(as, Color.RED);
		return as;
	}

	/** Convenience method that returns the AttributeSet to use for non-differing sections.
	 * @return Returns an AttributeSet.
	 */
	private AttributeSet getDefaultAttributes() {
		MutableAttributeSet as = new SimpleAttributeSet();
		return as;
	}

	/** Finds the beginning of the line containing character <code>i</code>
	 * @param leftText The string to find the line in.
	 * @param i The position to find the surrounding line for.
	 * @return Returns the index pointing to the beginning of the line containing the <code>i</code>-th character.
	 */
	private int findLine(String text, int i) {
		if(i == -1) return text.length();
		int j = text.lastIndexOf("\n", i-1);
		if(j == -1) return 0;
		return j+1;
	}

	/** Find the position of the first difference in (<code>leftText</code>,<code>rightText</code>).
	 * @param leftText A string.
	 * @param rightText Another string.
	 * @return Returns an int specifying the position of the first difference.
	 */
	private int findFirstDiffers(String leftText, String rightText) {
		int i = 0;
		int k = Math.min(leftText.length(), rightText.length());
		while(i < k && (leftText.charAt(i) == rightText.charAt(i))) ++i;
		if(i == leftText.length() && i == rightText.length()) return -1;
		return i;
	}
}
