/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 28.04.2005
 *
 */
package de.tudarmstadt.yajs.gui;

import java.util.Iterator;
import java.util.Map;

/**
 * A concrete subclass of <code>MapItemSelection</code>. Uses a Predicate to filter elements from a Map.
 * @author  markus
 */
public class FilteredMapItemSelection extends MapItemSelection {
	/** The underlying map.
	 * This could be more flexible (i.e. a collection).
	 * But then I would have to care about elements
	 * having type "Map.Entry" and alike...
	 */
	private Map base;
	/** The filter predicate to use. */
	private Predicate filter;
	/** Constructor. Creates a <code>FilteredMapItemSelection</code> from the given map and predicate.
	 * @param base The underlying map.
	 * @param filter A predicate serving as a filter.
	 */
	public FilteredMapItemSelection(Map base, Predicate filter)
	{
		this.base = base;
		this.filter = filter;
	}
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.gui.ItemSelection#buildItems()
	 */
	protected void buildItems() {
		synchronized(base) {
			Iterator i = base.entrySet().iterator();
			while(i.hasNext())
			{
				Map.Entry e = (Map.Entry) i.next();
				if(filter.contains(e.getValue())) 
					keys.add(e.getKey());
			}	
		}
	}

}
