/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui;
import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.lib.Login;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
/**
 * A login dialog. Prompts for user name and password.
 */
public class LoginDialog extends JDialog
{

		/**
	 * 
	 */
	private static final long serialVersionUID = -6709967244841239726L;
		/**
		 * @uml.property  name="nameField"
		 * @uml.associationEnd  multiplicity="(1 1)"
		 */
		private JTextField nameField;
		/**
		 * @uml.property  name="passwordField"
		 * @uml.associationEnd  multiplicity="(1 1)"
		 */
		private JPasswordField passwordField;
		/**
		 * @uml.property  name="isOnline"
		 */
		private boolean isOnline;
	/** Constructor. Sets the parent component to <code>ui</code> and the window title to <code>programName</code>.
	 */
	public LoginDialog(JFrame ui, String programName, String usertype)
	{
		super(ui, programName + " - Login", true);

		Container pane = getContentPane();
		pane.setLayout(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		GridBagConstraints inputConstraints = new GridBagConstraints();
		GridBagConstraints buttonConstraints = new GridBagConstraints();
		
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		labelConstraints.fill = GridBagConstraints.HORIZONTAL;
		labelConstraints.weightx = 0;
		labelConstraints.weighty = 1;
		labelConstraints.insets = new Insets(8, 8, 8, 8);
		
		inputConstraints.gridx = 1;
		inputConstraints.gridy = 0;
		inputConstraints.fill = GridBagConstraints.HORIZONTAL;
		inputConstraints.weightx = 1;
		inputConstraints.weighty = 1;
		inputConstraints.insets = new Insets(8, 8, 8, 8);
		
		buttonConstraints.gridx = 0;
		buttonConstraints.gridy = 2;
		buttonConstraints.gridwidth = 2;
		buttonConstraints.fill = GridBagConstraints.HORIZONTAL;
		buttonConstraints.anchor = GridBagConstraints.SOUTHEAST;
		buttonConstraints.weightx = 1;
		buttonConstraints.weighty = 1;
		buttonConstraints.insets = new Insets(8, 8, 8, 8);
		
		pane.add(new JLabel(Lang.getString("Name")), labelConstraints);
		pane.add(getNameField(), inputConstraints);
		
		labelConstraints.gridy = 1;
		inputConstraints.gridy = 1;

		pane.add(new JLabel(Lang.getString("Password")), labelConstraints);
		pane.add(getPasswordField(), inputConstraints);
	
		JPanel buttons = new JPanel(new FlowLayout(FlowLayout.TRAILING));
		
		JButton offlineButton=new JButton("offline");
		offlineButton.setMnemonic('F');
		
		offlineButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				isOnline=false;
				presetValues(System.getProperty("user.name"), System.getProperty("user.name"));
				dispose();
			}	
		});
		
		JButton okbutton = new JButton(Lang.getString("login"));
		okbutton.setMnemonic('O');
		okbutton.addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				isOnline=true;
				dispose();
			}	
		});
		buttons.add(okbutton);
		buttons.add(offlineButton);
		offlineButton.setEnabled(usertype.equalsIgnoreCase("contestant"));
		pane.add(buttons, buttonConstraints);
		getRootPane().setDefaultButton(okbutton);
		setSize(300, 150);
		
		
			
		
	}
	
	/**
	 * Getter for the password field.
	 * @return  Returns the password field.
	 * @uml.property  name="passwordField"
	 */
	private JPasswordField getPasswordField() {
		if(passwordField == null) {
			passwordField = new JPasswordField();
		}
		return passwordField;
	}

	/**
	 * Getter for the user-name field.
	 * @return  Returns the user-name text field.
	 * @uml.property  name="nameField"
	 */
	private JTextField getNameField() {
		if(nameField == null) 
		{
			nameField = new JTextField();
		}
		return nameField;
	}

	/** Getter for the Login object defined by the user's entries. 
	 * @return Returns a newly created Login object corresponding to the user's inputs.
	 */
	public Login getLogin()
	{
		return new Login(nameField.getText(), new String(passwordField.getPassword()),isOnline);		
	}

	/** Use predefined login name/password.
	 * Disables the name/password text fields and presets their values.
	 * @param name
	 * @param password
	 */
	public void presetValues(String name, String password) {
		nameField.setText(name);
		nameField.setEnabled(false);
		passwordField.setText(password);
		passwordField.setEnabled(false);
	}
	
}
