/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 28.04.2005
 *
 */
package de.tudarmstadt.yajs.gui;

import java.util.ArrayList;

/** An abstract filter for <code>MyTableModel</code>.
 * Manages a list of keys into the map stored in the table model.
 * @author markus
 */
public abstract class MapItemSelection {
	protected ArrayList keys;

	/** Invalidate the list of keys.
	 * Achieved by setting <code>keys</code> to <code>null</code>
	 */
	public synchronized void invalidate() {
		keys = null;
	}

	/** Return the size of the list of keys.
	 * (Re-)evaluates the list if necessary, i.e. when the list has been invalidated.
	 * @return Returns the size of the list.
	 */
	public synchronized int size() {
		assertValid();
		return keys.size();
	}

	/** Re-evaluates the list if necessary.
	 */
	private synchronized void assertValid() {
		if(!isValid())
		{
			keys = new ArrayList();
			buildItems();
		}
	}

	/** Abstract method that provides the functionality to actually build the list of keys.
	 */
	protected abstract void buildItems();

	/** Getter that returns whether the current list is valid.
	 * @return Returns true if the list is valid, false if it is invalidated.
	 */
	private synchronized boolean isValid() {
		return keys != null;
	}
	/** Get the object (key) at position <code>index</code>.
	 * (Re-)evaluates the list if necessary, i.e. when the list has been invalidated.
	 * @param index
	 * @return
	 */
	public synchronized Object get(int index) {
		assertValid();
		return keys.get(index);
	}
}
