/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.tudarmstadt.yajs.gui;

import java.util.Observable;
import java.util.Observer;
import javax.swing.table.AbstractTableModel;

/**
 * A table model that presents elements from an ObservableMap. The layout is given by a MyTableItemLayout, an additional filtering step can be applied  using a MapItemSelection.
 * @author  markus
 */
public class MyTableModel extends AbstractTableModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8958934544889879355L;
	/** The layout to use. */
	private MyTableItemLayout itemlayout;
	/** The underlying map with entries. */
	private ObservableMap items;
	/** A filter. */
	private MapItemSelection selection;
	
	/** Constructor. */
	public MyTableModel(ObservableMap items, MyTableItemLayout itemlayout, MapItemSelection selection)
	{
		this.items = items;
		this.itemlayout = itemlayout;
		this.selection = selection;
		this.items.addObserver(
			new Observer(){
				public void update(Observable arg0, Object arg1) {
					clearResultCache();
					fireTableStructureChanged();
				}
			}
		);
	}
	
	/** Constructor. Calls <code>MyTableModel(items, itemlayout, new FilteredMapItemSelection(items, Predicate.alwaysTrue))</code>.
	 * I.e. shows all entries.
	 */
	public MyTableModel(ObservableMap items, MyTableItemLayout itemlayout)
	{
		this(items, itemlayout, new FilteredMapItemSelection(items, Predicate.alwaysTrue));
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return itemlayout.getColumnCount();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return selection.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int row, int col) {
		return itemlayout.getValue(getItem(row), col);
	}
	
	/** Due to a change in the map, the selection mapping might be wrong.
	 * This method invalidates the mapping and forces it to be reconstructed the next time it is used.
	 */
	private void clearResultCache() {
		selection.invalidate();
	}
	
	/** Getter for the object stored at row <code>index</code>.
	 * @param index The table row for which to get the corresponding object.
	 * @return Returns the object that corresponds to row <code>index</code>.
	 */
	public Object getItem(int index) {
		return items.get(selection.get(index));
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	public String getColumnName(int col) {
		return itemlayout.getColumnName(col);
	}
}
