/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 21.02.2005
 *
 */
package de.tudarmstadt.yajs.gui;

import java.util.AbstractMap;
import java.util.AbstractSet;
import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.Observer;
import java.util.Set;
import java.util.TreeMap;

/**
 * An observable map. This map allows registering listeners that will be notified whenever the map is changed. Note that this only affects changes made to the map, not changes made to the elements contained therein.
 * @author  markus
 */
public class ObservableMap extends AbstractMap {
	
	/** The underlying map. */
	private Map map = Collections.synchronizedMap(new TreeMap());
	/** The supporting observable object. */
	private SunSuxObservable observable = new SunSuxObservable();
	
	/** A concrete EntrySet implementation that performs notifications when necessary. */
	private class EntrySet extends AbstractSet
	{

		/* (non-Javadoc)
		 * @see java.util.AbstractCollection#size()
		 */
		public int size() {
			return map.size();
		}
		
		/* (non-Javadoc)
		 * @see java.util.AbstractCollection#iterator()
		 */
		public Iterator iterator() {
			return new Iterator()
			{
				Object at = null;
				Iterator iterator = map.entrySet().iterator();

				public void remove() {
					iterator.remove();
					observable.notifyObservers(at);
				}

				public boolean hasNext() {
					return iterator.hasNext();
				}

				public Object next() {
					return at = iterator.next();
				}
				
			};
		}
	}

	/* (non-Javadoc)
	 * @see java.util.Map#entrySet()
	 */
	public Set entrySet() {
		return new EntrySet();
	}
	
	/* (non-Javadoc)
	 * @see java.util.AbstractMap#put(java.lang.Object, java.lang.Object)
	 */
	public synchronized Object put(Object key, Object value) {
		Object oldvalue = map.put(key, value);
		observable.setChanged();
		observable.notifyObservers(key);
		return oldvalue;
	}
	
	/* (non-Javadoc)
	 * @see java.util.AbstractMap#remove(java.lang.Object)
	 */
	public synchronized Object remove(Object key) {
		boolean changed = map.containsKey(key);
		Object oldvalue = map.remove(key);
		if(changed) { observable.setChanged(); observable.notifyObservers(key); }
		return oldvalue;
	}
	
	/** Add an observer.
	 * @param observer An observer to add.
	 */
	public void addObserver(Observer observer)
	{
		observable.addObserver(observer);
	}
}
