/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 11.02.2005
 *
 */
package de.tudarmstadt.yajs.gui;

import de.tudarmstadt.yajs.lib.Result;
import de.tudarmstadt.yajs.lib.Status;

import de.tudarmstadt.lang.*;
/** This class prints a <code>Result</code> object in a human readable way.
 * It is a collection of static methods working on <code>Result</code> objects.
 * @author markus
 */
public class ResultPrinter {
	/** Get a string describing the reason given in the result. 
	 * @param r The result to work on.
	 * @return Returns a string description of the reason.
	 */
	public static String getReason(Result r) {
		return r.getReason();
	}
	
	/** Get a string describing the status. 
	 * This will be along the lines of "Not yet judged", "accepted", etc.
	 * @param r The result to work on.
	 * @return Returns a string description of the status. 
	 */
	public static String getStatus(Result r) {
		if(r.getStatus() == Status.UNJUDGED) return Lang.getString("Judgementrevoked");
		if(r.getStatus() == Status.NEW) return Lang.getString("Notyetjudged");
		if(r.getStatus() == Status.IGNORED) return Lang.getString("Submissionignored");
		return r.getStatus().toString();
	}
	
	/** Get a string describing the verdict, i.e. both the status and the reason.
	 * @param r The result to work on.
	 * @return Returns a string description of the verdict.
	 */
	public static String getVerdict(Result r) {
		if(r.getStatus() == Status.UNJUDGED) return Lang.getString("Judgementrevoked");
		if(r.getStatus() == Status.NEW) return Lang.getString("Notyetjudged");
		if(r.getStatus() == Status.IGNORED) return Lang.getString("Submissionignored")+": " + getReason(r);
		return getStatus(r) + " - " + getReason(r);
	}
}
