/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.contestant;

import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.gui.FilteredMapItemSelection;
import de.tudarmstadt.yajs.gui.MyTableItemLayout;
import de.tudarmstadt.yajs.gui.MyTableModel;
import de.tudarmstadt.yajs.gui.Predicate;
import de.tudarmstadt.yajs.gui.judge.ClarificationsModel;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventListener;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventListener;
import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.ClientClarificationAnswer;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
/**
 * A panel that manages presentation of clarifications. Clarifications can be filtered using a predicate. <b>REFACTOR</b> Shouldn't this be a MapItemSelection?
 * @author  markus
 */
public class ClarificationsPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -8756355617289991077L;
	private ClarificationsModel model;
	private JTable clarificationstable;
	private JEditorPane clarificationText;
	private ClarificationEventGenerator clarificationSignal;
	private ClarificationSelectedEventGenerator clarificationSelectedSignal; 
	private RefreshClarificationsEventGenerator refreshClarificationsSignal;
	private AskClarificationEventGenerator askClarificationSignal;
	protected String problemString;
	protected String clarificationString;
	protected String toString;
	protected String unansweredString;
	protected String pleasesendreportbugString;
	private String clarificationoverviewString;
	private JButton askButton;
	private JButton refreshButton;
	
	
	/** Constructor.
	 * @param model The clarification model.
	 * @param filter A filter to apply.
	 */
	public ClarificationsPanel(ClarificationsModel model, Predicate filter) {
		super();
		this.model = model;
		this.clarificationSignal = new ClarificationEventGenerator();
		this.clarificationSelectedSignal = new ClarificationSelectedEventGenerator();
		this.refreshClarificationsSignal = new RefreshClarificationsEventGenerator();
		this.askClarificationSignal = new AskClarificationEventGenerator();
		buildPanel(filter);
		
		
	}

	/** Initialization method that creates the panel. */
	private void buildPanel(Predicate filter)
	{
		/* Initialize layout (grid bag layout) */
		
		setLayout(new GridBagLayout());
		GridBagConstraints tableConstraints = new GridBagConstraints();
		GridBagConstraints textConstraints = new GridBagConstraints();
		GridBagConstraints buttonConstraints = new GridBagConstraints();
		
		tableConstraints.fill = GridBagConstraints.BOTH;
		tableConstraints.gridx = 0;
		tableConstraints.gridy = 0;
		tableConstraints.weightx = 1.0;
		tableConstraints.weighty = 1.0;
		
		textConstraints.fill = GridBagConstraints.BOTH;
		textConstraints.gridx = 0;
		textConstraints.gridy = 1;
		textConstraints.weightx = 1.0;
		textConstraints.weighty = 1.0;
		
		buttonConstraints.fill = GridBagConstraints.BOTH;
		buttonConstraints.gridx = 0;
		buttonConstraints.gridy = 2;
		buttonConstraints.weightx = 1.0;
		buttonConstraints.weighty = 0.0;
		
		
		/* initialize the result table */

		MyTableItemLayout clarificationlayout = new MyTableItemLayout() {

			public int getColumnCount() {
				return 4;
			}

			public String getColumnName(int col) {
				switch(col)
				{
					case 0: return "#";
					case 1: return problemString;
					case 2: return clarificationString;
					case 3: return toString;
				}
				return "";
			}

			public Object getValue(Object item, int col) {
				ClientClarificationAnswer r = (ClientClarificationAnswer) item;
				switch(col)
				{
					case 0: return new Long(r.getRequestIndex());
					case 1:
						if(r.isGeneralClarification()) return Lang.getString("General");
						return r.getProblem().getID() + " - " + r.getProblem().getName();
					case 2: return r.getState() == ClarificationAnswerState.UNANSWERED
						? unansweredString
						: r.getAnswer();
					case 3: return r.getState() == ClarificationAnswerState.UNANSWERED
						? ""
						: r.getState().toString();
					default: return pleasesendreportbugString;
				}
			}
		};

		clarificationstable = new JTable( new MyTableModel(getModel().getClarifications(), clarificationlayout, new FilteredMapItemSelection(getModel().getClarifications(), filter)) );
		clarificationstable.addMouseListener(new MouseAdapter() {
			/* (non-Javadoc)
			 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent arg0) {
				super.mouseClicked(arg0);
				updateClarificationText();
			}	
		});

		JScrollPane runsscrollpane = new JScrollPane(clarificationstable);
		runsscrollpane.setBorder(BorderFactory.createTitledBorder(clarificationoverviewString));
		
		add(runsscrollpane, tableConstraints);
	
		/* Add text view */
		clarificationText = new JEditorPane();
		clarificationText.setEditable(false);
		clarificationText.setContentType(Lang.getString("textplain"));
		JScrollPane clarificationsPane = new JScrollPane(clarificationText);
		add(clarificationsPane, textConstraints);
		
		/* Add buttons */
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new FlowLayout(FlowLayout.TRAILING));
		
		askButton = new JButton(Lang.getString("Ask"));
		askButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				askClarificationSignal.emitAskClarification(new AskClarificationEvent(this));
			}
		});
		buttons.add(askButton);
		
		refreshButton = new JButton(Lang.getString("Refresh"));
		buttons.add(refreshButton);
		refreshButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				refreshClarificationsSignal.emitRefreshClarifications( new RefreshClarificationsEvent(ClarificationsPanel.this));
			}
		});
		
		add(buttons, buttonConstraints);
	}
	
	/** Show the currently selected clarification's content.
	 */
	protected void updateClarificationText() {
		MyTableModel tm = (MyTableModel) getClarificationsTable().getModel(); 
		ClientClarificationAnswer clarification = (ClientClarificationAnswer) tm.getItem(getClarificationsTable().getSelectedRow()); 
		clarificationText.setText(clarification.getAnswer());
	}

	/**
	 * Retrieve the underlying ClarificationsModel.
	 * @return  The clarification model.
	 * @uml.property  name="model"
	 */
	private ClarificationsModel getModel() {
		return model;
	}

	/** Registers a RefreshClarificationsEventListener that will be called back whenever the user wants to refresh the list of clarifications.
	 * @param listener The listener to register.
	 */
	public void addRefreshClarificationsEventListener(RefreshClarificationsEventListener listener) {
		refreshClarificationsSignal.registerListener(listener);
	}
	
	/** Getter for the clarifications table.
	 * @return Returns the clarification table.
	 */
	public JTable getClarificationsTable() {
		return clarificationstable;
	}

	/** Registers a AskClarificationsEventListener that will be called back whenever the user wants to ask for clarification.
	 * @param listener The listener to register.
	 */
	public void addAskClarificationEventListener(AskClarificationEventListener listener) {
		askClarificationSignal.registerListener(listener);
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void updateRuntimeGUILanguage() {
		problemString=Lang.getString("Problem");
		clarificationString=Lang.getString("Clarification");
		toString=Lang.getString("To");
		unansweredString=Lang.getString("unanswered");
		pleasesendreportbugString=Lang.getString("Pleasesendbugreport");
		clarificationoverviewString=Lang.getString("Clarificationoverview");
		askButton.setText(Lang.getString("Ask"));
		refreshButton.setText(Lang.getString("Refresh"));
		
	}

}
