/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.contestant;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.FileSelector;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEvent;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEvent;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEvent;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEvent;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEvent;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.TestEvent;
import de.tudarmstadt.yajs.gui.contestant.events.TestEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.TestEventListener;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.Problem;

/**
 * A panel used in the submit panel in order to allow testing, submitting and fetching data for a single problem.
 * @author  markus
 */
public class ProblemPanel extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 6475245499606586274L;

	/** The problem this panel is associated with. */
	private Problem problem;
	
	/** The submit model. */
	private SubmitModel model;
	/** The programming language the currently selected file is written in. */
	private Language submissionLanguage;
	/** The zip file that contains all the original templates*/

	/** A file selector to choose a file. */
	protected FileSelector fileSelector;
	/** The editor button. */
	protected EditorButton editorButton;
	/** The manual test button. */
	protected JButton manTestButton;
	/** The submit button. */
	protected JButton submitButton;
	/** The refetch test data button. */
	protected JButton refetchButton;
	/** The test button. */
	protected JButton testButton;
	/** The language button. */
	protected JButton createTemplateSrc;
	/** A label indicating the latest judgement. */
	protected JLabel judgementLabel;
	
//	protected ChangLanguageEventGenerator changeLanguageSignal = new ChangLanguageEventGenerator();
	protected SubmitEventGenerator submitSignal = new SubmitEventGenerator();
	protected RefetchEventGenerator refetchSignal = new RefetchEventGenerator();
	protected TestEventGenerator testSignal = new TestEventGenerator();
	protected EditorEventGenerator editorSignal = new EditorEventGenerator();
	protected ManualTestEventGenerator mantestSignal = new ManualTestEventGenerator();
	protected CreateTemplateSrcEventGenerator createTemplateSrcSignal= new CreateTemplateSrcEventGenerator();
	private boolean isOnline;
	private boolean contestIsRunning=false;
	/** Constructor. 
	 * @param problem The problem this panel is associated with.
	 * @param model The submit model.
	 */
	public ProblemPanel(Problem problem, SubmitModel model,boolean isOnline) {
		this.isOnline=isOnline;			
		this.problem = problem;
		this.model = model;
		init();
		wire();
		
	}
	
	/** Initialization method that creates the panel.
	 */
	private void init() {
		setLayout(new GridLayout(3,3));
		createTemplateSrc = new JButton(Lang.getString("ProblemPanel.1")); //$NON-NLS-1$
		add(createTemplateSrc);
		manTestButton = new JButton(Lang.getString("ProblemPanel.6")); //$NON-NLS-1$
		manTestButton.setEnabled(false);
		add(manTestButton);
		submitButton = new JButton(Lang.getString("submit")); //$NON-NLS-1$
		submitButton.setEnabled(false);
		add(submitButton);
	
		fileSelector = new FileSelector(Lang.getString("select"));
		add(fileSelector);
		
		
		testButton = new JButton(Lang.getString("ProblemPanel.4")); //$NON-NLS-1$
		testButton.setEnabled(false);
		add(testButton);
		judgementLabel = new JLabel(Lang.getString("nosubmission")); //$NON-NLS-1$
		add(judgementLabel);
		editorButton = new EditorButton(Lang.getString("editor")); //$NON-NLS-1$
		add(editorButton);
	
		refetchButton = new JButton(Lang.getString("fetchtests")); //$NON-NLS-1$
		add(refetchButton);
		
		
		
		
		
		
		
		
		
	}
	/**Set the contest status*/
	public void setContestStatus(boolean isRunning){contestIsRunning=isRunning;}
	
	/** Initialization method that registers event listeners. */
	protected void wire() {
		testButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				testSignal.emitTest(new TestEvent(ProblemPanel.this, getProblem(), getFile(), getSubmissionLanguage()));
			}
		});
		
		refetchButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				refetchSignal.emitRefetch(new RefetchEvent(ProblemPanel.this, getProblem(),getSubmissionLanguage()));
				
			}
		});
		
		submitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				submitSignal.emitSubmitProblem(getSubmitEvent());
			}
		});
		editorButton.addButtonListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				editorSignal.emitEditor(getEditorEvent());
			}
		});
		manTestButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mantestSignal.emitTest(new ManualTestEvent(ProblemPanel.this, getProblem(), getFile(), getSubmissionLanguage()));
			}
		});
		createTemplateSrc.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent arg0) {
//				Object[] selections = model.getLanguages().toArray();
//				Language language=(Language) JOptionPane.showInputDialog(null, Lang.getString("Pleasechoosealanguage"), Lang.getString("Selectalanguage"), JOptionPane.QUESTION_MESSAGE, null, selections, selections[0]); //$NON-NLS-1$ //$NON-NLS-2$
//				if (language!=null){
//					setLanguage(language);
//					setFileUsingEditor(null);
//				}
				
				createTemplateSrcSignal.emitTest(new CreateTemplateSrcEvent( ProblemPanel.this,getProblem()));
				//use fetchOfflineTestData with the observer pattern.
				//check if no existing source code for that language exists, 
				//copy over from the zip to the problem directory.
				
				//if it exists, an additional confirmation dialog pops up before overwriting.

				
			}
		});
		
		fileSelector.addVetoableChangeListener(new VetoableChangeListener() {
			public void vetoableChange(PropertyChangeEvent arg0) throws PropertyVetoException {
				File file = (File) arg0.getNewValue();
				fireVetoableChange(Lang.getString("file"), arg0.getOldValue(), arg0.getNewValue()); //$NON-NLS-1$
				testButton.setEnabled(file != null && file.exists() && file.isFile());
				manTestButton.setEnabled(file != null && file.exists() && file.isFile());
				//if(isOnline)submitButton.setEnabled(file != null && file.exists() && file.isFile());
			}
		});
	}
	
	/**
	 * Get the submission language. 
	 * @return  Returns the submission language.
	 * @uml.property  name="submissionLanguage"
	 */
	public Language getSubmissionLanguage() {
		return submissionLanguage;
	}
	
	/**
	 * Set the submission language.
	 * @param l  The language to set.
	 * @uml.property  name="submissionLanguage"
	 */
	public void setSubmissionLanguage(Language l) {
		submissionLanguage = l;
	}
	
	/** Get the SubmitEvent that corresponds to the current user input.
	 * @return Returns a newly created SubmitEvent object wrapping the current input.
	 */
	protected SubmitEvent getSubmitEvent() {
		return new SubmitEvent(this, getProblem(), getSubmissionLanguage(), getFile()); 
	}
	/** Get the EditorEvent that corresponds to the current user input.
	 * @return Returns a newly created EditorEvent object wrapping the current input.
	 */
	protected EditorEvent getEditorEvent() {
		return new EditorEvent(this, getProblem(), getSubmissionLanguage(),getFile()); 
	}

	/**
	 * Get the problem this panel is associated with.
	 * @return  Returns the problem this panel is associated with.
	 * @uml.property  name="problem"
	 */
	protected Problem getProblem() {
		return problem;
	}
	
	/** Set the text indicating the current judgement. */
	public void setJudgementText(String text) {
		judgementLabel.setText(text);
	}
	
	/** Get the currently selected file. 
	 * @return Returns the currently selected file.
	 */
	public File getFile() {
		return fileSelector.getFile();
	}
	public void setLanguage(Language language){
		this.submissionLanguage=language;
	}
	/** Set the currently selected file. 
	 *
	 */
	public void setFileUsingEditor(File file) {
		fileSelector.setFile(file);
		testButton.setEnabled(file != null && file.exists() && file.isFile());
		manTestButton.setEnabled(file != null && file.exists() && file.isFile());
		if(isOnline&&contestIsRunning)submitButton.setEnabled(file != null && file.exists() && file.isFile());
	
	}
	/** Registers a SubmitEventListener that will be called back whenever the user wants to submit the currently selected file. 
	 * @param listener The listener to register.
	 */
	public void addSubmitEventListener(SubmitEventListener listener) {
		submitSignal.registerListener(listener);
	}
	
	/** Registers a RefetchEventListener that will be called back whenever the user wants to refetch test data for the associated problem. 
	 * @param listener The listener to register.
	 */
	public void addRefetchEventListener(RefetchEventListener listener) {
		refetchSignal.registerListener(listener);
	}
	/** Registers a CreateTemplateSrcEventListener 
	 * * @param listener The listener to register.
	 */
	public void addCreateTemplateSrcEventListener(CreateTemplateSrcEventListener listener) {
		createTemplateSrcSignal.registerListener(listener);
	}

	/** Registers a ManualTestEventListener that will be called back whenever the user wants to test the currently selected file manually. 
	 * @param listener The listener to register.
	 */
	public void addManualTestEventListener(ManualTestEventListener listener) {
		mantestSignal.registerListener(listener);
	}

	/** Registers a TestEventListener that will be called back whenever the user wants to test the currently selected file. 
	 * @param listener The listener to register.
	 */
	public void addTestEventListener(TestEventListener listener) {
		testSignal.registerListener(listener);
	}
	
	/** Register a EditorEventListener .
	 * @param  listener The listener to register.
	 */
	public void addEditorEventListener(EditorEventListener listener) {
		editorSignal.registerListener(listener);
	}

	/** Enables or disables fetching test data for the associated problem.
	 * @param b True if fetching should be enabled, false otherwise.
	 */
	
	public void enableFetching(boolean b) {
		refetchButton.setEnabled(b);
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void updateRuntimeGUILanguage() {

		fileSelector.setText(Lang.getString("select"));
		submitButton.setText(Lang.getString("submit")); //$NON-NLS-1$
		createTemplateSrc.setText(Lang.getString("ProblemPanel.1")); //$NON-NLS-1$
		editorButton.setText(Lang.getString("editor")); //$NON-NLS-1$
		testButton.setText(Lang.getString("ProblemPanel.4")); //$NON-NLS-1$
		judgementLabel.setText(Lang.getString("nosubmission")); //$NON-NLS-1$
		manTestButton.setText(Lang.getString("ProblemPanel.6")); //$NON-NLS-1$
		refetchButton.setText(Lang.getString("fetchtests")); //$NON-NLS-1$
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void setFileForEditor(File file) {
		editorButton.setTextFieldText(file.getName());
		this.setFileUsingEditor(file);
		
	}
	

}
