/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.contestant;

import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.gui.MyTableItemLayout;
import de.tudarmstadt.yajs.gui.MyTableModel;
import de.tudarmstadt.yajs.gui.ResultPrinter;
import de.tudarmstadt.yajs.gui.contestant.events.RefreshResultsEvent;
import de.tudarmstadt.yajs.gui.contestant.events.RefreshResultsEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.RefreshResultsEventListener;
import de.tudarmstadt.yajs.lib.Result;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
/**
 * A panel that shows judgements in a table.
 * @author  markus
 */
public class ResultPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -2041538246200305088L;
	/** The result model, the interface to query for judgements. */
	private ResultModel resultmodel;
	private RefreshResultsEventGenerator refreshresultssignal;
	private JButton refreshbutton;

	protected String problemString=Lang.getString("Problem");
	protected String languageString=Lang.getString("Language");
	protected String timeString=Lang.getString("Time");
	protected String judgementString=Lang.getString("Judgement");

	/** Initialization method that creates the panel. */
	private void buildPanel()
	{
		/* Initialize layout (grid bag layout) */
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gbl);
		
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		
		gbc.weightx = gbc.weighty = 1.0;
		
		/* initialize the result table */
		
		MyTableItemLayout itemlayout = new MyTableItemLayout() {

			public int getColumnCount() {
				return 5;
			}

			public String getColumnName(int col) {
				switch(col)
				{
					case 0: return "#";
					case 1: return problemString;
					case 2: return languageString;
					case 3: return timeString;
					case 4: return judgementString;
				}
				return "";
			}

			public Object getValue(Object item, int col) {
				Result r = (Result) item;
				switch(col)
				{
					case 0: return new Long(r.getIndex());
					case 1: return r.getProblem().getID() + " - " + r.getProblem().getName();
					case 2: return r.getLanguage().getName();
					case 3: return new Long(r.getTime());
					case 4: return ResultPrinter.getVerdict(r); 
					default: return Lang.getString("Pleasesendbugreport");
				}
			}
			
		};
		
		MyTableModel runsmodel = new MyTableModel( getModel().getResults(), itemlayout );
		JTable runstable = new JTable( runsmodel );
		runstable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane runsscrollpane = new JScrollPane(runstable);
		
		runsscrollpane.setBorder(BorderFactory.createTitledBorder(Lang.getString("Submissionoverview")));
		
		gbl.setConstraints(runsscrollpane, gbc);
		
		add(runsscrollpane);
		
		/* Add buttons */
		
		gbc.weightx = gbc.weighty = 0.0;
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new FlowLayout(FlowLayout.TRAILING));
		refreshbutton = new JButton(Lang.getString("refresh"));
		buttons.add(refreshbutton);
		refreshbutton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				refreshresultssignal.emitEvent( new RefreshResultsEvent(ResultPanel.this) );
			}
		});
		gbl.setConstraints(buttons, gbc);
		add(buttons);
	}
	
	
	/** Retrieve the underlying ResultModel.
	 * @return Returns the result model.
	 */
	private ResultModel getModel() {
		return resultmodel;
	}

	/** Construct a ResultPanel.
	 * @param resultmodel The result model to use.
	 */
	public ResultPanel(ResultModel resultmodel) {
		super();
		this.resultmodel = resultmodel;
		this.refreshresultssignal = new RefreshResultsEventGenerator();
		buildPanel();
	}
		
	/** register an EventListener for RefreshResultEvents.
	 * @param l A listener.
	 */
	public void addRefreshResultsListener(RefreshResultsEventListener l) {
		refreshresultssignal.registerListener(l);
	}


	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void updateRuntimeGUILanguage() {
		problemString=Lang.getString("Problem");
		languageString=Lang.getString("Language");
		timeString=Lang.getString("Time");
		judgementString=Lang.getString("Judgement");
		refreshbutton.setText(Lang.getString("refresh"));
	}


	



}
