/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.contestant;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.ContestStatusPanel;
import de.tudarmstadt.yajs.gui.ResultPrinter;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEvent;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEvent;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.FileSelectionEvent;
import de.tudarmstadt.yajs.gui.contestant.events.FileSelectionEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.FileSelectionEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEvent;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEvent;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEvent;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.TestEvent;
import de.tudarmstadt.yajs.gui.contestant.events.TestEventGenerator;
import de.tudarmstadt.yajs.gui.contestant.events.TestEventListener;
import de.tudarmstadt.yajs.lib.ContestStatus;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.Result;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * A panel that allows selecting files for submission. Displays a list of <code>ProblemPanel</code>s together with a status line and a bunch of buttons.
 * @author  markus
 */
public class SubmitPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8004056866409552708L;
	/** The status panel. */
	private ContestStatusPanel statusPanel;
	/** The panel containing the problem panels. */
	private JPanel problemsPanel;
	/** The submit button. */
	private JButton submitbutton;
	/** The submit model. */
	private SubmitModel submitmodel;
	/** A map from <code>Problem</code>s to <code>ProblemPanel</code>s. */
	private Map problemsToPanels;

	/** Getter for the problemPanel for a specific problem. */
	protected ProblemPanel getProblemPanel(Problem p) {
		return (ProblemPanel) problemsToPanels.get(p);
	}
	
	protected SubmitEventGenerator submitsignal = new SubmitEventGenerator();
	protected RefetchEventGenerator refetchSignal = new RefetchEventGenerator();
	protected ManualTestEventGenerator mantestSignal = new ManualTestEventGenerator();
	protected TestEventGenerator testSignal = new TestEventGenerator();
	protected EditorEventGenerator editorSignal = new EditorEventGenerator();
	protected FileSelectionEventGenerator fileSelectionSignal = new FileSelectionEventGenerator();
	protected CreateTemplateSrcEventGenerator createTemplateSrcSignal= new CreateTemplateSrcEventGenerator();
	private boolean isOnline;
	
	/** Initialization method to build the problem panels using <code>panel</code> as the container. 
	 * @param panel The container panel.
	 */
	private JPanel buildProblemPanels(JPanel panel)
	{
		Iterator i = submitmodel.getProblems().iterator();
		int row = 0;
		while(i.hasNext()) {
			final Problem problem = (Problem) i.next();
			{
				GridBagConstraints constraints = new GridBagConstraints();
				constraints.gridx = 0;
				constraints.gridy = row;
				constraints.weightx = 1;
				constraints.fill = GridBagConstraints.BOTH;
				ProblemPanel pp = new ProblemPanel(problem, submitmodel,isOnline);
			//	pp.setContestStatus(statusPanel.getStatusID().equals("isRunning"));
				pp.addVetoableChangeListener(new VetoableChangeListener() {
					public void vetoableChange(PropertyChangeEvent arg0) throws PropertyVetoException {
						if(arg0.getPropertyName().equals("file") && arg0.getNewValue() != null) {
							fileSelectionSignal.emitFileSelection(new FileSelectionEvent(SubmitPanel.this, (File) arg0.getNewValue(), problem));	
						}
					}		
				});
				pp.addSubmitEventListener(new SubmitEventListener() {
					public void submitProblem(SubmitEvent e) {
						submitsignal.emitSubmitProblem(e);
					}
				});
				pp.addRefetchEventListener(new RefetchEventListener() {
					public void refetch(RefetchEvent event) {
						refetchSignal.emitRefetch(event);
					}
				});
				pp.addTestEventListener(new TestEventListener() {
					public void test(TestEvent event) {
						testSignal.emitTest(event);
					}
				});
				pp.addManualTestEventListener(new ManualTestEventListener() {
					public void test(ManualTestEvent event) {
						mantestSignal.emitTest(event);
					}
				});
				pp.addCreateTemplateSrcEventListener(new CreateTemplateSrcEventListener() {
					public void test(CreateTemplateSrcEvent event) {
						createTemplateSrcSignal.emitTest(event);
					}
				});
				pp.addEditorEventListener(new EditorEventListener() {
					public void edit(EditorEvent event) {
						editorSignal.emitEditor(event);
					}
				});
				pp.setBorder(BorderFactory.createTitledBorder(problem.getID() + " - " + problem.getName()));
				problemsToPanels.put(problem, pp);
				panel.add(pp, constraints);
			}
			++row;
		}
		return panel;
	}
	
	/** Initialization method that creates the panel. */
	private void buildPanel()
	{
		setLayout(new GridBagLayout());
		
		GridBagConstraints statusConstraints = new GridBagConstraints();
		GridBagConstraints problemConstraints = new GridBagConstraints();
		
		statusConstraints.gridx = 0;
		statusConstraints.gridy = 0;
		statusConstraints.fill = GridBagConstraints.BOTH;
		statusConstraints.weightx = 1;
		statusConstraints.weighty = 0;
		
		problemConstraints.gridx = 0;
		problemConstraints.gridy = 1;
		problemConstraints.fill = GridBagConstraints.BOTH;
		problemConstraints.weightx = 1;
		problemConstraints.weighty = 1;
		
		add(getStatusPanel(), statusConstraints);
		JScrollPane scrollPane=new JScrollPane(getProblemsPanel());
		add(scrollPane, problemConstraints);
		
		
	}
	
	/**
	 * Getter for the panel containing the problem panels.
	 * @return  Returns a <code>JPanel</code> containing the problem panels.
	 * @uml.property  name="problemsPanel"
	 */
	private JPanel getProblemsPanel() {
		if(problemsPanel == null) {
			problemsPanel = new JPanel();
			problemsPanel.setLayout(new GridBagLayout());
			buildProblemPanels(problemsPanel);
		}
		return problemsPanel;
	}
	
	public int getProblemNumber(){
	
		return submitmodel.getProblems().size();
	
	}
	/**
	 * Getter for the contest status panel. 
	 * @return  Returns the <code>ContestStatusPanel</code>.
	 * @uml.property  name="statusPanel"
	 */
	protected ContestStatusPanel getStatusPanel() {
		if(statusPanel == null) {
			statusPanel = new ContestStatusPanel();
		}
		return statusPanel;
	}
	
	/** Constructor. */
	public SubmitPanel(SubmitModel submitmodel)
	{	
		this.submitmodel = submitmodel;
		this.problemsToPanels = new HashMap();
		buildPanel();
	}
	public SubmitPanel(SubmitModel submitmodel,boolean isOnline)
	{	
		this(submitmodel);
		this.isOnline=isOnline;
	
	}
	
	/** Getter for the <code>SubmitModel</code>.
	 * @return Returns the submit model.
	 */
	private SubmitModel getModel() {
		return submitmodel;
	}

	/** Registers a SubmitEventListener that will be called back whenever the user wants to submit a file.
	 * @param l The listener to register.
	 */
	public void addSubmitListener(SubmitEventListener l) {
		submitsignal.registerListener(l);
	}

	/** Registers a RefetchEventListener that will be called back whenever the user wants to refetch test data.
	 * <b>REFACTOR</b> Rename to adopt the name RefetchEventListener? Change name?
	 * @param l The listener to register.
	 */
	public void addRefetchListener(RefetchEventListener l) {
		refetchSignal.registerListener(l);
	}
	
	/** Registers a TestEventListener that will be called back whenever the user wants to test a file.
	 * @param listener The listener to register.
	 */
	public void addTestEventListener(TestEventListener listener) {
		testSignal.registerListener(listener);
	}
	/** Registers a TestEventListener that will be called back whenever the user wants to test a file.
	 * @param listener The listener to register.
	 */
	public void addManualTestEventListener(ManualTestEventListener listener) {
		mantestSignal.registerListener(listener);
	}
	
	/** Register a EditorEventListener.
	 * @param  listener The listener to register       
	 */
	public void addEditorEventListener(EditorEventListener listener) {
		editorSignal.registerListener(listener);
	}
	/** Update the problemPanels due to a change in the problem set.
	 * Reconstructs the panel containing the problem panels.
	 */
	public void updateProblemPanels() {
		getProblemsPanel().removeAll();
		buildProblemPanels(getProblemsPanel());
		revalidate();
	}
	
	/** Update a result as a judgement was received. 
	 * Sets the text displayed in the corresponding problem panel.
	 */
	public void updateResult(Result result) {
		String text = ResultPrinter.getVerdict(result);
		ProblemPanel panel = (ProblemPanel) problemsToPanels.get(result.getProblem());
		if(panel == null) {
			System.err.println(Lang.getString("SumitPanel10") + result.getProblem().getID() + " - " + result.getProblem().getName() + "\n" + text);
			return;
		}
		panel.setJudgementText(text);
	}

	/** Enables or disables fetching of test data for a given problem.
	 * @param problem The problem.
	 * @param b True if fetching should be enabled, false otherwise.
	 */
	public void enableFetching(Problem problem, boolean b) {
		ProblemPanel panel = getProblemPanel(problem);
		if(panel == null) {
			System.err.println("submitpanel11" + problem.getID());
			return;
		}
		panel.enableFetching(b);
	}

	/** Registers a FileSelectionEventListener that will be called back whenever a file is selected by the user.
	 * @param listener The listener to register.
	 */
	public void addFileSelectionEventListener(FileSelectionEventListener listener) {
		fileSelectionSignal.registerListener(listener);
	}

	/** Set the language for a given problem.
	 * @param problem The problem to set the language for. 
	 * @param language The language to set.
	 */
	public void setLanguageForProblem(Problem problem, Language language) {
		ProblemPanel panel = getProblemPanel(problem);
		panel.setSubmissionLanguage(language);
	}

	/** Update the contest status. Shows the updated status in the status panel.
	 * @param contestStatus The current contest status.
	 */
	public void updateContestStatus(ContestStatus contestStatus) {
		getStatusPanel().updateContestStatus(contestStatus);
		Iterator i = submitmodel.getProblems().iterator();
		while(i.hasNext()){
			final Problem problem = (Problem) i.next();
			getProblemPanel(problem).setContestStatus(statusPanel.getStatusID().equals("isRunning"));
			
		}
		
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void updateRuntimeGUILanguage() {
		Iterator i = submitmodel.getProblems().iterator();
		while(i.hasNext()){
			final Problem problem = (Problem) i.next();
			getProblemPanel(problem).updateRuntimeGUILanguage();
		}
		
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void addCreateTemplateSrcEventListener(CreateTemplateSrcEventListener listener) {
		createTemplateSrcSignal.registerListener(listener);
		
	}

	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void setFileForEditor(Problem problem,File file) {
		ProblemPanel panel = getProblemPanel(problem);
		panel.setFileForEditor(file);
		
	}
}
