/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.gui.contestant;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.DiffPanel;
import de.tudarmstadt.yajs.gui.ManualOutputPanel;
import de.tudarmstadt.yajs.gui.Predicate;
import de.tudarmstadt.yajs.gui.ResultPrinter;
import de.tudarmstadt.yajs.gui.contestant.events.CreateTemplateSrcEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.EditorEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.FileSelectionEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.ManualTestEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.RefetchEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.RefreshResultsEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.SubmitEventListener;
import de.tudarmstadt.yajs.gui.contestant.events.TestEventListener;
import de.tudarmstadt.yajs.gui.judge.XMLDisplayPanel;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshScoreboardEventListener;
import de.tudarmstadt.yajs.lib.ContestStatus;
import de.tudarmstadt.yajs.lib.Language;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.Result;
import de.tudarmstadt.yajs.lib.Status;
import de.tudarmstadt.yajs.tester.Diff;
/**
 * The user interface for a contestant (i.e. for <em>turmeric</em>).
 * @author  markus
 */
public class YAJSContestantUI extends JFrame {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -163940117060706042L;
	private SubmitPanel submitPanel;
	private ResultPanel resultPanel;
	private AboutPanel aboutPanel;
	private ContestantUIModel model;
	private XMLDisplayPanel scoreboardPanel;
	private RefreshScoreboardEventGenerator scoreboardRefreshSignal = new RefreshScoreboardEventGenerator();
	private ClarificationsPanel clarificationsPanel;
	private JRadioButton language1,language2;
	private JMenuBar menuPanel;
	protected RefreshClarificationsEventGenerator refreshClarificationsSignal = new RefreshClarificationsEventGenerator();
	protected AskClarificationEventGenerator askClarificationSignal = new AskClarificationEventGenerator();
	private boolean isOnline;
	private JTabbedPane tp;
	

	
	/** Constructor. */
	public YAJSContestantUI(ContestantUIModel model, boolean isOnline)
	{
		super();
		this.model = model;
		this.isOnline=isOnline;
		aboutPanel=new AboutPanel();
		
		submitPanel = new SubmitPanel( model, isOnline );
		submitPanel.setVisible(isOnline);
		resultPanel = new ResultPanel( model );
			
		scoreboardPanel = new XMLDisplayPanel(Lang.getString("scoreboard.xsl")); //$NON-NLS-1$
		scoreboardPanel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				scoreboardRefreshSignal.emitRefresh( new RefreshScoreboardEvent(e.getSource()));
			}
		});
	
		this.clarificationsPanel = new ClarificationsPanel(model, Predicate.alwaysTrue);
		clarificationsPanel.addAskClarificationEventListener(new AskClarificationEventListener() {
			public void askClarification(AskClarificationEvent e) {
				askClarificationSignal.emitAskClarification(e);
			}
		});	
		clarificationsPanel.addRefreshClarificationsEventListener(new RefreshClarificationsEventListener() {
			public void refreshClarifications(RefreshClarificationsEvent e) {
				refreshClarificationsSignal.emitRefreshClarifications(e);
			}
		});
		tp = new JTabbedPane();
		
		tp.addTab(Lang.getString("About"), null, aboutPanel, Lang.getString("Informationaboutrefefree")); //$NON-NLS-1$ //$NON-NLS-2$
		tp.addTab(Lang.getString("Submit"), null, submitPanel, Lang.getString("Submitsolutionstotheproblems")); //$NON-NLS-1$ //$NON-NLS-2$
		
		if (isOnline){
			tp.addTab(Lang.getString("SubmissionHistory"), null, resultPanel, Lang.getString("Viewpreviousandpendingruns")); //$NON-NLS-1$ //$NON-NLS-2$
			tp.addTab(Lang.getString("Scoreboard"), null, scoreboardPanel, Lang.getString("Viewthescoreboard")); //$NON-NLS-1$ //$NON-NLS-2$
			tp.addTab(Lang.getString("Clarifications"), null, clarificationsPanel, Lang.getString("Viewclarifications")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	
		tp.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if(tp.getSelectedComponent() == scoreboardPanel)
					scoreboardRefreshSignal.emitRefresh(new RefreshScoreboardEvent(scoreboardPanel));
			}
		});
		menuPanel=new JMenuBar();
		setJMenuBar(menuPanel);
		language1=new JRadioButton(Lang.getString("YAJSContestantUI.9")); //$NON-NLS-1$
		language2=new JRadioButton(Lang.getString("YAJSContestantUI.10")); //$NON-NLS-1$
		language1.setSelected(Lang.isCurrentLanguage(language1.getText()));
		language2.setSelected(Lang.isCurrentLanguage(language2.getText()));
		addLanguage1ActionListener();
		addLanguage2ActionListener();
		int problemCount=submitPanel.getProblemNumber();
		
		//111+111*problemcount;
		setPreferredSize(new Dimension(1000,120+120*problemCount));
		menuPanel.add(language1);
		menuPanel.add(language2);
	
		getContentPane().add(tp);
	
		pack();
	}
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	private void addLanguage1ActionListener() {
		language1.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				language2.setSelected(false);
				setRuntimeGUILanguage(language1.getText());
				
			}
		});
		
	}
	private void addLanguage2ActionListener() {
		language2.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				language1.setSelected(false);
				setRuntimeGUILanguage(language2.getText());
				
			}
		});
		
	}
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	protected void setRuntimeGUILanguage(String language) {
		Lang.setLanguage(language);
		language1.setText(Lang.getString("YAJSContestantUI.9")); //$NON-NLS-1$
		language2.setText(Lang.getString("YAJSContestantUI.10")); //$NON-NLS-1$
		tp.setTitleAt(0, Lang.getString("About"));
		tp.setTitleAt(1, Lang.getString("Submit"));
		
		if (isOnline){
			tp.setTitleAt(2, Lang.getString("SubmissionHistory"));
			tp.setTitleAt(3, Lang.getString("Scoreboard"));
			tp.setTitleAt(4, Lang.getString("Clarifications"));
		
		}
		submitPanel.updateRuntimeGUILanguage();
		resultPanel.updateRuntimeGUILanguage();
		scoreboardPanel.updateRuntimeGUILanguage();
		clarificationsPanel.updateRuntimeGUILanguage();
	}
	/** Registers a SubmitEventListener that is called back whenever the user wants to submit a file.
	 * @param l The listener to register.
	 */		
	public void addSubmitListener(SubmitEventListener l)
	{
		submitPanel.addSubmitListener(l);
	}

	/** Registers a RefetchEventListener that is called back whenever the user wants to refetch test data. 
	 * <b>REFACTOR</b> The naming makes only little sense.
	 * @param l The listener to register.
	 */
	public void addRefetchListener(RefetchEventListener l)
	{
		submitPanel.addRefetchListener(l);
	}
	/** Registers a RefreshResultsEventListener that is called back whenever the user wants to refresh the list of submissions manually.
	 * This is only needed if the server does not provide us with that information automatically.
	 * @param l The listener to register.
	 */
	public void addRefreshResultsListener(RefreshResultsEventListener l)
	{
		resultPanel.addRefreshResultsListener(l);
	}
	/** Registers a FileSelectionEventListener that is called back whenever the user has selected a file.
	 * @param listener The listener to register.
	 */
	public void addFileSelectionEventListener(FileSelectionEventListener listener) {
		submitPanel.addFileSelectionEventListener(listener);
	}
	
	/** Registers a RefreshScoreboardEventListener that is called back whenever the user wants to refresh the scoreboard.
	 * @param listener The listener to register.
	 */
	public void addRefreshScoreboardListener(RefreshScoreboardEventListener listener) {
		scoreboardRefreshSignal.registerListener(listener);
	}
	
	/** Registers an AskClarificationEventListener that is called back whenever the user wants to ask for clarification.
	 * @param listener The listener to register.
	 */
	public void addAskClarificationEventListener(AskClarificationEventListener listener) {
		askClarificationSignal.registerListener(listener);
	}

	/** Registers a RefreshClarificationsEventListener that is called back whenever the user wants to refresh the list of clarifications.
	 * Usually this information is provided by the server automatically.
	 * @param listener The listener to register.
	 */
	public void addRefreshClarificationsEventListener(RefreshClarificationsEventListener listener) {
		refreshClarificationsSignal.registerListener(listener);
	}
	
	public void addEditorEventListener(EditorEventListener listener) {
		submitPanel.addEditorEventListener(listener);
	}
	
	/** Registers a TestEventListener that is called back whenever the user wants to test a file.
	 * @param listener The listener to register.
	 */
	public void addTestEventListener(TestEventListener listener) {
		submitPanel.addTestEventListener(listener);
	}
	/** Registers a ManualTestEventListener that is called back whenever the user wants to test a file manually.
	 * @param listener The listener to register.
	 */
	public void addManualTestEventListener(ManualTestEventListener listener) {
		submitPanel.addManualTestEventListener(listener);
	}
	/** Show an error Message.
	 * @param string The message to show.
	 */
	public void showErrorMessage(String string) {
		JOptionPane.showMessageDialog(this, string, "Error", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
	}
	
	/** Update the result information displayed in the submit panel.
	 * @param result The updated result.
	 */
	public void updateResultText(Result result) {
		submitPanel.updateResult(result);
	}
	
	/** Show an updated result. Pops up a message box displaying the judges' verdict.
	 * @param result The result to show.
	 */
	public void showUpdatedResult(Result result) {
		String message0, title0;
		if(result.getStatus() == Status.NEW) {
			message0 = Lang.getString("Submissionreceivedforproblem") +  //$NON-NLS-1$
				result.getProblem();
			title0 = Lang.getString("Submissionreceivedafter") + result.getTime() + Lang.getString("minutes"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			message0 = Lang.getString("Newjudgementforproblem") +  //$NON-NLS-1$
				result.getProblem() + ":\n\n" + //$NON-NLS-1$
				ResultPrinter.getVerdict(result);
			title0 = Lang.getString("Updatedresult");			 //$NON-NLS-1$
		}
		final String message = message0, title = title0;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(YAJSContestantUI.this, message, title, JOptionPane.INFORMATION_MESSAGE);				
			}
		});
	}

	/** Update the problem panels due to a change in the problem set.
	 * Might be called if the server overrides local problem settings.
	 */
	public void updateProblemPanels() {
		submitPanel.updateProblemPanels();
	}
	
	/** Update the scoreboard.
	 * @param scoreboard The scoreboard as received from the server, an XML document.
	 */
	public void updateScoreboard(String scoreboard) {
		scoreboardPanel.setText(scoreboard);
	}

	/** Enable or disable fetching of test-data for a given problem.
	 * Needed when fetching is actually in progress so that the user does not fetch the same data five or six times.
	 * @param problem The problem.
	 * @param b True if fetching should be enabled, false otherwise.
	 */
	public void enableFetching(Problem problem, boolean b) {
		submitPanel.enableFetching(problem, b);
	}

	/** Display the results of a test.
	 * This will open a new dialog containing a <code>DiffPanel</code> and the test error output.
	 * @param problem The problem for which the test was run.
	 * @param theDiff The difference between output and expected output that the test produced.
	 * @param error The output on stderr that the test produced.
	 * @param suggestion The suggestion line the test produced.
	 */
	public void displayTestResult(final Problem problem,final File file, final Language language, final Diff theDiff, final String error, String suggestion) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				final JDialog dlg = new JDialog();
				Container content = dlg.getContentPane();
			//	content.setSize(new Dimension(1000,1000));		
				JEditorPane stdErr = new JEditorPane();
				stdErr.setAutoscrolls(true);
				stdErr.setEditable(false);
				stdErr.setText(error);
				
				JTabbedPane tab = new JTabbedPane();
				tab.add(Lang.getString("stdoutfirstdifference"), new DiffPanel(theDiff.getOutput(), theDiff.getExpected())); //$NON-NLS-1$
				tab.add(Lang.getString("stderr"), stdErr); //$NON-NLS-1$

				// No difference? Show stderr:
				if(theDiff.getOutput().equals("") && theDiff.getExpected().equals("")) //$NON-NLS-1$ //$NON-NLS-2$
					tab.setSelectedIndex(1);
				if (error.contains("error")||error.contains("RUNTIME"))
					tab.setSelectedIndex(1);
				JButton editButton=new JButton(Lang.getString("editor"));
				editButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						String EDITORCOMMAND= language.getEditor(); //$NON-NLS-1$
						String PROBLEMCOMMAND=file.getPath();
						String[] command = new String[] { EDITORCOMMAND, PROBLEMCOMMAND};
						try {
							Runtime.getRuntime().exec(command);							
							
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}	
					}
				});
				JButton closeButton = new JButton(Lang.getString("close")); //$NON-NLS-1$
				closeButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						dlg.dispose();
					}
				});
				JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
				buttonPanel.add(closeButton);
				buttonPanel.add(editButton);
				content.setLayout(new GridBagLayout());
				
				GridBagConstraints c1 = new GridBagConstraints();
				GridBagConstraints c2 = new GridBagConstraints();
				
				c1.gridx = 0;
				c1.gridy = 0;
				c1.fill = GridBagConstraints.BOTH;
				c1.weightx = 1;
				c1.weighty = 1;
				
				c2.gridx = 0;
				c2.gridy = 1;
				c2.fill = GridBagConstraints.BOTH;
				c2.weightx = 1;
				c2.weighty = 0;

				content.add(tab, c1);
				content.add(buttonPanel, c2);
				
				dlg.setLocationRelativeTo(YAJSContestantUI.this);
				dlg.setTitle(Lang.getString("Testresultsforproblem") + problem.getID()); //$NON-NLS-1$
				dlg.setSize(500,400);
				dlg.show();
			}	
		});
	}
	

	/** Set the programming language for a given problem.
	 * (This information has been determined by the application class).
	 * <b>REFACTOR</b> Should we keep this in the application class? It does not seem to belong here.
	 * @param problem The problem for which to set the language.
	 * @param language The language to set.
	 */
	public void setLanguageForProblem(Problem problem, Language language) {
		submitPanel.setLanguageForProblem(problem, language);
	}
	/** Set the programming language for a given problem.
	 * (This information has been determined by the application class).
	 * <b>REFACTOR</b> Should we keep this in the application class? It does not seem to belong here.
	 * @param problem The problem for which to set the language.
	 * @param language The language to set.
	 */
	public void setFileForEditor(Problem problem,File file) {
		submitPanel.setFileForEditor(problem,file);
	}

	/** Update the contest status.
	 * @param contestStatus The current contest status.
	 */
	public void updateContestStatus(ContestStatus contestStatus) {
		submitPanel.updateContestStatus(contestStatus);
	}
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void displayManualTestResult(final Problem problem,final File file,final Language language, final String themOutput, final String error, String suggestion) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				final JDialog dlg = new JDialog();
				Container content = dlg.getContentPane();
						
				JEditorPane stdErr = new JEditorPane();
				stdErr.setAutoscrolls(true);
				stdErr.setEditable(false);
				stdErr.setText(error);
				
				JTabbedPane tab = new JTabbedPane();
				tab.add("Manual output", new ManualOutputPanel(themOutput)); //$NON-NLS-1$
				tab.add(Lang.getString("stderr"), stdErr); //$NON-NLS-1$

				
				if(themOutput.equals("") ) //$NON-NLS-1$
					tab.setSelectedIndex(1);
				if (error.contains("error")||error.contains("RUNTIME"))
					tab.setSelectedIndex(1);
				JButton editButton=new JButton(Lang.getString("editor"));
				editButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						String EDITORCOMMAND= language.getEditor(); //$NON-NLS-1$
						String PROBLEMCOMMAND=file.getPath();
						String[] command = new String[] { EDITORCOMMAND, PROBLEMCOMMAND};
						try {
							Runtime.getRuntime().exec(command);							
							
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}	
					}
				});
				JButton closeButton = new JButton(Lang.getString("close")); //$NON-NLS-1$
				closeButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						dlg.dispose();
					}
				});
				JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
				buttonPanel.add(closeButton);
				buttonPanel.add(editButton);
				
				content.setLayout(new GridBagLayout());
				
				GridBagConstraints c1 = new GridBagConstraints();
				GridBagConstraints c2 = new GridBagConstraints();
				
				c1.gridx = 0;
				c1.gridy = 0;
				c1.fill = GridBagConstraints.BOTH;
				c1.weightx = 1;
				c1.weighty = 1;
				
				c2.gridx = 0;
				c2.gridy = 1;
				c2.fill = GridBagConstraints.BOTH;
				c2.weightx = 1;
				c2.weighty = 0;

				content.add(tab, c1);
				content.add(buttonPanel, c2);
				
				dlg.setLocationRelativeTo(YAJSContestantUI.this);
				dlg.setTitle(Lang.getString("Testresultsforproblem") + problem.getID()); //$NON-NLS-1$
				dlg.setSize(500,400);
				dlg.show();
			}	
		});
	}
	/**
	 * @param  
	 * @return      
	 * @see         
	 */
	public void addCreateTemplateSrcEventListener(CreateTemplateSrcEventListener listener) {
		submitPanel.addCreateTemplateSrcEventListener(listener);
		
	}
	
	
}
