/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 01.06.2005
 *
 */
package de.tudarmstadt.yajs.gui.contestant.events;

import java.beans.PropertyVetoException;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author markus
 *
 */
public class FileSelectionEventGenerator {
	private HashSet listeners = new HashSet();

	/** Registers a listener.
	 * @param listener The listener to register.
	 */
	public synchronized void registerListener(FileSelectionEventListener listener) {
		listeners.add(listener);
	}

	/** Unregisters a listener.
	 * @param listener The listener to unregister.
	 */
	public synchronized void unregisterListener(FileSelectionEventListener listener) {
		listeners.remove(listener);
	}
	

	/** Return a copy of the listeners set so we can iterate over it.
	 * @return A copy of the set listeners.
	 */
	private synchronized HashSet getListenersCopy() { 
		return new HashSet(listeners);
	}

	/**
	 * @param event
	 * @throws PropertyVetoException
	 */
	public void emitFileSelection(FileSelectionEvent event) throws PropertyVetoException {
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			FileSelectionEventListener lis = (FileSelectionEventListener) i.next();
			lis.fileSelected(event);
		}
	}

///
}
