/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 08.05.2005
 *
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationEventListener;
import de.tudarmstadt.yajs.lib.ClarificationAnswer;
import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.PostedClarificationRequest;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
/**
 * A dialog that allows answering clarification requests.
 * @author  markus
 */
public class ClarificationDialog extends JDialog {
	
	private PostedClarificationRequest request = null;
	private ClarificationAnswer previous = null;
	private javax.swing.JPanel jContentPane = null;
	private JPanel jPanel = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	private JButton resetButton = null;
	private JScrollPane jScrollPane = null;
	private JEditorPane clarificationTextArea = null;
	private ClarificationEventGenerator clarificationSignal = new ClarificationEventGenerator();
	private ButtonGroup towhomGroup = null;
	private JPanel towhomPanel = null;
	private JRadioButton toEveryoneButton = null;
	private JRadioButton toQuestionerButton = null;
	
	/**
	 * This is the default constructor
	 */
	public ClarificationDialog(PostedClarificationRequest request, ClarificationAnswer previous) {
		super();
		setTitle(Lang.getString("Answeringclarificationrequest") + request.getIndex());
		this.request = request;
		this.previous = previous;
		this.towhomGroup = new ButtonGroup();
		initialize();
		reset();
		wire();
	}
	
	/** Reset all fields to initial values.
	 * Sets the clarification text to the request, followed by the previous answer if there is one.
	 * The request is prefixed by 'Q:', the answer is prefixed by 'A:'.
	 */
	protected void reset() {
		if(previous.getState() == ClarificationAnswerState.UNANSWERED) {
			clarificationTextArea.setText("Q: " + request.getContent() + "\n\nA: ");
			toQuestionerButton.setSelected(true);
		} else {
			clarificationTextArea.setText(previous.getAnswer());
			if(previous.getState() == ClarificationAnswerState.EVERYONE)
				toEveryoneButton.setSelected(true);
			else toQuestionerButton.setSelected(true);
		}
	}
	
	/** Event wiring.
	 */
	private void wire() {
		resetButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				reset();	
			}
		});
	
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				clarificationSignal.emitUnchanged(new ClarificationEvent(ClarificationDialog.this, previous));
				ClarificationDialog.this.dispose();
			}
		});	
		
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				clarificationSignal.emitChanged(new ClarificationEvent(ClarificationDialog.this, getClarificationAnswer()));
				ClarificationDialog.this.dispose();
			}
		});
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent arg0) {
				clarificationSignal.emitUnchanged(new ClarificationEvent(ClarificationDialog.this, previous));
			}
		});
	}
	/** Get the answer to the clarification request determined by the input fields of this dialog.
	 * @return Returns a ClarificationAnswer corresponding to the user's inputs.
	 */
	protected ClarificationAnswer getClarificationAnswer() {
		return new ClarificationAnswer(request.getIndex(), getState(), getText());
	}
	/** Get the ClarificationsAnswerState determined by the user's input.
	 * @return Returns the state corresponding to the user's input.
	 */
	private ClarificationAnswerState getState() {
		if(getToEveryoneButton().isSelected())
			return ClarificationAnswerState.EVERYONE;
		else if(getToQuestionerButton().isSelected())
			return ClarificationAnswerState.QUESTIONER;
		return ClarificationAnswerState.UNANSWERED;
	}
	/** Get the clarification text specified in the input field.
	 * @return Returns the text in the clarificationTextArea.
	 */
	private String getText() {
		return clarificationTextArea.getText();
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300,200);
		this.setContentPane(getJContentPane());
	}
	/**
	 * This method initializes jContentPane
	 * @return  javax.swing.JPanel
	 * @uml.property  name="jContentPane"
	 */
	private javax.swing.JPanel getJContentPane() {
		if(jContentPane == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(new GridBagLayout());
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 2;
			gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.weightx = 1.0;
			gridBagConstraints5.weighty = 1.0;
			gridBagConstraints5.fill = java.awt.GridBagConstraints.BOTH;
			jContentPane.add(getJPanel(), gridBagConstraints2);
			jContentPane.add(getTowhomPanel(), gridBagConstraints1);
			jContentPane.add(getJScrollPane(), gridBagConstraints5);
		}
		return jContentPane;
	}
	
	/**
	 * This method initializes toWhomPanel
	 * @return  javax.swing.JPanel
	 * @uml.property  name="towhomPanel"
	 */
	private JPanel getTowhomPanel() {
		if(towhomPanel == null) {
			towhomPanel = new JPanel();
			towhomPanel.setLayout(new GridLayout(1,2));
			towhomPanel.setBorder(BorderFactory.createTitledBorder(Lang.getString("Sendanswerto...")));
			towhomPanel.add(getToEveryoneButton());
			towhomPanel.add(getToQuestionerButton());
		}
		return towhomPanel;
	}

	/**
	 * This method initializes jPanel	
	 * @return  javax.swing.JPanel
	 * @uml.property  name="jPanel"
	 */    
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new FlowLayout(FlowLayout.TRAILING));
			if(previous != null) {
				jPanel.add(getResetButton());	
			}
			jPanel.add(getCancelButton());
			jPanel.add(getOkButton());
		}
		return jPanel;
	}
	/**
	 * This method initializes jButton	
	 * @return  javax.swing.JButton
	 * @uml.property  name="okButton"
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Lang.getString("OK"));
		}
		return okButton;
	}
	/**
	 * This method initializes jButton1	
	 * @return  javax.swing.JButton
	 * @uml.property  name="cancelButton"
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Lang.getString("Cancel"));
		}
		return cancelButton;
	}
	/**
	 * This method initializes jButton1	
	 * @return  javax.swing.JButton
	 * @uml.property  name="resetButton"
	 */    
	private JButton getResetButton() {
		if (resetButton == null) {
			resetButton = new JButton();
			resetButton.setText(Lang.getString("Reset"));
		}
		return resetButton;
	}
	/**
	 * This method initializes jScrollPane	
	 * @return  javax.swing.JScrollPane
	 * @uml.property  name="jScrollPane"
	 */    
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getClarificationTextArea());
		}
		return jScrollPane;
	}
	/**
	 * This method initializes clarificationTextArea	
	 * @return  javax.swing.JTextArea
	 * @uml.property  name="clarificationTextArea"
	 */    
	private JEditorPane getClarificationTextArea() {
		if (clarificationTextArea == null) {
			clarificationTextArea = new JEditorPane();
			
		}
		return clarificationTextArea;
	}

	/**
	 * Return the radio button that is used to signal that the answer should be sent to everyone.
	 * @return  Returns this radio button.
	 * @uml.property  name="toEveryoneButton"
	 */
	private JRadioButton getToEveryoneButton() {
		if(toEveryoneButton == null) {
			toEveryoneButton = new JRadioButton(Lang.getString("Everyone"));
			getTowhomGroup().add(toEveryoneButton);
		}
		return toEveryoneButton;
	}
	
	/**
	 * Get the button group that groups the "send back to" buttons.
	 * @return  Returns this button group.
	 * @uml.property  name="towhomGroup"
	 */
	private ButtonGroup getTowhomGroup() {
		if(towhomGroup == null) {
			towhomGroup = new ButtonGroup();
		}
		return towhomGroup;
	}

	/**
	 * Return the radio button that is used to signal that the answer should be sent to everyone.
	 * @return  Returns this radio button.
	 * @uml.property  name="toQuestionerButton"
	 */
	private JRadioButton getToQuestionerButton() {
		if(toQuestionerButton == null) {
			toQuestionerButton = new JRadioButton(Lang.getString("Questioner"));
			getTowhomGroup().add(toQuestionerButton);
		}
		return toQuestionerButton;
	}
	
	/** Add a clarification event listener.
	 * @param listener The listener to add.
	 */
	public void addClarificationEventListener(ClarificationEventListener listener) {
		clarificationSignal.registerListener(listener);
	}
	
 }
