/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 04.05.2005
 *
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.yajs.ClarificationPair;
import de.tudarmstadt.yajs.gui.FilteredMapItemSelection;
import de.tudarmstadt.yajs.gui.MyTableItemLayout;
import de.tudarmstadt.yajs.gui.MyTableModel;
import de.tudarmstadt.yajs.gui.Predicate;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEvent;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.AskClarificationEventListener;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEvent;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.ClarificationSelectedEventListener;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshClarificationsEventListener;
import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.JudgeClarificationAnswer;
import de.tudarmstadt.yajs.lib.PostedClarificationRequest;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Observable;
import java.util.Observer;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

/**
 * A panel displaying a table of clarifications.
 * @author  markus
 */
public class ClarificationsPanel extends JPanel {

	private ClarificationsModel model;
	private JTable clarificationstable;
	private JTextPane clarificationText;
	private ClarificationEventGenerator clarificationSignal;
	private ClarificationSelectedEventGenerator clarificationSelectedSignal; 
	private RefreshClarificationsEventGenerator refreshClarificationsSignal;
	private AskClarificationEventGenerator askClarificationSignal;
	
	/** Constructor.
	 * @param model The ClarificationsModel.
	 * @param filter A predicate used to filter clarifications.
	 */
	public ClarificationsPanel(ClarificationsModel model, Predicate filter) {
		super();
		this.model = model;
		model.getClarifications().addObserver(new Observer() {
			public void update(Observable arg0, Object arg1) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						updateClarificationText();		
					}
				});
			}
		});
		this.clarificationSignal = new ClarificationEventGenerator();
		this.clarificationSelectedSignal = new ClarificationSelectedEventGenerator();
		this.refreshClarificationsSignal = new RefreshClarificationsEventGenerator();
		this.askClarificationSignal = new AskClarificationEventGenerator();
		buildPanel(filter);
	}

	/** Initialization method creating the panel. */
	private void buildPanel(Predicate filter)
	{
		/* Initialize layout (grid bag layout) */
		
		setLayout(new GridBagLayout());
		GridBagConstraints tableConstraints = new GridBagConstraints();
		GridBagConstraints textConstraints = new GridBagConstraints();
		GridBagConstraints buttonConstraints = new GridBagConstraints();
		
		tableConstraints.fill = GridBagConstraints.BOTH;
		tableConstraints.gridx = 0;
		tableConstraints.gridy = 0;
		tableConstraints.weightx = 1.0;
		tableConstraints.weighty = 1.0;
		
		textConstraints.fill = GridBagConstraints.BOTH;
		textConstraints.gridx = 0;
		textConstraints.gridy = 1;
		textConstraints.weightx = 1.0;
		textConstraints.weighty = 1.0;
		
		buttonConstraints.fill = GridBagConstraints.BOTH;
		buttonConstraints.gridx = 0;
		buttonConstraints.gridy = 2;
		buttonConstraints.weightx = 1.0;
		buttonConstraints.weighty = 0.0;
		
		
		/* initialize the result table */

		MyTableItemLayout clarificationlayout = new MyTableItemLayout() {

			public int getColumnCount() {
				return 7;
			}

			public String getColumnName(int col) {
				switch(col)
				{
					case 0: return "#";
					case 1: return "Submitter";
					case 2: return "Problem";
					case 3: return "Time";
					case 4: return "Question";
					case 5: return "Answer";
					case 6: return "By/To";
				}
				return "";
			}

			public Object getValue(Object item, int col) {
				ClarificationPair p = (ClarificationPair) item;
				
				PostedClarificationRequest r = p.getRequest();
				JudgeClarificationAnswer a = p.getAnswer();
				switch(col)
				{
					case 0: return new Long(r.getIndex());
					case 1: return r.getQuestioner();
					case 2:
						if(r.isGeneralQuestion()) return "<General>";
						return r.getProblem().getID() + " - " + r.getProblem().getName();
					case 3: return (a.getTime() < 0) ? "" : new Long(a.getTime()).toString();
					case 4: return r.getContent();
					case 5: return a.getAnswer();
					case 6: 
							if(a.getState() == ClarificationAnswerState.UNANSWERED) 
								return "";
							else
								return a.getJudgeName() + "/" + a.getState();
					default: return "Please send bugreport!";
				}
			}
		};

		clarificationstable = new JTable( new MyTableModel(getModel().getClarifications(), clarificationlayout, new FilteredMapItemSelection(getModel().getClarifications(), filter)) );
		clarificationstable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent arg0) {
				updateClarificationText();
			}
		});
		clarificationstable.addMouseListener(new MouseAdapter() {
			/* (non-Javadoc)
			 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent arg0) {
				super.mouseClicked(arg0);
				if(arg0.getClickCount() > 1) mouseDoubleClicked(arg0);
			}
			
			public void mouseDoubleClicked(MouseEvent arg0)
			{
				clarificationSelected();
			}
		});
		clarificationstable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		JScrollPane runsscrollpane = new JScrollPane(clarificationstable);
		runsscrollpane.setBorder(BorderFactory.createTitledBorder("Clarification overview"));
		
		add(runsscrollpane, tableConstraints);
	
		/* Add text view */
		clarificationText = new JTextPane();
		clarificationText.setEditable(false);
		clarificationText.setContentType("text/plain");
		setEmptyClarificationText();
		JScrollPane clarificationsPane = new JScrollPane(clarificationText);
		add(clarificationsPane, textConstraints);
		
		/* Add buttons */
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new FlowLayout(FlowLayout.TRAILING));
		
		final JButton askButton = new JButton("Ask");
		askButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				askClarificationSignal.emitAskClarification(new AskClarificationEvent(this));
			}
		});
		buttons.add(askButton);
		
		final JButton answerButton = new JButton("Answer");
		answerButton.setEnabled(false);
		answerButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				clarificationSelected();
			}
		});
		clarificationstable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent arg0) {
				answerButton.setEnabled(false);
				for(int i=arg0.getFirstIndex(); i<=arg0.getLastIndex(); ++i)
				{
					if(clarificationstable.isRowSelected(i))
						answerButton.setEnabled(true);
				}
			}
		});
		buttons.add(answerButton);
		
		final JButton refreshButton = new JButton("Refresh");
		buttons.add(refreshButton);
		refreshButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				refreshClarificationsSignal.emitRefreshClarifications( new RefreshClarificationsEvent(ClarificationsPanel.this));
			}
		});
		
		add(buttons, buttonConstraints);
	}
	
	/** Reset the clarification text displayed to empty. */
	protected void setEmptyClarificationText() {
		Document doc = clarificationText.getEditorKit().createDefaultDocument();
/*		try {
			doc.insertString(doc.getLength(), "", null);
		} catch (BadLocationException e) {
			System.err.println("This should not happen (ever)");
			e.printStackTrace();
		}*/
		clarificationText.setDocument(doc);
	}
	
	/** Show the currently selected clarification's content.
	 */
	protected void updateClarificationText() {
		MyTableModel tm = (MyTableModel) getClarificationsTable().getModel();
		if(getClarificationsTable().getSelectedRows().length == 0) {
			setEmptyClarificationText();
			return;
		}
		ClarificationPair p = (ClarificationPair) tm.getItem(getClarificationsTable().getSelectedRow());
		Document doc = clarificationText.getEditorKit().createDefaultDocument();
		if(p.getRequest() != null) {
			String rtext = "Original question:\n" + p.getRequest().getContent() + "\n\n";
			MutableAttributeSet attributes = new SimpleAttributeSet();
			StyleConstants.setForeground(attributes, Color.GRAY);
			try {
				doc.insertString(doc.getLength(), rtext, attributes);
			} catch (BadLocationException e) {
				System.err.println("This should never happen, please report.");
				e.printStackTrace();
			}
		}
		if(p.getAnswer() != null) {
			String atext = "Answer:\n" + p.getAnswer().getAnswer();
			MutableAttributeSet attributes = new SimpleAttributeSet();
			StyleConstants.setForeground(attributes, Color.BLUE);
			try {
				doc.insertString(doc.getLength(), atext, attributes);
			} catch (BadLocationException e) {
				System.err.println("This should never happen, please report.");
				e.printStackTrace();
			}
		}
		clarificationText.setDocument(doc);
	}

	/**
	 * Retrieve the underlying ClarificationsModel.
	 * @return  Returns the ClarificationsModel.
	 * @uml.property  name="model"
	 */
	private ClarificationsModel getModel() {
		return model;
	}
	
	/** Called when a clarification was selected.
	 * Displays the clarification contents.
	 */
	private void clarificationSelected()
	{
		MyTableModel tm = (MyTableModel) getClarificationsTable().getModel();
		if(getClarificationsTable().getSelectedRow() == -1) return;
		ClarificationPair p = (ClarificationPair) tm.getItem(getClarificationsTable().getSelectedRow());
		long nr = p.getRequest().getIndex();
		clarificationSelectedSignal.emitClarificationSelected(new ClarificationSelectedEvent(this, nr));
	}

	/** Registers a RefreshClarificationsEventListener that will be called back whenever the user wants to refresh the list of clarifications.
	 * @param listener The listener to register.
	 */
	public void addRefreshClarificationsEventListener(RefreshClarificationsEventListener listener) {
		refreshClarificationsSignal.registerListener(listener);
	}


	/** Registers a ClarificationSelectedEventListener that will be called back whenever the user selects a clarification request in order to answer it.
	 * @param listener The listener to register.
	 */
	public void addClarificationSelectedEventListener(ClarificationSelectedEventListener listener) {
		clarificationSelectedSignal.registerListener(listener);
	}
	
	/** Get the clarifications table.
	 * @return Returns the clarificationstable.
	 */
	public JTable getClarificationsTable() {
		return clarificationstable;
	}

	/** Registers an AskClarificationEventListener that will be called back whenever the user wants to ask for clarification.
	 * @param listener The listener to register.
	 */
	public void addAskClarificationEventListener(AskClarificationEventListener listener) {
		askClarificationSignal.registerListener(listener);
	}

}
