/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 18.04.2005
 *
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.yajs.gui.ObservableMap;
import de.tudarmstadt.yajs.lib.JudgeResult;
import de.tudarmstadt.yajs.lib.Login;

/**
 * The judge ui model. Offers methods to access the judgements and clarifications.
 * @author  markus
 */
public class JudgeUIModel implements ClarificationsModel {
	private Login login;
	private ObservableMap judgements;
	private ObservableMap clarifications;
	
	/** Constructor. */
	public JudgeUIModel(Login login, ObservableMap judgements, ObservableMap clarifications) {
		super();
		this.login = login;
		this.judgements = judgements;
		this.clarifications = clarifications;
	}
	
	/**
	 * Getter for the map of judgements.
	 * @return  The map from ids to judgements.
	 * @uml.property  name="judgements"
	 */
	public ObservableMap getJudgements() {
		return judgements;
	}
	
	/**
	 * Getter for the clarifications.
	 * @return  Returns the clarifications.
	 * @uml.property  name="clarifications"
	 */
	public ObservableMap getClarifications() {
		return clarifications;
	}

	/** Getter for individual judgements.
	 * @param id The id of the judgement to get.
	 * @return The judgement that is mapped to id.
	 */
	public JudgeResult getJudgement(long id) {
		return (JudgeResult) getJudgements().get( new Long(id) );
	}

	/** Get the judge's name.
	 * @return The judge's name (rather id).
	 */
	public String getOwnName() {
		return login.getName();
	}
	
}
