/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 18.04.2005
 *
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.lang.*;
import de.tudarmstadt.yajs.gui.DiffPanel;
import de.tudarmstadt.yajs.gui.judge.events.JudgementEvent;
import de.tudarmstadt.yajs.gui.judge.events.JudgementEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.JudgementEventListener;
import de.tudarmstadt.yajs.lib.JudgeResult;
import de.tudarmstadt.yajs.lib.Status;
import de.tudarmstadt.yajs.tester.Diff;
import de.tudarmstadt.yajs.tester.TestCompletedEvent;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
/**
 * The judgement dialog. Dialog that lets judges test a submission and select a judgement.
 * @author  markus
 */
public class JudgementDialog extends JDialog {

	private javax.swing.JPanel jContentPane = null;
	private JPanel jPanel = null;
	private JScrollPane jScrollPane = null;
	private JTextArea sourceView = null;
	private JPanel jPanel1 = null;
	private JComboBox judgementCombo = null;
	private JPanel buttonPanel = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	private JButton testButton = null;
	private JTabbedPane testTabs = null;
	private DiffPanel stdOutPanel = null;
	private JEditorPane stdErrPanel = null;
	
	private JudgeResult previousresult = null;
	private JudgementEventGenerator judgementsignal = new JudgementEventGenerator();
	private String whoami;
	
	private JTextField reasonField = null;
	private JComboBox statusSelection = null;
	
	/**
	 * Wrapper around status and reason in order to predefine judgements.
	 */
	private static class JudgementInfo {
		private Status status;
		private String reason;
		
		/** Constructor. */
		public JudgementInfo(Status status, String reason)
		{
			this.status = status;
			this.reason = reason;
		}
		
		/** Get a string representation of this judgement. 
		 * @return Returns a string representation of the judgement.
		 */
		public String toString()
		{
			if(status == null) return Lang.getString("Custom");
			if(status == Status.UNJUDGED) return Lang.getString("<unjudged>");
			return informalName(status) + " - " + reason;
		}
		
		/** Instead of the status' name, get an informal representation of the status (simply 'yes' or 'no').
		 * @param status The status to work on.
		 * @return Returns "Yes", "No" or "?", depending on the status.
		 */
		private static String informalName(Status status) {
			if(status == Status.ACCEPTED) return Lang.getString("Yes");
			if(status == Status.REJECTED) return Lang.getString("No");
			return "?";
		}
		/**
		 * Get the reason of this judgement.
		 * @return  Returns the reason.
		 * @uml.property  name="reason"
		 */
		public String getReason() {
			return reason;
		}
		/**
		 * Get the status of this judgement.
		 * @return  Returns the status.
		 * @uml.property  name="status"
		 */
		public Status getStatus() {
			return status;
		}
	}

	
	/**
	 * Constructor given the previous result
	 */
	public JudgementDialog(String whoami, JudgeResult jr, String source) {
		super();
		setTitle(Lang.getString("Judgingsubmission") + jr.getIndex() + " , "+ Lang.getString("problem") + jr.getProblem());
		this.previousresult = jr;
		this.whoami = whoami;
		initialize();
		wire();
		getSourceView().setText(source);
		if(previousresult != null) setJudgement(previousresult);
		pack();
	}
	
	/** Set values corresponding to previous judgement.
	 * @param result
	 */
	private void setJudgement(JudgeResult result) {
		getStatusSelection().setSelectedItem(result.getStatus());
		getReasonField().setText(result.getReason());
	}
	
	/**
	 * Rewires some events to fire appropriate JudgementEvent
	 */
	private void wire() {
		final DocumentListener reasonListener = new DocumentListener() {
			public void changedUpdate(DocumentEvent arg0) {
				JudgementInfo ji = (JudgementInfo) getJudgementCombo().getSelectedItem();
				if(ji != null && !getReasonField().getText().equals(ji.getReason()))
					getJudgementCombo().setSelectedItem(null);
			}

			public void insertUpdate(DocumentEvent arg0) {
				JudgementInfo ji = (JudgementInfo) getJudgementCombo().getSelectedItem();
				if(ji != null && !getReasonField().getText().equals(ji.getReason()))
					getJudgementCombo().setSelectedItem(null);
			}

			public void removeUpdate(DocumentEvent arg0) {
				JudgementInfo ji = (JudgementInfo) getJudgementCombo().getSelectedItem();
				if(ji != null && !getReasonField().getText().equals(ji.getReason()))
					getJudgementCombo().setSelectedItem(null);
			}
		};
		
		getCancelButton().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				judgementsignal.emitUnchanged(new JudgementEvent(JudgementDialog.this, getJudgement()));
				JudgementDialog.this.dispose();
			}
		});	
		
		getOkButton().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				judgementsignal.emitChanged(new JudgementEvent(JudgementDialog.this, getJudgement()));
				JudgementDialog.this.dispose();
			}
		});
		
		getTestButton().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				judgementsignal.emitTestSubmission(new JudgementEvent(JudgementDialog.this, previousresult));
			}
		});
		
		getStatusSelection().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JudgementInfo ji = (JudgementInfo) getJudgementCombo().getSelectedItem();
				if(ji != null && !getStatusSelection().getSelectedItem().equals(ji.getStatus()))
					getJudgementCombo().setSelectedItem(null);
			}
		});
		
		getReasonField().getDocument().addDocumentListener(reasonListener);
		
		getJudgementCombo().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JudgementInfo ji = (JudgementInfo) judgementCombo.getSelectedItem(); 
				if(ji == null) return;
				getStatusSelection().setSelectedItem(ji.getStatus());
				PlainDocument doc = new PlainDocument();
				try {
					doc.insertString(0, ji.getReason(), null);
				} catch (BadLocationException e) {
					e.printStackTrace();
				}
				doc.addDocumentListener(reasonListener);
				getReasonField().getDocument().removeDocumentListener(reasonListener);
				getReasonField().setDocument(doc);
			}
		});
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent arg0) {
				judgementsignal.emitUnchanged(new JudgementEvent(JudgementDialog.this, getJudgement()));
			}
		});
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300,200);
		this.setContentPane(getJContentPane());
	}
	/**
	 * This method initializes jContentPane
	 * @return  javax.swing.JPanel
	 * @uml.property  name="jContentPane"
	 */
	private javax.swing.JPanel getJContentPane() {
		if(jContentPane == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(new GridBagLayout());
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.weightx = 1;
			gridBagConstraints1.weighty = 1;
			gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
			gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints1.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			gridBagConstraints2.weighty = 0;
			gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 2;
			gridBagConstraints3.weighty = 0;
			gridBagConstraints3.fill = java.awt.GridBagConstraints.NONE;
			gridBagConstraints3.anchor = GridBagConstraints.SOUTHEAST;
			gridBagConstraints3.gridwidth = 1;
			jContentPane.add(getJPanel(), gridBagConstraints1);
			jContentPane.add(getJPanel1(), gridBagConstraints2);
			jContentPane.add(getButtonPanel(), gridBagConstraints3);
		}
		return jContentPane;
	}
	
	/**
	 * Get the editor pane used to display stderr output. 
	 * @return  Returns the stdErrPanel.
	 * @uml.property  name="stdErrPanel"
	 */
	private JEditorPane getStdErrPanel() {
		if(stdErrPanel == null) {
			stdErrPanel = new JEditorPane();
			stdErrPanel.setContentType(Lang.getString("textplain"));
			stdErrPanel.setEditable(false);
		}
		return stdErrPanel;
	}
	
	/**
	 * This method initializes jPanel	
	 * @return  javax.swing.JPanel
	 * @uml.property  name="jPanel"
	 */    
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(null, Lang.getString("ProblemSource"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.weightx = 1;
			gridBagConstraints1.weighty = 1;
			gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			gridBagConstraints2.weightx = 1;
			gridBagConstraints2.weighty = 0;
			gridBagConstraints2.fill = java.awt.GridBagConstraints.NONE;
			gridBagConstraints2.anchor = java.awt.GridBagConstraints.SOUTHEAST;
			jPanel.add(getTestTabs(), gridBagConstraints1);
			jPanel.add(getTestButton(), gridBagConstraints2);
		}
		return jPanel;
	}
	
	/**
	 * Get the tabbed pane displaying test results. 
	 * @return  Returns the testTabs.
	 * @uml.property  name="testTabs"
	 */
	private JTabbedPane getTestTabs() {
		if (testTabs == null) {
			testTabs = new JTabbedPane();
			testTabs.addTab(Lang.getString("Source"), getJScrollPane());
			testTabs.addTab(Lang.getString("stdout"), new JScrollPane(getStdOutPanel()));
			testTabs.addTab(Lang.getString("stderr"), new JScrollPane(getStdErrPanel()));
		}
		return testTabs;
	}
	
	/**
	 * Get the DiffPanel used to display differences in the output. 
	 * @return  Returns stdOutPanel.
	 * @uml.property  name="stdOutPanel"
	 */
	private DiffPanel getStdOutPanel() {
		if (stdOutPanel == null) {
			stdOutPanel = new DiffPanel("","");
		}
		return stdOutPanel;
	}
	
	/**
	 * This method initializes jScrollPane	
	 * @return  javax.swing.JScrollPane
	 * @uml.property  name="jScrollPane"
	 */    
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane(getSourceView());
		}
		return jScrollPane;
	}
	/**
	 * This method initializes jEditorPane	
	 * @return  javax.swing.JTextArea
	 * @uml.property  name="sourceView"
	 */    
	private JTextArea getSourceView() {
		if (sourceView == null) {
			sourceView = new JTextArea() {
				/**
				 * 
				 */
				private static final long serialVersionUID = -6356242111490782523L;

				public java.awt.Dimension getPreferredScrollableViewportSize() {
					return new java.awt.Dimension(300,200);
				}
			};
			sourceView.setEditable(false);
			sourceView.setTabSize(2);
		}
		return sourceView;
	}
	/**
	 * This method initializes jPanel1	
	 * @return  javax.swing.JPanel
	 * @uml.property  name="jPanel1"
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.setLayout(new BoxLayout(jPanel1, BoxLayout.Y_AXIS));
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null, Lang.getString("Judgement"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			jPanel1.add(getJudgementCombo(), null);
			jPanel1.add(getStatusSelection(), null);
			jPanel1.add(getReasonField(), null);
		}
		return jPanel1;
	}
	/**
	 * This method initializes jComboBox	
	 * @return  javax.swing.JComboBox
	 * @uml.property  name="judgementCombo"
	 */
	private JComboBox getJudgementCombo() {
		if (judgementCombo == null) { 
			final JudgementInfo[] selections = new JudgementInfo[] {
				new JudgementInfo(Status.ACCEPTED, Lang.getString("Correct")),
				new JudgementInfo(Status.REJECTED, Lang.getString("Presentationerror")),
				new JudgementInfo(Status.REJECTED, Lang.getString("Wronganswer")),
				new JudgementInfo(Status.REJECTED, Lang.getString("Timelimitexceeded")),
				new JudgementInfo(Status.REJECTED, Lang.getString("Run-timeerror")),
				new JudgementInfo(Status.REJECTED, Lang.getString("Compilationerror")),
				new JudgementInfo(Status.REJECTED, Lang.getString("Contactstaff")),
			};
			
			judgementCombo = new JComboBox(selections);
			judgementCombo.setSelectedItem(null);
		}
		return judgementCombo;
	}
	
	/**
	 * This method initializes jPanel2	
	 * @return  javax.swing.JPanel
	 * @uml.property  name="buttonPanel"
	 */    
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			FlowLayout flowLayout4 = new FlowLayout();
			buttonPanel = new JPanel();
			buttonPanel.setLayout(flowLayout4);
			flowLayout4.setAlignment(java.awt.FlowLayout.RIGHT);
			buttonPanel.add(getCancelButton(), null);
			buttonPanel.add(getOkButton(), null);
		}
		return buttonPanel;
	}
	/**
	 * This method initializes jButton	
	 * @return  javax.swing.JButton
	 * @uml.property  name="okButton"
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Lang.getString("OK"));
			okButton.setToolTipText(Lang.getString("Confirmjudgement"));
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					
				}
			});
		}
		return okButton;
	}
	/**
	 * This method initializes jButton1	
	 * @return  javax.swing.JButton
	 * @uml.property  name="cancelButton"
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Lang.getString("Cancel"));
		}
		return cancelButton;
	}
	/**
	 * This method initializes jButton2	
	 * @return  javax.swing.JButton
	 * @uml.property  name="testButton"
	 */    
	private JButton getTestButton() {
		if (testButton == null) {
			testButton = new JButton();
			testButton.setText(Lang.getString("Test"));
		}
		return testButton;
	}

	/** Method called when the user clicks okay. If the result does not differ from the previous result, this calls <code>cancel()</code>.
	 * Otherwise, it emits a new JudgementEvent.
	 */
	private void confirmResult()
	{
		JudgeResult jr = getJudgement();
		if(jr.equals(previousresult)) cancel();
		judgementsignal.emitChanged(new JudgementEvent(this,jr));
	}

	/** Cancel this judging dialog, i.e. emit a JudgementEvent indicating that nothing changed.
	 */
	private void cancel() {
		judgementsignal.emitUnchanged(new JudgementEvent(this, getJudgement()));
	}
	/** Get the judgement determined by the user's inputs.
	 * @return Returns a JudgeResult object corresponding to the user's input.
	 */
	private JudgeResult getJudgement() {
		JudgeResult jr = new JudgeResult(
				whoami, 
				previousresult.getContestant(),
				previousresult.getIndex(),
				previousresult.getProblem(),
				previousresult.getLanguage(),
				previousresult.getTime(),
				getJudgementStatus(),
				getJudgementReason(),
				""
				);
		return jr;
	}
	
	/** Get the value of the reasonField.
	 * @return Returns the text in the reasonField.
	 */
	private String getJudgementReason() {
		return reasonField.getText();
	}
	/** Get the value of the statusSelection.
	 * @return Returns the status selected in the statusSelection.
	 */
	private Status getJudgementStatus() {
		return (Status) statusSelection.getSelectedItem();
	}
	
	/** Registers a JudgementEventListener that will be called back whenever the user closes this dialog, by pressing okay, cancel or by closing the window. */
	public void addJudgementEventListener(JudgementEventListener listener)
	{
		judgementsignal.registerListener(listener);
	}
	
	/**
	 * This method initializes jTextField	
	 * @return  javax.swing.JTextField
	 * @uml.property  name="reasonField"
	 */
	private JTextField getReasonField() {
		if (reasonField == null) {
			reasonField = new JTextField();
		}
		return reasonField;
	}
	/**
	 * This method initializes jComboBox	
	 * @return  javax.swing.JComboBox
	 * @uml.property  name="statusSelection"
	 */    
	private JComboBox getStatusSelection() {
		if (statusSelection == null && previousresult != null && previousresult.getStatus() != Status.NEW) {
			final Status[] statuses = new Status[]
			{
				Status.UNJUDGED,
				Status.ACCEPTED,
				Status.REJECTED,
				Status.IGNORED
			};
			statusSelection = new JComboBox(statuses);
		}
		if (statusSelection == null && (previousresult == null || previousresult.getStatus() == Status.NEW)) {
			final Status[] statuses = new Status[]
			{
				Status.ACCEPTED,
				Status.REJECTED,
				Status.IGNORED
			};
			statusSelection = new JComboBox(statuses);
		}

		
		return statusSelection;
	}
	/** Display the results of a test. This method is called when the tester has completed in order to display the results.
	 * @param diff The output difference.
	 * @param error The stderr output.
	 * @param suggestion The suggestion line in the stderr output.
	 */
	public void displayTestResult(Diff diff,String error, String suggestion) {
		if(diff != null) stdOutPanel.setText(diff.getOutput(), diff.getExpected());
		else stdOutPanel.setText("","");
		stdErrPanel.setText(error);
		adjustJudgement(suggestion, diff);
	}
	/** Adjust the judgement according to the test result suggestion and the output difference.
	 * @param suggestion The suggestion returned by the tester.
	 * @param diff The difference in the output.
	 */
	private void adjustJudgement(String suggestion, Diff diff) {
		// TODO CLEAN ME!
		if(suggestion.indexOf(':') == -1) {
			System.err.println(Lang.getString("Noinsuggestion"));
			statusSelection.setSelectedItem(Status.REJECTED);
			reasonField.setText(Lang.getString("Run-timeerror"));
			testTabs.setSelectedIndex(2);
			return;
		}
		String code = suggestion.substring(0, suggestion.indexOf(':'));
		if(code.equals(Lang.getString("SUCCESS"))) {
			if(diff.doesDiffer()) {
				statusSelection.setSelectedItem(Status.REJECTED);
				reasonField.setText(Lang.getString("Wronganswer"));
				testTabs.setSelectedIndex(1);
			} else {
				statusSelection.setSelectedItem(Status.ACCEPTED);
				reasonField.setText(Lang.getString("Correct"));
				testTabs.setSelectedIndex(2);
			}
		} else if(code.equals(Lang.getString("RUNTIME"))) {
			statusSelection.setSelectedItem(Status.REJECTED);
			reasonField.setText(Lang.getString("Run-timeerror"));
			testTabs.setSelectedIndex(2);
		} else if(code.equals(Lang.getString("COMPILE"))) {
			statusSelection.setSelectedItem(Status.REJECTED);
			reasonField.setText(Lang.getString("Compilationerror"));
			testTabs.setSelectedIndex(2);	
		} else if(code.equals(Lang.getString("TIMELIMIT"))) {
			statusSelection.setSelectedItem(Status.REJECTED);
			reasonField.setText(Lang.getString("Timelimitexceeded"));
			testTabs.setSelectedIndex(2);
		} else if(code.equals(Lang.getString("ERROR"))) {
			testTabs.setSelectedIndex(2);
		}
	}
	/** Enable or disable testing. Needed as we do not want to run several testing processes for the same submission.
	 * @param b True if testing should be enabled, false otherwise.
	 */
	public void setTestingEnabled(boolean b) {
		testButton.setEnabled(b);
	}
	
  }
