/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 13.04.2005
 *
 */
package de.tudarmstadt.yajs.gui.judge;

import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.gui.ContestStatusPanel;
import de.tudarmstadt.yajs.gui.judge.events.RefreshServerStatusEvent;
import de.tudarmstadt.yajs.gui.judge.events.RefreshServerStatusEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.RefreshServerStatusEventListener;
import de.tudarmstadt.yajs.gui.judge.events.TogglePauseEvent;
import de.tudarmstadt.yajs.gui.judge.events.TogglePauseEventGenerator;
import de.tudarmstadt.yajs.gui.judge.events.TogglePauseEventListener;
import de.tudarmstadt.yajs.lib.ContestStatus;
import de.tudarmstadt.yajs.lib.ContestStatusEnumeration;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A panel showing miscellaneous other things, like the contest status, the server status...
 * @author  markus
 */
public class MiscellaneousPanel extends JPanel {
	private JLabel statusLabel;
	
	private ContestStatusPanel statusPanel;
	private XMLDisplayPanel serverStatusPanel;
	
	private JButton togglePauseButton;
	private TogglePauseEventGenerator togglePauseSignal = new TogglePauseEventGenerator();
	private RefreshServerStatusEventGenerator refreshServerStatusSignal = new RefreshServerStatusEventGenerator();
	private JPanel buttonPanel;
	private JButton runningButton;
	private JButton pausedButton;
	private JButton beforeButton;
	private JButton stoppedButton;
	
	/** Constructor. */
	public MiscellaneousPanel()
	{
		super(new GridBagLayout());
		GridBagConstraints statusConstraints = new GridBagConstraints();
		GridBagConstraints serverStatusConstraints = new GridBagConstraints();
		GridBagConstraints buttonConstraints = new GridBagConstraints();
		
		statusConstraints.gridx = 0;
		statusConstraints.gridy = 0;
		statusConstraints.weightx = 1;
		statusConstraints.weighty = 0;
		statusConstraints.fill = GridBagConstraints.BOTH;
		
		serverStatusConstraints.gridx = 0;
		serverStatusConstraints.gridy = 1;
		serverStatusConstraints.weightx = 1;
		serverStatusConstraints.weighty = 1;
		serverStatusConstraints.fill = GridBagConstraints.BOTH;
		
		buttonConstraints.gridx = 0;
		buttonConstraints.gridy = 2;
		buttonConstraints.weightx = 1;
		buttonConstraints.weighty = 0;
		buttonConstraints.fill = GridBagConstraints.BOTH;
		
		add(getStatusPanel(), statusConstraints);
		add(getServerStatusPanel(), serverStatusConstraints);
		add(getButtonPanel(), buttonConstraints);
		
		getServerStatusPanel().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				refreshServerStatusSignal.emitRefresh(new RefreshServerStatusEvent(arg0.getSource()));
			}
		});
	}
	
	/**
	 * Getter for the panel displaying the server status.
	 * @return  Returns the XMLDisplayPanel to display the server status.
	 * @uml.property  name="serverStatusPanel"
	 */
	private XMLDisplayPanel getServerStatusPanel() {
		if(serverStatusPanel == null) {
			serverStatusPanel = new XMLDisplayPanel("serverstatus.xsl"); //$NON-NLS-1$
			serverStatusPanel.setBorder(BorderFactory.createTitledBorder(Lang.getString("MiscellaneousPanel.1"))); //$NON-NLS-1$
		}
		return serverStatusPanel;
	}

	/**
	 * Getter for the status panel. 
	 * @return  Returns the panel used to display the contest status.
	 * @uml.property  name="statusPanel"
	 */
	protected ContestStatusPanel getStatusPanel() {
		if(statusPanel == null) {
			statusPanel = new ContestStatusPanel();
		}
		return statusPanel;
	}

	/**
	 * Get the button panel containing the status buttons. Creates the panel if needed.
	 * @return  Returns the button panel.
	 * @uml.property  name="buttonPanel"
	 */ 
	protected JPanel getButtonPanel() {
		if(buttonPanel == null) {
			buttonPanel = new JPanel(new GridLayout(1,4));
			buttonPanel.add(getBeforeButton());
			buttonPanel.add(getRunningButton());
			buttonPanel.add(getPausedButton());
			buttonPanel.add(getStoppedButton());
			buttonPanel.setBorder(BorderFactory.createTitledBorder(Lang.getString("MiscellaneousPanel.2"))); //$NON-NLS-1$
		}
		return buttonPanel;
	}

	/**
	 * Get the button that will set the contest status to "Before the contest". 
	 * @return  Returns the before button.
	 * @uml.property  name="beforeButton"
	 */
	protected JButton getBeforeButton() {
		if(beforeButton == null) {
			beforeButton = new JButton(Lang.getString("MiscellaneousPanel.3")); //$NON-NLS-1$
			beforeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					togglePauseSignal.emitEvent(new TogglePauseEvent(MiscellaneousPanel.this, ContestStatusEnumeration.BEFORE));
				}
			});
		}
		return beforeButton;
	}
	
	/**
	 * Get the button that will set the contest status to "Running". 
	 * @return  Returns the running button.
	 * @uml.property  name="runningButton"
	 */
	protected JButton getRunningButton() {
		if(runningButton == null) {
			runningButton = new JButton(Lang.getString("MiscellaneousPanel.4")); //$NON-NLS-1$
			runningButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					togglePauseSignal.emitEvent(new TogglePauseEvent(MiscellaneousPanel.this, ContestStatusEnumeration.RUNNING));
				}
			});
		}
		return runningButton;
	}
	
	/**
	 * Get the button that will set the contest status to "Paused". 
	 * @return  Returns the paused button.
	 * @uml.property  name="pausedButton"
	 */
	protected JButton getPausedButton() {
		if(pausedButton == null) {
			pausedButton = new JButton(Lang.getString("MiscellaneousPanel.5")); //$NON-NLS-1$
			pausedButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					togglePauseSignal.emitEvent(new TogglePauseEvent(MiscellaneousPanel.this, ContestStatusEnumeration.PAUSED));
				}
			});
		}
		return pausedButton;
	}
	
	/**
	 * Get the button that will set the contest status to "After the contest". 
	 * @return  Returns the after button.
	 * @uml.property  name="stoppedButton"
	 */
	protected JButton getStoppedButton() {
		if(stoppedButton == null) {
			stoppedButton = new JButton(Lang.getString("MiscellaneousPanel.6")); //$NON-NLS-1$
			stoppedButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					togglePauseSignal.emitEvent(new TogglePauseEvent(MiscellaneousPanel.this, ContestStatusEnumeration.AFTER));
				}
			});
		}
		return stoppedButton;
	}
	
	/** Update the contest status. 
	 * @param contestStatus The contest status to display.
	 */
	public void updateContestStatus(ContestStatus contestStatus) {
		getStatusPanel().updateContestStatus(contestStatus);
	}

	/** Registers a TogglePauseEventListener that will be called back whenever the user wants to change the contest status.
	 * @param listener The listener to register.
	 */
	public void addTogglePauseEventListener(TogglePauseEventListener listener) {
		togglePauseSignal.registerListener(listener);
	}
	
	/** Registers a RefreshServerStatusEventListener that will be called back whenever the user wants to refresh the server status.
	 * @param listener The listener to register.
	 */
	public void addRefreshServerStatusEventListener(RefreshServerStatusEventListener listener) {
		refreshServerStatusSignal.registerListener(listener);
	}

	/** Update the server status.
	 * @param status The server status to display.
	 */
	public void updateServerStatus(String status) {
		serverStatusPanel.setText(status);
	}
}
