/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 10.03.2005
 *
 */
package de.tudarmstadt.yajs.lib;

import de.tudarmstadt.events.EventGenerator;
import de.tudarmstadt.yajs.OfflineHelper;
import de.tudarmstadt.yajs.lib.events.AutoConfigEvent;
import de.tudarmstadt.yajs.lib.events.AutoConfigEventGenerator;
import de.tudarmstadt.yajs.lib.events.AutoConfigEventListener;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEvent;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEventGenerator;
import de.tudarmstadt.yajs.lib.events.ClientClarificationAnswerEventListener;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEvent;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEventGenerator;
import de.tudarmstadt.yajs.lib.events.ConnectionErrorEventListener;
import de.tudarmstadt.yajs.lib.events.LoginEvent;
import de.tudarmstadt.yajs.lib.events.LoginEventGenerator;
import de.tudarmstadt.yajs.lib.events.LoginEventListener;
import de.tudarmstadt.yajs.lib.events.ScoreboardEvent;
import de.tudarmstadt.yajs.lib.events.ScoreboardEventGenerator;
import de.tudarmstadt.yajs.lib.events.ScoreboardEventListener;
import de.tudarmstadt.yajs.lib.events.StatusEvent;
import de.tudarmstadt.yajs.lib.events.StatusEventGenerator;
import de.tudarmstadt.yajs.lib.events.StatusEventListener;
import de.tudarmstadt.yajs.lib.fsm.ClientFSM;
import de.tudarmstadt.yajs.lib.fsm.actions.Action;
import de.tudarmstadt.yajs.lib.fsm.actions.ClientAction;
import java.io.IOException;
/**
 * The client base class for both contestants and judges.
 * @author  markus
 */
public abstract class Client {

	private MessageSocket sock;
	private ScoreboardEventGenerator scoreboardSignal;
	private EventGenerator errorSignal;
	private EventGenerator loginSignal;
	private StatusEventGenerator statusSignal;
	private AutoConfigEventGenerator autoConfigSignal;
	private ClientClarificationAnswerEventGenerator clientClarificationAnswerSignal;
	private ContestInfo contest;
	private ServerInfo server;
	private ConnectionInfo connection;
	private Login login;
	private ClientFSM fsm;
	private ContestStatus status;
	
	
	protected abstract class ConcreteClientHelper implements ClientHelper
	{	
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#sendHeartbeat()
		 */
		public void sendHeartbeatRequest() {
			Client.this.sendHeartbeatRequest();
		}
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#sendHeartbeatRequest(de.tudarmstadt.yajs.lib.fsm.actions.Action)
		 */
		public void sendHeartbeatRequest(Action action) {
			Client.this.sendHeartbeatRequest(action);
		}
		
		/** Get the login flags, i.e. "judge" or "contestant".
		 * @return Returns a string containing the login flags.
		 */
		protected abstract String getLoginFlags();
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#sendLogin()
		 */
		public void sendLogin()
		{
			sock.send(ClientAction.LOGIN_REQUEST + "\n" + getLoginFlags() + "\n" + login.name + "\n" + login.password + "\n");
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#sendMessage(java.lang.String)
		 */
		public void sendMessage(String msg) {
			sock.send(msg);
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#setServerInfo(de.tudarmstadt.yajs.lib.ServerInfo)
		 */
		public void setServerInfo(ServerInfo info) {
			server = info;
			checkServerInfo();
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#setConnectionInfo(de.tudarmstadt.yajs.lib.ConnectionInfo)
		 */
		public void setConnectionInfo(ConnectionInfo ci) {
			connection = ci;
			checkConnectionInfo();
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#getConnectionInfo()
		 */
		public ConnectionInfo getConnectionInfo() {
			return Client.this.getConnectionInfo();
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#getContestInfo()
		 */
		public ContestInfo getContestInfo() {
			return contest;
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#updateContestInfo(de.tudarmstadt.yajs.lib.Problem[], de.tudarmstadt.yajs.lib.Language[])
		 */
		public void updateContestInfo(Problem[] problems, Language[] languages) {
			
			contest = new ContestInfo(contest.getServer(), new LanguageSet(languages), new ProblemSet(problems));
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#updateContestStatus(de.tudarmstadt.yajs.lib.ContestStatus)
		 */
		public void updateContestStatus(ContestStatus status) {
			if(Client.this.status != null && Client.this.status.equals(status)) return;
			Client.this.status = status;
			Client.this.signalUpdatedStatus(status);
		}
		/* (non-Javadoc)this method is originally protected, not public.
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#signalLogin(boolean)
		 */
		public void signalLogin(boolean success) {
			Client.this.signalLogin(success);
			
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#signalScoreboardUpdate(java.lang.String)
		 */
		public void signalScoreboardUpdate(String scoreboard) {
			
			Client.this.signalScoreboardUpdate(scoreboard);
		}
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#setAutoConfig(de.tudarmstadt.yajs.lib.ProblemSet, de.tudarmstadt.yajs.lib.LanguageSet)
		 */
		public void setAutoConfig(ProblemSet problems, LanguageSet languages) {
			Client.this.signalAutoConfig(problems, languages);
		}
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#closeWithError(java.lang.String)
		 */
		public void closeWithError(String error) {
			Client.this.closeWithError(error);
		}
		
		/*
		 * (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.ClientHelper#signalClientClarificationAnswer(long, java.lang.String, de.tudarmstadt.yajs.lib.ClarificationAnswerState, boolean, java.lang.String)
		 */
		public void signalClientClarificationAnswer(long requestnr, String problemID,
				ClarificationAnswerState state, boolean b, String answer) {
			clientClarificationAnswerSignal.emitClientClarificationAnswer(new ClientClarificationAnswerEvent(Client.this, b, new ClientClarificationAnswer(requestnr, getContestInfo().getProblems().getProblem(problemID), state, answer)));
		}
		
	}
	/** Constructor. Sets the contest information and the login.
	 * @param contest The contest information.
	 * @param login The login to use.
	 * @param con 
	 * @throws IOException
	 */
	public Client(ContestInfo contest) throws IOException
	{
		this.loginSignal = new LoginEventGenerator();
		this.errorSignal = new ConnectionErrorEventGenerator();
		this.statusSignal = new StatusEventGenerator();
		this.scoreboardSignal = new ScoreboardEventGenerator();
		this.autoConfigSignal = new AutoConfigEventGenerator();
		this.clientClarificationAnswerSignal = new ClientClarificationAnswerEventGenerator();
		this.contest = contest;
	
		this.login = new Login("","",false);
	}
	/** Constructor. Sets the contest information and the login.
	 * @param contest The contest information.
	 * @param login The login to use.
	 * @param con 
	 * @throws IOException
	 */
	public Client(ContestInfo contest, Login login) throws IOException
	{
		this.loginSignal = new LoginEventGenerator();
		this.errorSignal = new ConnectionErrorEventGenerator();
		this.statusSignal = new StatusEventGenerator();
		this.scoreboardSignal = new ScoreboardEventGenerator();
		this.autoConfigSignal = new AutoConfigEventGenerator();
		this.clientClarificationAnswerSignal = new ClientClarificationAnswerEventGenerator();
		this.contest = contest;
		if (login.isOnline()){
			System.out.print("Connecting to " + contest.getServer() + "...");
			this.sock = new MessageSocket(contest.getServer());
			System.out.println("done.");
		}
		this.login = login;
	}
	/** Dispatch an auto-config event when the client has received auto-configuration.
	 * @param problems The problemset that was received.
	 * @param languages The set of languages that was received.
	 */
	public void signalAutoConfig(ProblemSet problems, LanguageSet languages) {
		autoConfigSignal.emitAutoConfig(new AutoConfigEvent(this, problems, languages));
	}
	/** Dispatch a scoreboard update event when the client has received a scoreboard.
	 * @param scoreboard The scoreboard as sent by the server.
	 */
	public void signalScoreboardUpdate(String scoreboard) {
		scoreboardSignal.emitScoreboardReceived(new ScoreboardEvent(this, scoreboard));
	}
	/** Dispatch a status event when the client has received a contest status.
	 * @param status
	 */
	private void signalUpdatedStatus(ContestStatus status) {
		statusSignal.emitStatusChanged(new StatusEvent(this, status));
	}
	/** Set the FSM handling network traffic/protocol.
	 * @param fsm The fsm to use.
	 */
	protected synchronized void setFSM(ClientFSM fsm)
	{
		this.fsm = fsm;
	}
	
	/** Signals a login response.
	 * @param success true, if the login was successful.
	 */
	public void signalLogin(boolean success) {
		loginSignal.emitEvent(new LoginEvent(this, success));
	}
	/** Resets the client to the initial state (expecting hello).
	 */
	public synchronized void reset()
	{
		fsm.reset();
	}
	/** Process the arrived messages
	 * @throws UnexpectedMessage
	 * @throws MalformedMessage
	 */
	public void process() throws UnexpectedMessage, MalformedMessage, ConnectionError
	{
		// I am the only consumer of messages on sock, so the following is perfectly fine
		// Otherwise we would have to have a lock here...
		int avail = sock.available();
		for(int i=0; i<avail; i++)
			{
				String msg = sock.receive();
				fsm.delta(msg);
			}
		
		
	}
	
	/**
	 * Get the login. 
	 * @return  Returns the login.
	 * @uml.property  name="login"
	 */
	public Login getLogin() { return login; }
	/** Get the server information.
	 * @return Returns the server information.
	 */
	public ServerInfo getServerInfo() { return server; }
	/** Get the connection information.
	 * @return Returns the connection information.
	 */
	public ConnectionInfo getConnectionInfo() { return connection; }

	/** Close the socket.
	 * Terminates the connection to the server by disconnecting.
	 */
	public void closeSocket()
	{
		if(sock != null)
			try {
				sock.close();
			} catch (IOException e) {
				// This _is_ fatal
				e.printStackTrace();
				System.exit(-1);
			}
	}
	
	/** Close the connection due to an error.
	 * The error will be propagated through a ConnectionErrorEvent.
	 * @param error A description of the error.
	 */
	protected void closeWithError(String error)
	{
		System.err.println("Closing connection due to error:\n" + error);
		errorSignal.emitEvent(new ConnectionErrorEvent(this, error));
		closeSocket();
	}
	
	/** Test whether the connection is still alive.
	 * The connection is alive if it is still open on both ends.  
	 * @return True, if and only if the client is still connected to the server.
	 */
	public boolean isAlive()
	{
		return sock.isAlive();
	}
	
	/** Send a message.
	 * Sends the message msg to the server.
	 * @param msg The message to send.
	 */
	protected void send(String msg)
	{
		sock.send(msg);
	}
	
	/** Send a clarification request.
	 * @param c The clarification request.
	 */
	public void requestClarification(ClarificationRequest c)
	{
		send(ClientAction.CLARIFICATION_REQUEST + "\n" + (c.getProblem() != null ? c.getProblem().getID() : "") + "\n" + c.getContent()); 
	}
	
	/** Add a LoginEventListener.
	 * A LoginEvent will be dispatched exactly once during a session, when 
	 * the login is completed, or when it failed.
	 * @param l
	 */
	public void addLoginEventListener(LoginEventListener l)
	{
		loginSignal.registerListener(l);
	}
	
	/** Add a ConnectionErrorEventListener.
	 * When the connection is closed on the other side a ConnectionErrorEvent
	 * is dispatched.
	 * @param l
	 */
	public void addConnectionErrorEventListener(ConnectionErrorEventListener l)
	{
		errorSignal.registerListener(l);
	}
	
	/** Add a StatusEventListener.
	 * Emitted, when the contest status (state, elapsed time, duration) changes.
	 * @param l A StatusEventListener.
	 */
	public void addStatusEventListener(StatusEventListener l) {
		statusSignal.registerListener(l);
	}
	
	/** Add a ScoreboardEventListener.
	 * This event is emitted when a scoreboard arrives.
	 * @param listener
	 */
	public void addScoreboardEventListener(ScoreboardEventListener listener) {
		scoreboardSignal.registerListener(listener);
	}
	
	/** Add an AutoConfigEventListener.
	 * The AutoConfigEvent is emitted when auto configuration was received. 
	 * @param listener
	 */
	public void addAutoConfigEventListener(AutoConfigEventListener listener) {
		autoConfigSignal.registerListener(listener);
	}
	
	/** Add a ClientClarificationAnswerEventListener.
	 * The event is emitted when an answer to a clarification request was received. 
	 * @param listener
	 */
	public void addClientClarificationAnswerEventListener(ClientClarificationAnswerEventListener listener) {
		clientClarificationAnswerSignal.registerListener(listener);
	}
	
	/** Check whether the current ServerInfo is acceptable.
	 * This usually includes a check whether the server actually accepts
	 * the kind of client that is trying to connect.
	 * It will be called whenever the ServerInfo flags are updated (at the moment,
	 * only during the login sequence).
	 */
	protected abstract void checkServerInfo();
	
	/** Check whether the current ConnectionInfo is acceptable.
	 * Analogous to checkServerInfo, this usually checks whether we indeed have
	 * a connection of the kind that we wanted to establish.
	 */
	protected abstract void checkConnectionInfo();
	
	/** Send a heartbeat request.
	 */
	public void sendHeartbeatRequest() {
		sendHeartbeatRequest(null);
	}
	
	/** Send a heartbeat request that synchronizes, i.e. calls another Action when received.
	 * This is synchronized, as sending the request and registering the sync action must be atomic.
	 */
	public synchronized void sendHeartbeatRequest(Action a) {
		send(ClientAction.HEARTBEAT_REQUEST + "\n");
		fsm.addSyncAction(a);
	}
	
	/** Query the server for the scoreboard.
	 * 
	 */
	public void queryScoreboard() {
		send(ClientAction.SCOREBOARD_REQUEST + "\n");
	}
	
	/** Query the server for a list of clarifications.
	 * 
	 */
	public void queryClarifications() {
		send(ClientAction.CLARIFICATION_LIST + "\n");
	}
}
