/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 10.03.2005
 *
 */
package de.tudarmstadt.yajs.lib;

import de.tudarmstadt.yajs.lib.fsm.actions.Action;


/**
 * ClientHelper is an interface to the client that is passed to the fsm in order to call-back some methods.
 * @author  markus
 */
public interface ClientHelper {
	/** Send the login request.
	 */ 
	public void sendLogin();
	/** Send a message.
	 * @param msg The message to send.
	 */
	public void sendMessage(String msg);
	/** Set the server information.
	 * @param info The server information to set.
	 */
	public void setServerInfo(ServerInfo info);
	/**
	 * Set the connection information.
	 * @param ci  The connection information to set.
	 * @uml.property  name="connectionInfo"
	 */
	public void setConnectionInfo(ConnectionInfo ci);
	/** Get the connection information.
	 * @return Returns the connection information.
	 */
	public ConnectionInfo getConnectionInfo();
	/** Get the contest information.
	 * @return Returns the contest information.
	 */
	public ContestInfo getContestInfo();
	/** Update the contest information.
	 */
	public void updateContestInfo(Problem[] problems, Language[] languages);
	
	/** Signal completion of the login process.
	 * @param success Indicates if the login was successful.
	 */
	public void signalLogin(boolean success);
	/** Send a heartbeat request.
	 */
	public void sendHeartbeatRequest();
	/** Update the contest status.
	 * @param cstatus The contest status that was received.
	 */
	public void updateContestStatus(ContestStatus status);
	/** Update the scoreboard.
	 * @param scoreboard The scoreboard that was received.
	 */
	public void signalScoreboardUpdate(String scoreboard);
	/** Send a heartbeat request.
	 * @param adapter Action to perform when the heartbeat reply is received.
	 */
	public void sendHeartbeatRequest(Action adapter);
	/** Set the problemset and languages due to auto configuration.
	 * @param problems The set of problems that was received.
	 * @param languages The set of languages that was received.
	 */
	public void setAutoConfig(ProblemSet problems, LanguageSet languages);
	
	/** Close the client due to an error.
	 * @param error
	 */
	public void closeWithError(String error);
	/** Signal the arrival of a clarification.
	 * @param requestnr The number of the request this answer relates to.
	 * @param problemID The id of the problem it refers to.
	 * @param state The state of the clarification.
	 * @param b 
	 * @param string 
	 */
	public void signalClientClarificationAnswer(long requestnr, String problemID, ClarificationAnswerState state, boolean b, String string);
}
