/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on Dec 5, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.tudarmstadt.yajs.lib;

import java.util.HashSet;
import java.util.Set;

/** Connection information.
 * After connection, the server sends some information and flags.
 * These are stored here.
 * @author markus
 */
public class ConnectionInfo {

	private static final String FLAG_CONTESTANT = "contestant";
	private static final String FLAG_JUDGE = "judge";
	private static final String FLAG_NOTIFIES = "notifies";
	private static final String FLAG_STATUS = "status";
	private static final String FLAG_AUTOCONFIG = "autoconfig";
	private static final String FLAG_SCOREBOARD = "scoreboard";
	private static final String FLAG_CLARIFICATIONS = "clarifications";
	private static final String FLAG_PAUSE = "pause";
	
	private String login;
	private HashSet flags;
	
	/** Constructor. Initializes the fields from the given values.
	 * @param loginname The login name of the client.
	 * @param flagset The set of flags.
	 */
	public ConnectionInfo(String loginname, Set flagset) {
		login = new String(loginname);
		flags = new HashSet(flagset); 
	}
	
	/** Test whether this is a contestant connection. 
	 * @return Returns true if this is a contestant connection.
	 */
	public boolean isContestantConnection() { return flags.contains(FLAG_CONTESTANT); }
	/** Test whether this is a judge connection. 
	 * @return Returns true if this is a judge connection.
	 */
	public boolean isJudgeConnection() { return flags.contains(FLAG_JUDGE); }
	/** Test whether the server sends notification messages. Otherwise clients have to poll all sorts of data.
	 * @return Returns true if the server sends notifications.
	 */
	public boolean serverNotifies() { return flags.contains(FLAG_NOTIFIES); }
	/** Test whether the server sends a server status. 
	 * @return Returns true if the server supports sending server statuses.
	 */
	public boolean serverSendsStatus() { return flags.contains(FLAG_STATUS); }
	/** Test whether the server sends auto configuration.
	 * @return Returns true if the server sends auto configuration.
	 */
	public boolean serverSendsAutoConfig() { return flags.contains(FLAG_AUTOCONFIG); }
	/** Test whether the server supports sending a scoreboard. 
	 * @return Returns true if the server supports sending a scoreboard. 
	 */
	public boolean serverHandlesScoreboard() { return flags.contains(FLAG_SCOREBOARD); }
	/** Test whether the server supports clarifications. 
	 * @return Returns true if the server supports clarifications. 
	 */
	public boolean serverHandlesClarifications() { return flags.contains(FLAG_CLARIFICATIONS); }
	/** Test whether the server supports setting the contest status. 
	 * @return Returns true if the server supports setting the contest status. 
	 */
	public boolean serverHandlesPause() { return flags.contains(FLAG_PAUSE); }
	
}
