/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 24.05.2005
 *
 */
package de.tudarmstadt.yajs.lib;

/**
 * A clarification answer as seen by judges. Also contains the name of the judge who sent the clarification and a lock string.
 * @author  markus
 */
public class JudgeClarificationAnswer extends ClarificationAnswer {

	private String judgeName;
	private String lockString;
	private long time;
	
	/** Constructor. Initializes fields with given values.
	 * @param requestIndex The index of the corresponding clarification request.
	 * @param state The state of the clarification.
	 * @param time The time after which the clarification was sent.
	 * @param answer The clarification itself.
	 * @param judgeName The name of the judge who sent the clarification.
	 * @param lockString The lock string.
	 */
	public JudgeClarificationAnswer(long requestIndex, ClarificationAnswerState state, long time, String answer, String judgeName, String lockString) {
		super(requestIndex, state, answer);
		this.judgeName = judgeName;
		this.lockString = lockString;
		this.time = time;
	}
	
	/**
	 * Get the name of the judge who sent the clarification.
	 * @return  Returns the judgeName.
	 * @uml.property  name="judgeName"
	 */
	public String getJudgeName() {
		return judgeName;
	}
	/**
	 * Get the lock string.
	 * @return  Returns the lockString.
	 * @uml.property  name="lockString"
	 */
	public String getLockString() {
		return lockString;
	}
	
	/**
	 * Getter for the time this answer was sent at.
	 * @return  Returns the time when this answer was sent.
	 * @uml.property  name="time"
	 */
	public long getTime() {
		return time;
	}
}
