/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 17.03.2005
 *
 */
package de.tudarmstadt.yajs.lib;


/**
 * A result as seen by a judge. Judges can also see the contestant who sent the submission and whether the submission is currently locked.
 * @author  markus
 */
public class JudgeResult extends Result {
	
	private String judge;
	private String contestant;
	private String lock;
	
	/**
	 * Get the lock string.
	 * @return  Returns the lock.
	 * @uml.property  name="lock"
	 */
	public String getLock() {
		return lock;
	}
	/**
	 * Get the contestant who sent the submission.
	 * @return  Returns the contestant.
	 * @uml.property  name="contestant"
	 */
	public String getContestant() {
		return contestant;
	}
	/**
	 * Get the judge who sent the judgement.
	 * @return  Returns the judge.
	 * @uml.property  name="judge"
	 */
	public String getJudge() {
		return judge;
	}

	
	/** Constructor.
	 * Initializes fields with given values.
	 * @param judge The judge who sent the judgement.
	 * @param contestant The contestant who sent the submission.
	 * @param index The index of the submission.
	 * @param problem The problem of the submission.
	 * @param language The language of the submission.
	 * @param time The time after which the submission was sent.
	 * @param status The status of the submission.
	 * @param reason The reason given by the judge.
	 * @param lock The lock string.
	 */
	public JudgeResult(String judge, String contestant, long index, Problem problem, Language language, long time, Status status, String reason, String lock) {
		super(index, problem, language, time, status, reason);
		this.judge = new String(judge);
		this.contestant = new String(contestant);
		this.lock = lock;
	}
	
	/** Constructor.
	 * This is a copy constructor initializing the fields with the same values as those in <code>j</code>.
	 * @param j The JudgeResult to copy.
	 */
	public JudgeResult(JudgeResult j) {
		this(j.getJudge(), j.getContestant(), j.getIndex(), j.getProblem(), j.getLanguage(), j.getTime(), j.getStatus(), j.getReason(), j.getLock());
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object arg0) {
		JudgeResult other = (JudgeResult) arg0;
		return super.equals(arg0) && getJudge().equals(other.getJudge()) && getContestant().equals(other.getContestant()); 
	}
	/** Find out whether the submission still needs to be judged.
	 * This is the case if the status is either "unjudged" or "new".
	 * @return Returns true if the submission needs to be judged.
	 */
	public boolean needsJudgement() {
		return getStatus() == Status.UNJUDGED || getStatus() == Status.NEW;
	}
	
}
