/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 16.02.2005
 *
 */
package de.tudarmstadt.yajs.lib;

import java.util.AbstractSet;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;


/** A set of programming languages.
 * @author markus
 *
 */
public class LanguageSet extends AbstractSet {

	private Map languages = new TreeMap();
	
	/** Constructor. 
	 * Creates an empty set.
	 */
	public LanguageSet() {}
	
	/** Constructor.
	 * Creates a set containing the languages given in <code>languages</code>
	 * @param languages The languages this set should contain.
	 */
	public LanguageSet(Language[] languages) {
		for(int i=0; i<languages.length; ++i)
			add(languages[i]);
	}
	
	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#size()
	 */
	public int size() {
		return languages.size();
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#iterator()
	 */
	public Iterator iterator() {
		return languages.values().iterator();
	}
	
	/* (non-Javadoc)
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public boolean add(Object arg0) {
		return add( (Language) arg0 );
	}
	
	/** Add a language to this set.
	 * @param l The language to add.
	 * @return Returns true if the language was added.
	 */
	public boolean add(Language l)
	{
		return languages.put(l.getID(), l) != l;
	}

	/** Get the language with given id.
	 * @param id The id of the language to get.
	 * @return Returns the language with the given id if there is one, <code>null</code> otherwise.
	 */
	public Language getLanguage(String id)
	{
		return (Language) languages.get(id);
	}
}
