/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib;
import java.util.*;

/** A message queue (FIFO).
 * A thread-safe queue where messages can be pushed and popped.
 */
class MessageQueue
{
	private LinkedList q;

	/** Constructor. */
	public MessageQueue()
	{
		q = new LinkedList();
	}
	
	/** Get the number of messages in the queue.
	 * <b>Attention:</b> In a multi-threaded environment, this might change immediately after returning.
	 * @return Returns the number of messages in the queue.
	 */
	synchronized int size() 
	{
		return q.size();
	}
	
	/** Push a message to the queue.
	 */
	synchronized void push(String msg)
	{
		q.add(msg);
		notifyAll();
	}
	
	/** Pop a message from the queue.
	 * @return Returns the message that was popped from the queue.
	 */
	synchronized String pop() throws NoSuchElementException
	{
		String popped = (String) q.removeFirst();
		notifyAll();
		return popped;
	}
}
