/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 16.02.2005
 *
 */
package de.tudarmstadt.yajs.lib;

import java.util.AbstractSet;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/** A set of problems.
 * Reflects the problemset, i.e. all problems constituting this contest.
 * @author markus
 *
 */
public class ProblemSet extends AbstractSet {

	/** A map from problem ids to problems. */
	private Map problems = new TreeMap();
	
	/** Create an empty problem set.
	 */
	public ProblemSet() {}
	
	/** Create a problem set containing the problems from the argument. */
	public ProblemSet(Problem[] problems) {
		for(int i=0; i<problems.length; ++i)
			add(problems[i]);
	}
	
	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#size()
	 */
	public int size() {
		return problems.size();
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#iterator()
	 */
	public Iterator iterator() {
		return problems.values().iterator();
	}
	
	/* (non-Javadoc)
	 * @see java.util.Set#add(java.lang.Object)
	 */
	public boolean add(Object arg0) {
		return add( (Problem) arg0 );
	}
	
	/** Adds a problem to the problemset.
	 * @param p The problem to add.
	 * @return Returns true if the problem was added.
	 */
	public boolean add(Problem p)
	{
		return problems.put(p.getID(), p) != p;
	}

	/** Get the problem with id id.
	 * @param id A problem id.
	 * @return Returns the problem whose id is id. 
	 */
	public Problem getProblem(String id)
	{
		return (Problem) problems.get(id);
	}
}
