/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on Dec 5, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.tudarmstadt.yajs.lib;

import java.util.HashSet;
import java.util.Set;


/** The server information. This reflects the initial information sent by the server (name, flags...)
 * @author markus
 */
public class ServerInfo {
	
	private static final String FLAG_CONTESTANTS = "contestants";
	private static final String FLAG_JUDGES = "judges";
	
	/** The server id. */
	private String id;
	/** The flags the server sent. A set of strings. */
	private Set flags;
	
	/** Constructor. Sets id and flags.
	 * @param i The id string.
	 * @param f The flags.
	 */
	public ServerInfo(String i, Set f)
	{
		id = new String(i);
		flags = new HashSet(f);
	}
	
	/** Get the ID string. 
	 * @return Returns the id.
	 */
	public final String getID() { return id; }
	
	/** Test whether the server accepts contestants. 
	 * Looks up <code>FLAG_CONTESTANTS</code> in the set of flags.
	 * @return True if contestants are accepted.
	 */
	public boolean acceptsContestants() { return flags.contains(FLAG_CONTESTANTS); }
	/** Test whether the server accepts judges. 
	 * Looks up <code>FLAG_JUDGES</code> in the set of flags.
	 * @return True if judges are accepted.
	 */
	public boolean acceptsJudges() { return flags.contains(FLAG_JUDGES); }
}
