/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib;

/**
 * A submission.  Basically encapsulates a Problem, Language and a string storing the solution attempt.
 */
public class Submission
{
	private Problem problem;
	private Language language;
	private String solution;
	
	/** Constructor. Creates the triple from the given data.
	 * @param p The problem of this submission.
	 * @param l The language of this submission.
	 * @param s The string representing the code of the solution.
	 */
	public Submission(Problem p, Language l, String s)
	{
		problem = p;
		language = l;
		solution = s;
	}
	
	/**
	 * Get the problem.
	 * @return  Returns the problem this Submission encapsulates.
	 * @uml.property  name="problem"
	 */
	public Problem getProblem()
	{
		return problem;
	}

	/**
	 * Get the language.
	 * @return  Returns the language this Submission encapsulates.
	 * @uml.property  name="language"
	 */	
	public Language getLanguage()
	{
		return language;
	}
	
	/**
	 * Get the solution.
	 * @return  Returns the solution this Submission encapsulates.
	 * @uml.property  name="solution"
	 */
	public String getSolution()
	{
		return solution;
	}
}
