/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib.contestant;
import de.tudarmstadt.events.EventGenerator;
import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.lib.Client;
import de.tudarmstadt.yajs.lib.Console;
import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.Login;
import de.tudarmstadt.yajs.lib.Result;
import de.tudarmstadt.yajs.lib.Submission;
import de.tudarmstadt.yajs.lib.events.ResultEvent;
import de.tudarmstadt.yajs.lib.events.ResultEventGenerator;
import de.tudarmstadt.yajs.lib.events.ResultEventListener;
import de.tudarmstadt.yajs.lib.fsm.ContestantFSM;
import de.tudarmstadt.yajs.lib.fsm.actions.ContestantAction;
import java.io.IOException;


/**
 * The Contestant subclass of Client.
 */
public class Contestant extends Client
{
	private EventGenerator resultsignal;
	
	protected class ConcreteContestantHelper
		extends ConcreteClientHelper
		implements ContestantHelper
	{
		public void signalResultNotified(Result r) {
			resultsignal.emitEvent(new ResultEvent(Contestant.this, r ,true));
		}
		
		public void signalResultPolled(Result r) {
			resultsignal.emitEvent(new ResultEvent(Contestant.this, r ,false));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.Client.ConcreteClientHelper#getLoginFlags()
		 */
		protected String getLoginFlags() {
			return Lang.getString("Contestant.0"); //$NON-NLS-1$
		}	
	}
	
	public Contestant(ContestInfo contest, Login login, Console con) throws IOException
	{
		super(contest, login);
		this.resultsignal = new ResultEventGenerator();
		setFSM(new ContestantFSM(new ConcreteContestantHelper()));
	}
	public Contestant(ContestInfo contest, Console con) throws IOException
	{
		super(contest);
		this.resultsignal = new ResultEventGenerator();
		setFSM(new ContestantFSM(new ConcreteContestantHelper()));
	}

	public void queryResults() 
	{
		send(ContestantAction.QUERY_RESULTS + "\n"); //$NON-NLS-1$
	}

	public void submit(Submission s)
	{
		send(ContestantAction.SUBMIT + "\n" + s.getProblem().getID() + "\n" + s.getLanguage().getID() + "\n" + s.getSolution()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	public void addResultEventListener(ResultEventListener l)
	{
		resultsignal.registerListener(l);
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Client#checkServerInfo()
	 */
	protected void checkServerInfo() {
		if(!getServerInfo().acceptsContestants()) 
		{
			closeWithError(Lang.getString("Contestant.5")); //$NON-NLS-1$
		}
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Client#checkConnectionInfo()
	 */
	protected void checkConnectionInfo() {
		if(!getConnectionInfo().isContestantConnection())
		{
			closeWithError(Lang.getString("Contestant.6")); //$NON-NLS-1$
		}
	}
}
