/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 09.05.2005
 *
 */
package de.tudarmstadt.yajs.lib.events;

import java.util.HashSet;
import java.util.Iterator;

/**
 * @author markus
 *
 */
public class ClarificationLockEventGenerator {
	
	private HashSet listeners = new HashSet();
	
	/** Register a listener.
	 * @param listener
	 */
	public synchronized void registerListener(ClarificationLockEventListener listener)
	{
		listeners.add(listener);
	}
	/** Unregister a listener.
	 * @param listener
	 */
	public synchronized void unregisterListener(ClarificationLockEventListener listener)
	{
		listeners.remove(listener);
	}

	/** Return a copy of the listeners set so we can iterate over it.
	 * @return A copy of the set listeners.
	 */
	private synchronized HashSet getListenersCopy() { 
		return new HashSet(listeners);
	}

	/** Emit event "ClarificationRequest has been locked."
	 * @param event
	 */
	public void emitLocked(ClarificationLockEvent event)
	{		
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			ClarificationLockEventListener lis = (ClarificationLockEventListener) i.next();
			lis.clarificationLocked(event);
		}
	}

	/** Emit event "ClarificationRequest could not be locked."
	 * @param event
	 */
	public void emitUnlockable(ClarificationLockEvent event) {
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			ClarificationLockEventListener lis = (ClarificationLockEventListener) i.next();
			lis.clarificationUnlockable(event);
		}
	}

}
