/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib.events;

import java.util.HashSet;
import java.util.EventListener;
import java.util.EventObject;
import java.util.Iterator;

import de.tudarmstadt.events.EventGenerator;

public class LoginEventGenerator implements EventGenerator
{
	private HashSet listeners = new HashSet();

	public synchronized void registerListener(EventListener e)
	{
		LoginEventListener le = (LoginEventListener) e;
		listeners.add(le);
	}
	
	public synchronized void unregisterListener(EventListener e)
	{
		LoginEventListener le = (LoginEventListener) e;
		listeners.remove(le);
	}

	/** Return a copy of the listeners set so we can iterate over it.
	 * @return A copy of the set listeners.
	 */
	private synchronized HashSet getListenersCopy() { 
		return new HashSet(listeners);
	}
	
	public void emitEvent(EventObject e)
	{
		LoginEvent le = (LoginEvent) e;
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			LoginEventListener lis = (LoginEventListener) i.next();
			lis.loginCompleted(le);
		}
	}
}
