/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib.fsm;
import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.lib.ClientHelper;
import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.actions.Action;
import de.tudarmstadt.yajs.lib.fsm.actions.ClientAction;
import de.tudarmstadt.yajs.lib.fsm.actions.ClientActionAdapter;
import de.tudarmstadt.yajs.lib.fsm.actions.ErrorHandler;
import de.tudarmstadt.yajs.lib.fsm.actions.Heartbeat;
import de.tudarmstadt.yajs.lib.fsm.actions.HeartbeatReply;
import de.tudarmstadt.yajs.lib.fsm.actions.ReceiveClarificationReply;
import de.tudarmstadt.yajs.lib.fsm.actions.ReceiveServerInfo;
import de.tudarmstadt.yajs.lib.fsm.actions.UpdateScoreboard;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * @author  tim
 */
public abstract class ClientFSM
{
	/**
	 * @author markus
	 *
	 */
	protected abstract class ConcreteClientFSMHelper implements ClientFSMHelper {

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper#registerAction(java.lang.String, de.tudarmstadt.yajs.lib.fsm.actions.ClientAction)
		 */
		public void registerAction(String key, ClientAction a) {
			addAction(key, new ClientActionAdapter(a));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper#unregisterAction(java.lang.String)
		 */
		public void unregisterAction(String key) {
			removeAction(key);
		}
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper#initializeEarlyProtocol()
		 */
		public void initializeEarlyProtocol() {
			registerAction(ClientAction.HEARTBEAT_REPLY, ClientFSM.this.getHeartbeatSynchronizer());
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper#initializeProtocol()
		 */
		public void initializeProtocol() {
			registerAction(ClientAction.HEARTBEAT_REQUEST, new Heartbeat());
			registerAction(ClientAction.SCOREBOARD, new UpdateScoreboard());
			registerAction(ClientAction.CLARIFICATION_REPLY, new ReceiveClarificationReply());
		}


	}
	private Map map;
	private ClientHelper clienthelper;
	private HeartbeatReply heartbeatSynchronizer; 
	
	public void reset()
	{
		map.clear();
		getHelper().registerAction(Lang.getString("ClientFSM.0"), new ErrorHandler()); //$NON-NLS-1$
		getHelper().registerAction(Lang.getString("ClientFSM.1"), new ReceiveServerInfo()); //$NON-NLS-1$
	}
	
	/**
	 * @return  the heartbeatSynchronizer
	 * @uml.property  name="heartbeatSynchronizer"
	 */
	private HeartbeatReply getHeartbeatSynchronizer() {
		return heartbeatSynchronizer;
	}

	public ClientFSM(ClientHelper clienthelper) 
	{
		this.map = new HashMap();
		this.heartbeatSynchronizer = new HeartbeatReply();
		this.clienthelper = clienthelper;
	}
	
	protected abstract void performAction(Action action, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError;
	
	protected void addAction(String key, Action action)
	{
		map.put(key, action);
	}
	
	protected Action getAction(String key)
	{
		return (Action) map.get(key);
	}
	
	protected void removeAction(String key)
	{
		map.remove(key);
	}
	
	protected abstract ClientFSMHelper getHelper();
	
	public void delta(String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError
	{
		// System.err.println("Dispatching message " + msg);
		StringTokenizer st = new StringTokenizer(msg);
		String key = st.nextToken("\n"); //$NON-NLS-1$
		
		if(!map.containsKey(key)) return;
		Action action = (Action) map.get(key);
		performAction(action, msg);
	}

	/**
	 * @param a
	 */
	public void addSyncAction(Action a) {
		getHeartbeatSynchronizer().addSyncAction(a);
	}
}
