/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib.fsm;

import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.actions.Action;
import de.tudarmstadt.yajs.lib.fsm.actions.ClientAction;

public interface ClientFSMHelper
{
	public void registerAction(String key, ClientAction a);
	public void unregisterAction(String key);
	
	/** Set up login protocol.
	 * (The protocol that starts as soon as the "hello" message was seen). 
	 */
	public void initializeEarlyProtocol();
	
	/** Set up contestant/judge specific protocol.
	 */
	public void initializeProtocol();
	
	/** Perform another action.
	 * This has to be done here, as we do not have sufficient type information
	 * in Client/Contestant/JudgeAction.perform().
	 * The parameter a will be cast to the appropriate type, then the action will
	 * be performed with the given context. 
	 * @param a An action to perform.
	 * @param msg The context of the Action.
	 * @throws MalformedMessage
	 * @throws UnexpectedMessage
	 * @throws ConnectionError
	 */
	public void performOther(Action a, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError;

}