/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 10.03.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm;

import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.contestant.ContestantHelper;
import de.tudarmstadt.yajs.lib.fsm.actions.Action;
import de.tudarmstadt.yajs.lib.fsm.actions.ClientAction;
import de.tudarmstadt.yajs.lib.fsm.actions.ContestantAction;
import de.tudarmstadt.yajs.lib.fsm.actions.UpdateResults;

/**
 * @author  markus
 */

public class ContestantFSM extends ClientFSM {
	
	private ContestantHelper contestanthelper;
	private ClientFSMHelper fsmhelper;
	
	protected class ConcreteContestantFSMHelper 
		extends ConcreteClientFSMHelper 
		implements ContestantFSMHelper 
	{
		public void initializeProtocol() {
			super.initializeProtocol();
			registerAction(ContestantAction.RESULT, new UpdateResults());
		}
		public void registerAction(String key, ContestantAction a) {
			addAction(key, a);
		}
		public void performOther(Action a, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
			ContestantAction ca = (ContestantAction) a;
			ca.perform(ContestantFSM.this.contestanthelper, this, msg);
		}
	}
	
	/** Constructor for ContestantFSM.
	 * @param contestant The contestant helper.
	 */
	public ContestantFSM(ContestantHelper contestanthelper) {
		super(contestanthelper);
		this.contestanthelper = contestanthelper;
		this.fsmhelper = new ConcreteContestantFSMHelper();
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSM#performAction(de.tudarmstadt.yajs.lib.fsm.actions.Action, de.tudarmstadt.yajs.lib.fsm.FSMHelper, java.lang.String)
	 */
	protected void performAction(ClientAction action, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		((ContestantAction)action).perform(contestanthelper, fsm, msg);
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSM#getHelper()
	 */
	protected ClientFSMHelper getHelper() {
		return fsmhelper;
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSM#performAction(de.tudarmstadt.yajs.lib.fsm.actions.Action, java.lang.String)
	 */
	protected void performAction(Action action, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		((ContestantAction)action).perform(contestanthelper, fsmhelper, msg);
	}

}
