/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 10.03.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm;

import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.actions.Action;
import de.tudarmstadt.yajs.lib.fsm.actions.ClarificationLock;
import de.tudarmstadt.yajs.lib.fsm.actions.ClientAction;
import de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction;
import de.tudarmstadt.yajs.lib.fsm.actions.ReceiveClarificationAnswer;
import de.tudarmstadt.yajs.lib.fsm.actions.ReceiveClarificationRequest;
import de.tudarmstadt.yajs.lib.fsm.actions.ReceiveServerStatus;
import de.tudarmstadt.yajs.lib.fsm.actions.ReceiveSubmission;
import de.tudarmstadt.yajs.lib.fsm.actions.UpdateSubmissions;
import de.tudarmstadt.yajs.lib.judge.JudgeHelper;

/**
 * @author  markus
 */
public class JudgeFSM extends ClientFSM {

	private JudgeHelper judgehelper;
	private ClientFSMHelper fsmhelper;
	
	protected class ConcreteJudgeFSMHelper 
		extends ConcreteClientFSMHelper
		implements JudgeFSMHelper
	{
		public void registerAction(String key, JudgeAction a) {
			addAction(key, a);
		}
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper#initializeProtocol()
		 */
		public void initializeProtocol() {
			super.initializeProtocol();
			registerAction(JudgeAction.SUBMISSION_STATUS, new UpdateSubmissions());
			registerAction(JudgeAction.SUBMISSION, new ReceiveSubmission());
			registerAction(JudgeAction.CLARIFICATION_NOTIFY, new ReceiveClarificationRequest());
			registerAction(JudgeAction.CLARIFICATION_NOTIFY2, new ReceiveClarificationAnswer());
			registerAction(JudgeAction.CLARIFICATION_LOCKED, new ClarificationLock());
			registerAction(JudgeAction.SERVER_STATUS_REPLY, new ReceiveServerStatus());
		}

		public void performOther(Action a, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
			JudgeAction ja = (JudgeAction) a;
			ja.perform(JudgeFSM.this.judgehelper, this, msg);
		}

	}
	
	/**
	 * @param clienthelper
	 */
	public JudgeFSM(JudgeHelper judgehelper) {
		super(judgehelper);
		this.judgehelper = judgehelper;
		this.fsmhelper = new ConcreteJudgeFSMHelper();
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSM#performAction(de.tudarmstadt.yajs.lib.fsm.actions.Action, de.tudarmstadt.yajs.lib.fsm.FSMHelper, java.lang.String)
	 */
	protected void performAction(ClientAction action, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		((JudgeAction)action).perform(judgehelper, getHelper(), msg);
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSM#getHelper()
	 */
	protected ClientFSMHelper getHelper() {
		return fsmhelper;
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.ClientFSM#performAction(de.tudarmstadt.yajs.lib.fsm.actions.Action, java.lang.String)
	 */
	protected void performAction(Action action, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		((JudgeAction) action).perform(judgehelper, fsmhelper, msg);
	}
}
