/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 09.05.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;
import de.tudarmstadt.yajs.lib.judge.JudgeHelper;

/**
 * @author markus
 *
 */
public class ClarificationLock implements JudgeAction {
	
	protected class ClarificationLockedAndRefreshed  implements JudgeAction {

		private long nr;
		
		/** Create a ClarificationLockedAndRefreshed Action that will trigger a ClarificationLockedEvent.
		 * This is only done when the updated ClarificationRequest is retrieved (synchronization).
		 */
		public ClarificationLockedAndRefreshed(long nr) {
			this.nr = nr;
		}
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction#perform(de.tudarmstadt.yajs.lib.judge.JudgeHelper, de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper, java.lang.String)
		 */
		public void perform(JudgeHelper judge, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage {
			judge.signalClarificationLocked(nr);
		}
		
	}
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction#perform(de.tudarmstadt.yajs.lib.judge.JudgeHelper, de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper, java.lang.String)
	 */
	public void perform(JudgeHelper judge, ClientFSMHelper fsm, String msg)
			throws UnexpectedMessage, MalformedMessage {
		String[] split = msg.split("\n");
		if(split.length != 3) throw new MalformedMessage("Malformed clarification_locked message: " + msg + "\nExpected three lines.");
		long nr = Long.parseLong(split[1]);
		if(split[2].equals("failure")) judge.signalClarificationNotLockable(nr);
		else {
			// TODO Refactor?
			// send a guarded clarification_request
			judge.sendMessage(ClientAction.CLARIFICATION_LIST + "\n");
			judge.sendHeartbeatRequest(new ClarificationLockedAndRefreshed(nr));
		}
	}
}
