/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 14.03.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import de.tudarmstadt.yajs.lib.ClientHelper;
import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.contestant.ContestantHelper;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;
import de.tudarmstadt.yajs.lib.judge.JudgeHelper;

/**
 * An adapter to allow client actions to be handled by  Contestant and Judge FSMs. This more or less remodels inheritance, but works around the need for casts  in the inheritance approach.
 * @author  markus
 */
public class ClientActionAdapter implements ContestantAction, JudgeAction, ClientAction {

	private ClientAction action;
	
	/**
	 * 
	 * @param action
	 */
	public ClientActionAdapter(ClientAction action)
	{
		this.action = action;
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.ContestantAction#perform(de.tudarmstadt.yajs.lib.ContestantHelper, de.tudarmstadt.yajs.lib.fsm.FSMHelper, java.lang.String)
	 */
	public void perform(ContestantHelper contestant, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		action.perform(contestant, fsm, msg);
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction#perform(de.tudarmstadt.yajs.lib.JudgeHelper, de.tudarmstadt.yajs.lib.fsm.FSMHelper, java.lang.String)
	 */
	public void perform(JudgeHelper judge, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		action.perform(judge, fsm, msg);
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.Action#perform(de.tudarmstadt.yajs.lib.ClientHelper, de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper, java.lang.String)
	 */
	public void perform(ClientHelper client, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		action.perform(client, fsm, msg);
	}
}
