/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 30.04.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import de.tudarmstadt.yajs.lib.ClientHelper;
import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.ContestStatus;
import de.tudarmstadt.yajs.lib.ContestStatusEnumeration;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;

/**
 * @author markus
 *
 */
public class HeartbeatReply implements ClientAction {
	
	private List syncList =  Collections.synchronizedList(new LinkedList());
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.ClientAction#perform(de.tudarmstadt.yajs.lib.ClientHelper, de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper, java.lang.String)
	 */
	public void perform(ClientHelper client, ClientFSMHelper fsm, String msg)
			throws UnexpectedMessage, MalformedMessage, ConnectionError {
		/* TODO maybe think of a better solution...
		   this is ugly... would some decorator work?
		 */
		if(client.getConnectionInfo().serverSendsStatus())
		{
			String[] split = msg.split("\n");
			if(split.length != 4) throw new MalformedMessage("Malformed status/heartbeat reply. Expected 4 lines, got " + split.length + "\n");
			
			ContestStatusEnumeration status = null;
			if(split[1].equals("before")) 
				status = ContestStatusEnumeration.BEFORE;
			else if(split[1].equals("running"))
				status = ContestStatusEnumeration.RUNNING;
			else if(split[1].equals("paused"))
				status = ContestStatusEnumeration.PAUSED;
			else if(split[1].equals("after"))
				status = ContestStatusEnumeration.AFTER;
			else
				throw new MalformedMessage("Malformed status/heartbeat reply. Unknown contest status: " + split[1] + "\n\tExpected one of before, running, paused or after.");
			
			long elapsed = Long.parseLong(split[2]);
			long total = Long.parseLong(split[3]);
			
			ContestStatus cstatus = new ContestStatus(status, elapsed, total);
			client.updateContestStatus(cstatus);
		}
		performSynced(client, fsm, msg);
	}

	/**
	 * @throws UnexpectedMessage
	 * @throws MalformedMessage
	 * @throws ConnectionError
	 */
	private void performSynced(ClientHelper client, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage, ConnectionError {
		try {
			Action a = (Action) syncList.remove(0);
			if(a != null) fsm.performOther(a, msg);
		}
		catch(ArrayIndexOutOfBoundsException e) {
			throw new UnexpectedMessage("Unexpected heartbeat reply! Sent no request!");
		}
	}

	/** Adds an action for synchronization
	 * @param a The action to add.
	 */
	public void addSyncAction(Action a) {
		syncList.add(a);
	}
}
