/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 24.05.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.ConnectionError;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;
import de.tudarmstadt.yajs.lib.judge.JudgeHelper;

/**
 * @author markus
 *
 */
public class ReceiveClarificationAnswer implements JudgeAction {
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction#perform(de.tudarmstadt.yajs.lib.judge.JudgeHelper, de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper, java.lang.String)
	 */
	public void perform(JudgeHelper judge, ClientFSMHelper fsm, String msg)
			throws UnexpectedMessage, MalformedMessage, ConnectionError {
		BufferedReader reader = new BufferedReader( new StringReader(msg) );
		String code;
		long requestnr;
		String judgename;
		String stateString;
		long time;
		String notified;
		String lock;
		try {
			code = reader.readLine();
			requestnr = Long.parseLong(reader.readLine());
			judgename = reader.readLine();
			stateString = reader.readLine();
			String timeString = reader.readLine();
			time = (timeString.equals("") ? -1 : Long.parseLong(timeString));
			notified = reader.readLine();
			lock = reader.readLine();
		} catch (IOException e) {
			throw new MalformedMessage("IO Error while reading clarification answer: \n" + msg);
		} catch (NumberFormatException e) {
			throw new MalformedMessage("Malformed clarification number:\n" + msg);
		}
		
		ClarificationAnswerState state;
		if(stateString.equals("unanswered")) {
			state = ClarificationAnswerState.UNANSWERED;
		} else if(stateString.equals("questioner")) {
			state = ClarificationAnswerState.QUESTIONER;
		} else if(stateString.equals("everyone")) {
			state = ClarificationAnswerState.EVERYONE;
		} else {
			throw new MalformedMessage("Malformed clarification state: " + msg);
		}
		
		try {
			StringBuffer answer = new StringBuffer();
			char[] buf = new char[4096];
			int len = 0;
			while((len = reader.read(buf, 0, buf.length)) >= 0)
			{
				answer.append(buf, 0, len);
			}
			judge.signalClarificationAnswer(requestnr, judgename, state, time, (notified.equals("notifies")), lock, answer.toString());
		} catch (IOException e1) {
			throw new MalformedMessage("IO Error while reading clarification answer: \n" + msg);
		}
	}
}
