/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import java.util.HashSet;
import java.util.StringTokenizer;

import de.tudarmstadt.yajs.lib.ClientHelper;
import de.tudarmstadt.yajs.lib.ServerInfo;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;

public class ReceiveServerInfo implements ClientAction
{
	public ReceiveServerInfo() {}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Action#perform(de.tudarmstadt.yajs.lib.ClientHelper, de.tudarmstadt.yajs.lib.FSMHelper, java.lang.String)
	 */
	public void perform(ClientHelper client, ClientFSMHelper fsm, String s) {
		StringTokenizer st = new StringTokenizer(s, "\n");
		if(st.countTokens() != 4) return; // FIXME throw
		String code, id, info;
		code = st.nextToken();
		id = st.nextToken();
		info = st.nextToken();
		StringTokenizer ft = new StringTokenizer(st.nextToken());
		HashSet flags = new HashSet();
		while(ft.hasMoreTokens())
		{
			flags.add(ft.nextToken());
		}
		client.setServerInfo(new ServerInfo(id, flags));
		
		// Send a login request
		client.sendLogin();
		
		// Send a heartbeat so we know when the login process is finished
		// (As the server is not allowed to send anything _before_ a login_welcome)
		CompleteLogin onHeartbeat = new CompleteLogin(false);
//		CompleteLogin onHeartbeat = new CompleteLogin(true);
		
		client.sendHeartbeatRequest(new ClientActionAdapter(onHeartbeat));
		
		// System.err.println("sent login, awaiting welcome");
		
		// Another "hello" message is a protocol error
		fsm.registerAction(HELLO, new RejectUnexpected());
		
		// We expect a login_welcome
		fsm.registerAction(LOGIN_WELCOME, new ReceiveLoginWelcome(onHeartbeat));
		
		// Start the "early" protocol
		fsm.initializeEarlyProtocol();
	}
}
