/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on Dec 5, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import de.tudarmstadt.yajs.lib.ClientHelper;
import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.Result;
import de.tudarmstadt.yajs.lib.Status;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.contestant.ContestantHelper;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;

/**
 * @author markus
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class UpdateResults implements ContestantAction {

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Action#perform(de.tudarmstadt.yajs.lib.ClientHelper, de.tudarmstadt.yajs.lib.FSMHelper, java.lang.String)
	 */
	public void perform(ClientHelper client, ClientFSMHelper fsm, String s)
			throws UnexpectedMessage, MalformedMessage {
		perform( (ContestantHelper) client, fsm, s);
	}
	
	/** Overrridden method. 
	 * This action can only operate on contestants, so it accepts 
	 * ContestantHelper objects instead. 
	 * @param contestant The contestant helper.
	 * @param fsm The fsm implementing the protocol.
	 * @param msg The message that was received.
	 * @throws UnexpectedMessage
	 * @throws MalformedMessage
	 */
	public void perform(ContestantHelper contestant, ClientFSMHelper fsm, String msg)
			throws UnexpectedMessage, MalformedMessage {
		
		String[] split = msg.split("\n");
		/* funnily, String.split discards trailing empty strings */
		/* (Who the **** came up with that idea?) */
		if(split.length < 7 || split.length > 8) throw new MalformedMessage("Wrong number of tokens: " + split.length + " (should be 8)\n[" + msg + "]");
		
		String msgcode, sno, stime, prob, lang, notifies, sstatus, reason;
		msgcode = split[0];
		sno = split[1];
		stime = split[2];
		prob = split[3];
		lang = split[4];
		notifies = split[5];
		sstatus = split[6];
		reason = (split.length < 8) ? "" : split[7];
		
		long index;
		try
		{
			index = Long.parseLong(sno);
		}
		catch(NumberFormatException e)
		{
			throw new MalformedMessage("Malformed problem number in message:\n" + msg);
		}
		
		long time;
		try
		{
			time = Long.parseLong(stime);
		}
		catch(NumberFormatException e)
		{
			throw new MalformedMessage("Malformed time in message:\n" + msg);
		}
		
		Status status;
		if(sstatus.equals("new")) {
			status = Status.NEW;
		} else if(sstatus.equals("unjudged")) {
			status = Status.UNJUDGED;
		} else if(sstatus.equals("accepted")) {
			status = Status.ACCEPTED;
		} else if(sstatus.equals("rejected")) {
			status = Status.REJECTED;
		} else if(sstatus.equals("ignored")) {
			status = Status.IGNORED;
		} else throw new MalformedMessage("Malformed status in message:\n" + msg);
		
		boolean notified = notifies.equals("notifies");
		
		ContestInfo ci = contestant.getContestInfo();
		
		Result r = new Result(index, ci.getProblems().getProblem(prob), ci.getLanguages().getLanguage(lang), time, status, reason);
		
		if(notified)
			contestant.signalResultNotified(r);
		else contestant.signalResultPolled(r);
	}

}
