/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 14.03.2005
 *
 */
package de.tudarmstadt.yajs.lib.fsm.actions;

import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.JudgeResult;
import de.tudarmstadt.yajs.lib.MalformedMessage;
import de.tudarmstadt.yajs.lib.Status;
import de.tudarmstadt.yajs.lib.UnexpectedMessage;
import de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper;
import de.tudarmstadt.yajs.lib.judge.JudgeHelper;


/**
 * @author markus
 *
 */
public class UpdateSubmissions implements JudgeAction {

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction#perform(de.tudarmstadt.yajs.lib.JudgeHelper, de.tudarmstadt.yajs.lib.fsm.ClientFSMHelper, java.lang.String)
	 */
	public void perform(JudgeHelper judge, ClientFSMHelper fsm, String msg) throws UnexpectedMessage, MalformedMessage {
		
		String[] split = msg.split("\n");
		/* funnily, String.split discards trailing empty strings */
		/* (Who the **** came up with that idea?) */
		if(split.length < 9 || split.length > 11) throw new MalformedMessage("Wrong number of tokens: " + split.length + " (should be 8)\n[" + msg + "]");
		
		String msgcode, sno, who, stime, prob, lang, notifies, sjudge, sstatus, reason, lock;
		msgcode = split[0];
		sno = split[1];
		who = split[2];
		stime = split[3];
		prob = split[4];
		lang = split[5];
		notifies = split[6];
		sjudge = split[7];
		sstatus = split[8];
		reason = (split.length < 10) ? "" : split[9];
		lock = (split.length < 11) ? "" : split[10];
		
		long index;
		try
		{
			index = Long.parseLong(sno);
		}
		catch(NumberFormatException e)
		{
			throw new MalformedMessage("Malformed problem number in message:\n" + msg);
		}
		
		long time;
		try
		{
			time = Long.parseLong(stime);
		}
		catch(NumberFormatException e)
		{
			throw new MalformedMessage("Malformed time in message:\n" + msg);
		}
		
		Status status;
		if(sstatus.equals("new")) {
			status = Status.NEW;
		} else if(sstatus.equals("unjudged")) {
			status = Status.UNJUDGED;
		} else if(sstatus.equals("accepted")) {
			status = Status.ACCEPTED;
		} else if(sstatus.equals("rejected")) {
			status = Status.REJECTED;
		} else if(sstatus.equals("ignored")) {
			status = Status.IGNORED;
		} else throw new MalformedMessage("Malformed status in message:\n" + msg);
		
		boolean notified = notifies.equals("notifies");
		
		ContestInfo ci = judge.getContestInfo();
		JudgeResult jr = new JudgeResult(sjudge, who, index, ci.getProblems().getProblem(prob), ci.getLanguages().getLanguage(lang), time, status, reason, lock);
		
		if(notified)
			judge.signalJudgementNotified(jr);
		else judge.signalJudgementPolled(jr);
	}
}
