/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.tudarmstadt.yajs.lib.judge;
import de.tudarmstadt.events.EventGenerator;
import de.tudarmstadt.lang.Lang;
import de.tudarmstadt.yajs.lib.ClarificationAnswerState;
import de.tudarmstadt.yajs.lib.Client;
import de.tudarmstadt.yajs.lib.Console;
import de.tudarmstadt.yajs.lib.ContestInfo;
import de.tudarmstadt.yajs.lib.ContestStatusEnumeration;
import de.tudarmstadt.yajs.lib.JudgeClarificationAnswer;
import de.tudarmstadt.yajs.lib.JudgeResult;
import de.tudarmstadt.yajs.lib.Login;
import de.tudarmstadt.yajs.lib.PostedClarificationRequest;
import de.tudarmstadt.yajs.lib.Problem;
import de.tudarmstadt.yajs.lib.events.ClarificationLockEvent;
import de.tudarmstadt.yajs.lib.events.ClarificationLockEventGenerator;
import de.tudarmstadt.yajs.lib.events.ClarificationLockEventListener;
import de.tudarmstadt.yajs.lib.events.ClarificationRequestEvent;
import de.tudarmstadt.yajs.lib.events.ClarificationRequestEventGenerator;
import de.tudarmstadt.yajs.lib.events.ClarificationRequestEventListener;
import de.tudarmstadt.yajs.lib.events.JudgeClarificationAnswerEvent;
import de.tudarmstadt.yajs.lib.events.JudgeClarificationAnswerEventGenerator;
import de.tudarmstadt.yajs.lib.events.JudgeClarificationAnswerEventListener;
import de.tudarmstadt.yajs.lib.events.JudgementEvent;
import de.tudarmstadt.yajs.lib.events.JudgementEventGenerator;
import de.tudarmstadt.yajs.lib.events.JudgementEventListener;
import de.tudarmstadt.yajs.lib.events.ServerStatusEvent;
import de.tudarmstadt.yajs.lib.events.ServerStatusEventGenerator;
import de.tudarmstadt.yajs.lib.events.ServerStatusEventListener;
import de.tudarmstadt.yajs.lib.events.SubmissionLockEvent;
import de.tudarmstadt.yajs.lib.events.SubmissionLockEventGenerator;
import de.tudarmstadt.yajs.lib.events.SubmissionLockEventListener;
import de.tudarmstadt.yajs.lib.fsm.JudgeFSM;
import de.tudarmstadt.yajs.lib.fsm.actions.JudgeAction;
import java.io.IOException;

/**
 * @author  tim
 */
public class Judge extends Client
{
	private EventGenerator judgementSignal;
	private SubmissionLockEventGenerator sourceLockSignal;
	private ClarificationLockEventGenerator clarificationLockSignal;
	private ClarificationRequestEventGenerator clarificationRequestSignal;
	private JudgeClarificationAnswerEventGenerator judgeClarificationAnswerSignal;
	private ServerStatusEventGenerator serverStatusSignal = new ServerStatusEventGenerator();
	
	protected class ConcreteJudgeHelper
		extends ConcreteClientHelper
		implements JudgeHelper
	{

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.JudgeHelper#signalJudgementNotified(de.tudarmstadt.yajs.gui.judge.JudgeResult)
		 */
		public void signalJudgementNotified(JudgeResult jr) {
			judgementSignal.emitEvent(new JudgementEvent(Judge.this, jr ,true));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.JudgeHelper#signalJudgementPolled(de.tudarmstadt.yajs.gui.judge.JudgeResult)
		 */
		public void signalJudgementPolled(JudgeResult jr) {
			judgementSignal.emitEvent(new JudgementEvent(Judge.this, jr ,false));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#signalSubmissionLocked(int, java.lang.StringBuffer)
		 */
		public void signalSubmissionLocked(long submissionnr, StringBuffer source) {
			sourceLockSignal.emitLocked(new SubmissionLockEvent(Judge.this, submissionnr, source.toString()));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#signalSubmissionNotLockable(int)
		 */
		public void signalSubmissionNotLockable(long submissionnr) {
			sourceLockSignal.emitUnlockable(new SubmissionLockEvent(Judge.this, submissionnr, null));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#signalClarificationNotLockable(long)
		 */
		public void signalClarificationNotLockable(long nr) {
			clarificationLockSignal.emitUnlockable(new ClarificationLockEvent(Judge.this, nr));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#signalClarificationLocked(long)
		 */
		public void signalClarificationLocked(long nr) {
			clarificationLockSignal.emitLocked(new ClarificationLockEvent(Judge.this, nr));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#updateClarificationRequests(long, java.lang.String, de.tudarmstadt.yajs.lib.ClarificationRequest)
		 */
		public void signalClarificationRequest(long clarificationnr, String questioner, String problemid, String question, long time) {
			Problem p = getContestInfo().getProblems().getProblem(problemid);
			if(p == null) ;
			PostedClarificationRequest request = new PostedClarificationRequest(p, question, clarificationnr, time, questioner);
			clarificationRequestSignal.emitClarificationRequest(new ClarificationRequestEvent(Judge.this, request));
		}
		
		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.Client.ConcreteClientHelper#getLoginFlags()
		 */
		protected String getLoginFlags() {
			return Lang.getString("Judge.0"); //$NON-NLS-1$
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#signalClarificationAnswer(long, java.lang.String, de.tudarmstadt.yajs.lib.ClarificationAnswerState, long, boolean, java.lang.String, java.lang.String)
		 */
		public void signalClarificationAnswer(long requestnr, String judgename, ClarificationAnswerState state, long time, boolean b, String lock, String answer) {
			judgeClarificationAnswerSignal.emitJudgeClarificationAnswer(new JudgeClarificationAnswerEvent(Judge.this, b, new JudgeClarificationAnswer(requestnr, state, time, answer, judgename, lock)));
		}

		/* (non-Javadoc)
		 * @see de.tudarmstadt.yajs.lib.judge.JudgeHelper#signalServerStatusUpdate(java.lang.String)
		 */
		public void signalServerStatusUpdate(String status) {
			serverStatusSignal.emitServerStatusReceived(new ServerStatusEvent(Judge.this, status));
		}
		
	}
	
	/**
	 * @param contest
	 * @param login
	 * @param console
	 * @throws IOException
	 */
	public Judge(ContestInfo contest, Login login, Console console) throws IOException
	{
		super(contest, login);
		this.judgementSignal = new JudgementEventGenerator();
		this.sourceLockSignal = new SubmissionLockEventGenerator();
		this.clarificationLockSignal = new ClarificationLockEventGenerator();
		this.clarificationRequestSignal = new ClarificationRequestEventGenerator();
		this.judgeClarificationAnswerSignal = new JudgeClarificationAnswerEventGenerator();
		setFSM(new JudgeFSM(new ConcreteJudgeHelper()));
	}

	/**
	 * @param listener
	 */
	public void addJudgementEventListener(JudgementEventListener listener) {
		judgementSignal.registerListener(listener);
	}
	
	/**
	 * @param listener
	 */
	public void addSubmissionLockEventListener(SubmissionLockEventListener listener) {
		sourceLockSignal.registerListener(listener);
	}
	
	/**
	 * @param listener
	 */
	public void addClarificationLockEventListener(ClarificationLockEventListener listener) {
		clarificationLockSignal.registerListener(listener);
	}
	
	/**
	 * @param listener
	 */
	public void addClarificationRequestEventListener(ClarificationRequestEventListener listener) {
		clarificationRequestSignal.registerListener(listener);
	}
	
	/**
	 * @param listener
	 */
	public void addJudgeClarificationAnswerEventListener(JudgeClarificationAnswerEventListener listener) {
		judgeClarificationAnswerSignal.registerListener(listener);
	}
	
	/**
	 * @param listener
	 */
	public void addServerStatusEventListener(ServerStatusEventListener listener) {
		serverStatusSignal.registerListener(listener);
	}
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Client#checkServerInfo()
	 */
	protected void checkServerInfo() {
		if(!getServerInfo().acceptsJudges()) 
		{
			closeWithError(Lang.getString("Judge.1")); //$NON-NLS-1$
		}
	}

	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Client#checkConnectionInfo()
	 */
	protected void checkConnectionInfo() {
		if(!getConnectionInfo().isJudgeConnection())
		{
			closeWithError(Lang.getString("Judge.2")); //$NON-NLS-1$
		}
	}

	/**
	 * 
	 */
	public void querySubmissions() {
		send(JudgeAction.QUERY_SUBMISSION_STATUSES + "\n");		 //$NON-NLS-1$
	}

	/**
	 * @param submission
	 */
	public void fetchSubmission(long submission) {
		send(JudgeAction.FETCH_SUBMISSION + "\n" + submission + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/**
	 * @param clarification
	 */
	public void lockClarification(long clarification) {
		send(JudgeAction.LOCK_CLARIFICATION + "\n" + clarification + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * 
	 */
	public void switchState(ContestStatusEnumeration state) {
		String stateString = ""; //$NON-NLS-1$
		if(state == ContestStatusEnumeration.PAUSED) stateString =  Lang.getString("Judge.9"); //$NON-NLS-1$
		else if(state == ContestStatusEnumeration.RUNNING) stateString = Lang.getString("Judge.10"); //$NON-NLS-1$
		else if(state == ContestStatusEnumeration.AFTER) stateString = Lang.getString("Judge.11"); //$NON-NLS-1$
		else if(state == ContestStatusEnumeration.BEFORE) stateString = Lang.getString("Judge.12"); //$NON-NLS-1$
		send(JudgeAction.TOGGLE_PAUSE + "\n" + stateString + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param index
	 */
	public void cancelJudgement(JudgeResult result) {
		send(JudgeAction.JUDGE + "\n" + result.getIndex() + "\n\n\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param result
	 */
	public void sendJudgement(JudgeResult result) {
		send(JudgeAction.JUDGE + "\n" + result.getIndex() + "\n" + result.getStatus() + "\n" + result.getReason() + "\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}

	/**
	 * @param l
	 */
	public void cancelClarificationAnswer(long nr) {
		send(JudgeAction.ANSWER_CLARIFICATION + "\n" + nr + "\n\n");  //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param index
	 * @param problem
	 * @param content
	 */
	public void sendClarificationAnswer(long index, ClarificationAnswerState state, String content) {
		send(JudgeAction.ANSWER_CLARIFICATION + "\n" + index + "\n" + state + "\n" + content); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	/* (non-Javadoc)
	 * @see de.tudarmstadt.yajs.lib.Client#queryClarifications()
	 */
	public void queryClarifications() {
		// we want _all_ clarifications, we are judges!
		send(JudgeAction.CLARIFICATION_LIST + "\n"); //$NON-NLS-1$
	}

	/**
	 * 
	 */
	public void queryServerStatus() {
		send(JudgeAction.QUERY_SERVER_STATUS + "\n"); //$NON-NLS-1$
	}
}
