/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 24.06.2005
 *
 */
package de.tudarmstadt.yajs.tester;

import de.tudarmstadt.lang.Lang;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * A simple difference between two strings. Two files are compared line by line, if there is some difference, a small context around this line is stored.
 */
public class Diff {
	String output;
	String expected;
	boolean differs;
	
	/** Create a difference from two strings. */
	public Diff(String out, String exp, boolean differs) {
		this.output = out;
		this.expected = exp;
		this.differs = differs;
	}
	
	private final static int CONTEXT = 50; 
	
	private static String concatenateList(List l)
	{
		StringBuffer b = new StringBuffer();
		Iterator i = l.iterator();
		while(i.hasNext()) {
			String s = (String) i.next();
			b.append(s);
			b.append("\n"); //$NON-NLS-1$
		}
		return b.toString();
	}
	
	private static void readSomeMore(BufferedReader reader, List list) throws IOException {
		String line;
		int i = 0;
		while((line = reader.readLine()) != null && ++i < CONTEXT) {
			list.add(line);
		}
	}
	
	/** Create a difference from two readers. 
	 * @param is The first reader.
	 * @param expected The second reader.
	 * @return Returns a diff containing a context around the first difference found, or a description that no differences were found if there is none.
	 */
	public static Diff createFromReaders(BufferedReader is, BufferedReader expected) throws IOException {
		List historyl1 = new LinkedList();
		List historyl2 = new LinkedList();
		
		String l1 = null;
		String l2 = null;
		
		do {
			l1 = is.readLine();
			l2 = expected.readLine();
			
			historyl1.add(l1);
			if(historyl1.size() > CONTEXT)
				historyl1.remove(0);
			
			historyl2.add(l2);
			if(historyl2.size() > CONTEXT)
				historyl2.remove(0);
			
			if((l1 == null && l2 != null)
				|| (l1 != null && l2 == null)
				|| (l1 != null && l2 != null && !l1.equals(l2)))
			{
				readSomeMore(is, historyl1);
				readSomeMore(expected, historyl2);
				return new Diff(concatenateList(historyl1), concatenateList(historyl2), true);
			}
			
		} while(l1 != null && l2 != null);
		return new Diff(Lang.getString("Diff.1"), Lang.getString("Diff.2"), false); //$NON-NLS-1$ //$NON-NLS-2$
	}
	public static Diff createFromReaders(BufferedReader is) throws IOException {
		List historyl1 = new LinkedList();
	
		
		String l1 = null;
	
		
		do {
			l1 = is.readLine();
		
			
			historyl1.add(l1);
			if(historyl1.size() > CONTEXT)
				historyl1.remove(0);
			
		
			
			if(l1 != null )
			{
				readSomeMore(is, historyl1);
			
				return new Diff(concatenateList(historyl1), null, true);
			}
			
		} while(l1 != null);
		return new Diff(concatenateList(historyl1), null, true);
	}
	
	public boolean doesDiffer() { return differs; }
	
	public int getContextSize() { return CONTEXT * 2; }
	/**
	 * @return  the output
	 * @uml.property  name="output"
	 */
	public String getOutput() { return output; }
	/**
	 * @return  the expected
	 * @uml.property  name="expected"
	 */
	public String getExpected() { return expected; }
}
