/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 30.03.2005
 *
 */
package de.tudarmstadt.yajs.tester;

import java.io.File;
import java.util.EventObject;

/**
 * The test completed event.
 * @author  markus
 */
public class TestCompletedEvent extends EventObject {

	//private Diff diff;
	private File outputFile;
	private String error;
	private String suggestion;
	
	
	/** Constructor.
	 * Initializes fields and base class with given values.
	 * @param source The object emitting the event.
	 * @param outputFile The file to which the output was written.
	 * @param error The stderr output.
	 * @param suggestion The suggestion line in the output.
	 */
	public TestCompletedEvent(Object source, File outputFile, String error, String suggestion) {
		super(source);
		this.outputFile = outputFile;
		this.error = error;
		this.suggestion = suggestion;
	}
	
	/**
	 * Get the stderr output. 
	 * @return  Returns the stderr output.
	 * @uml.property  name="error"
	 */
	public String getError()
	{
		return error;
	}
	
	/**
	 * Get the suggested judgement.
	 * @return  Returns the suggestion that was returned by <code>native</code>.
	 * @uml.property  name="suggestion"
	 */
	public String getSuggestion()
	{
		return suggestion;
	}

	/**
	 * Get the file containing the output to stdout.
	 * @return  Returns the file with stdout output.
	 * @uml.property  name="outputFile"
	 */
	public File getOutputFile() {
		return outputFile;
	}

	
}
