/*
 * Copyright 2005 Markus Moll
 *
 * This file is part of Refefree.
 *
 * Refefree is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Refefree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/*
 * Created on 30.03.2005
 *
 */
package de.tudarmstadt.yajs.tester;

import java.util.HashSet;
import java.util.Iterator;

/** An event dispatcher for the test completed event.
 * @author markus
 *
 */
public class TestCompletedEventGenerator {
	private HashSet listeners = new HashSet();
	
	/** Register a listener.
	 * @param listener
	 */
	public synchronized void registerListener(TestCompletedEventListener listener)
	{
		listeners.add(listener);
	}
	/** Unregister a listener.
	 * @param listener
	 */
	public synchronized void unregisterListener(TestCompletedEventListener listener)
	{
		listeners.remove(listener);
	}

	/** Return a copy of the listeners set so we can iterate over it.
	 * @return A copy of the set listeners.
	 */
	private synchronized HashSet getListenersCopy() { 
		return new HashSet(listeners);
	}

	/** 
	 * @param event
	 */
	public void emitTestCompleted(TestCompletedEvent event)
	{		
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			TestCompletedEventListener lis = (TestCompletedEventListener) i.next();
			lis.testCompleted(event);
		}
	}
	/**
	 * 
	 */
	public void emitTestInterrupted(String error) {
		Iterator i = getListenersCopy().iterator();
		while(i.hasNext())
		{
			TestCompletedEventListener lis = (TestCompletedEventListener) i.next();
			lis.testInterrupted(error);
		}
	}
}
